 %% ==== BINARIZED IMAGE LINE DETECTION PROGRAM USING HOUGH TRANSFORM =======

tic
clear all ; close all  ; clc

set(groot, 'defaultAxesTickLabelInterpreter','latex'); 
set(groot, 'defaultLegendInterpreter','latex');
addpath(genpath(cd)) %genpath: generate path string p = genpath(folderName)
                     %addpath: adds a path string to the top of the search path
    

%% ============================= INPUT 0  =================================
fprintf('decide if it is necessary to LOAD previous results: \n')
% Loading previous results will not prevent the full calculation: go down
% to the plotting section if it is necessary to ONLY plot previous results.
LOAD_PREVIOUS_RESULTS = 1 ; 
%if =1 , results will be LOADED
if LOAD_PREVIOUS_RESULTS==0
  disp('Full calculation will be performed')  
else
  disp(' Previous data will be loaded and plotted')  
end

fprintf('decide if the results will be saved or not \n')
% DECIDE IF THE ANALYSIS WILL SAVE RESULTS OR NOT:
simulation_save_results = 0 ; 
%if =1 , results will be saved.
if simulation_save_results==0
  disp('Results will NOT be saved \n')  
else
  disp(' Results will be saved \n')  
end

toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  
%best mach equation coming from cfd simulation + pre processing of images through external software (Davis) for image alignment
%the x must be adimensionalized x=[0,1], and represents the start(zero) and
%end(1) of the acquisition domain.
estimated_machangle_start = 28; %[deg] - towards exit, at the start of the domain
estimated_machangle_middle = 38; %[deg]
estimated_machangle_end = 65; %[deg]  - towards throat, at the end of the domain
xx_f= [0 , 0.5 ,  1]';
yy_f = [estimated_machangle_start , estimated_machangle_middle, estimated_machangle_end ]';

ideal_mi = fit(xx_f, yy_f, 'poly2'); %this is the ideal estimation of the mi angle .



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ============================= INPUT 1: ================================ 
%read all the images and binarize them: input the folder in which the data
%is found:
  
a = dir(['PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection' '/*.tif']);
total = size(a,1);


%FolderIn='PR025NT002_Bobcat_Zoom_Run1_SS_Fabio_process_Align'; 
%FolderIn='PR025NT002_Bobcat_Zoom_Run1_SS_Seg_1_Align';
FolderIn='PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection'; 
 
%root name of the images
RootIn = 'B0';                         
%RootIn = 'Align0000';                     

%read calibration image:
% calibration_img= imread('calibration_PR025NT002_Bobcat_Zoom_Run1_SS_Fabio_process_Align.bmp'); %read the image used to calculate the scale factor and throat position
 calibration_img = imread('PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection/B00001.tif'); %read the image used to calculate the scale factor and throat position
%calibration_img = imread('calibration_PR025NT002_Bobcat_Zoom_Run1_SS_process_Align.tif'); 


%The dimensions of the calibration img are the dimensions wanted for the
%Schlieren imgs, in case the processed schlieren have different (larger) dimensions
m = length(calibration_img(1,:)); %columns
n = length(calibration_img(:,1)); %rows


%rotation adjustments to make the calibration match the data (IF necessary)
rotation_adjustment = -1.42; %[deg] %use zero at the first run. Later on, in calibration, ypu will have an estimate of the adjustment to make 
%to the calibration and the schlieren by looking at figure 4 and the command window.

image_rotate_cal = 0 + rotation_adjustment; %[degrees] Rotation of the calibration
image_rotate_data = 0 +rotation_adjustment ; %[deg] rotation of the schlieren data; positive rotation is counterclockwise rotation.
calibration_img = imrotate(calibration_img, image_rotate_cal); %rotate calibration img of x degrees IF it is not parallel to ground

calibration_img = calibration_img(1:end, 1:end); %when you rotate an image, 
%extra(black) pixels are added: you can remove them if you want by typing  ("end-number") 

%Choose the acquisition method:
MPI =1;      %1 means the calculation is done in series
             %2 means the calculation is done in parallel
             %3 means the calculation is done both in series and parallel (for comparison)

                  
%%%%%%%%%%%%%%%%%%%%% DATA SEQUENCE Allocation %%%%%%%%%%%%%%%%%%%%%%%%%%%%
Nimg = 250:1:355; %vector that represents all the images to acquire


%%%%%%%%%%%%%%%%% Hough Transform Binarisation Parameters %%%%%%%%%%%%%%%%
threshold = [0.01, 0.088]; % [low,high]: increase low to avoid detecting small gradients;
%decrease high value to detect smaller gradients.

gauss_noise_filter = 2.5; % gaussian sigma filter: 
%Increase to reduce noise: if the image is very sharp i recommend a filter >=2
%this filter makes lines rarer .

%******************* Apply the filters above to the raw data
%Modify the acquisition cycle depending on the type of image you have

%%%%%%%%%%%%%%%%% Adjustment Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
brightness_data = 70; % [pixels] Adapt the brightness of the images
% threshold_imgad = [0  0.75]; %[low, high]: adjust brightness: increase higher threshold to get white out of light grey area
threshold_imgad = NaN; %Use NaN if to use the automatic adjustment. Recommended.

%-----------------series implementation--------------------
if MPI == 1 || MPI == 3
    tic
    for x = Nimg
        %This for loop takes the raw data: rotates it, increases the brightness, adjusts the intensity of the image to
        %make the lines more visible, easily detectable. Then it takes the image
        %and binarises (1,0)s such that edges can be detected.
        
        fprintf('Acquiring image %s %d\n',RootIn,x)
        %StringIn = sprintf('%s/%s%04d.bmp',FolderIn,RootIn,x); Uncomment this line if you have bmps
        StringIn = sprintf('%s/%s%04d.tif',FolderIn,RootIn,x);  %Uncomment this line if you have tifs
        
    
        Raw_img = imread(StringIn);
        Raw_img = imrotate(Raw_img, image_rotate_data); %rotate calibration img of x degrees IF it is not parallel to ground
%for bmps: uncomment these 2 lines.
        %grayscale = rgb2gray(Raw_img(1:n,1:m,:)); %if the image is rgb(bmpfiles), convert to greyscale. This depends on the image.     
        %grayscale= grayscale+ brightness_data; %for bmps

        grayscale = Raw_img + brightness_data; % add brightness if the image is too dark

        if isnan(threshold_imgad)
            I(:,:,x) = imadjust(grayscale);
        else
            I(:,:,x) = imadjust(grayscale, threshold_imgad); % [low, high]: adjust brightness: increase lower threshold to get white out of light grey area
        end
        %adjust this value to achieve better detected lines on the midplane of
        %the image. Always use the automatic to SEE how it behaves. 
        %If the higher threshold is too low then you will saturate some parts of the image

      %Binarized images:        
        edge_BW(:,:,x) = edge( I(:,:,x), 'Canny', threshold, gauss_noise_filter );
        %edge_BW(:,:,x) = imcomplement(edge_BW(:,:,x)); %black line over white background: just for reference or better picture       
    end
    toc
end


% figure(1000)
%This is only useful when you understand the two thresholds in imadjust. 
% imshowpair(grayscale,I(:,:,Nimg(end)), 'montage')

% figure(2000)
% imshow(edge_BW)

%-----------------parallel implementation---------------------- %
tic
%preallocation 
% edge_BW = zeros(n,m,Nimg);
% I = zeros(n,m,Nimg);
if MPI == 2 || MPI == 3
    tic
    parfor t = Nimg(1): Nimg(end)
        
        %[edge_BW_2, I_2, grayscale_2] = edge_BWparbmp(t, brightness_data, threshold, gauss_noise_filter,FolderIn,RootIn ,threshold_imgad,image_rotate_data,n,m);
         [edge_BW_2, I_2] = edge_BWpar(t, brightness_data, threshold, gauss_noise_filter,FolderIn,RootIn ,threshold_imgad,image_rotate_data);
        % disp(V);
        % figure(3000)
        % imshow(V(:,:,t))
        % edge_BW_2(:,:,t) = V(1);
        % edge_BW(:,:,t) = V(:,:,t);
        edge_BW(:,:,t) = edge_BW_2(:,:,t);
        I(:,:,t) = I_2(:,:,t);
%         grayscale(:,:,t) = grayscale_2;
    end
    toc
end

% result_1 = isequal(I, I_3);
% result_2 = isequal(edge_BW, edge_BW_3);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ============================= INPUT 2: Working Space Parameters ================================ 

%N and M are equal to n and m if nothing is modified.  
%Here it is possible to modify the size of the schlieren images only

N = length(edge_BW(:,1,1)); % number of rows of the binarized schlieren img
M = length(edge_BW(1,1:end,1)); % number of columns of the binarized schlieren img

offset = 16; % [pixels] increment offset to move up, decrement to move down in the image 

%Check if the image is centered. 
Y_0 = round(N/2-offset); 

%indentify the position at which the method finishes extracting lines, in percentage : 60% or 65% on the pr25 zoomed in is good
C_end = 0.75; % tweak this value depending on your analysis of fig 1. See where to stop.

M_finish= floor(C_end*M); %(pixels) number of pixels at which the program stops acquiring lines

%Plot original first img with throat and axis, and the edge binarization version with midplane Y_0
figure(1)
imshowpair(edge_BW(:,:,Nimg(1)),I(:,:,Nimg(1)) , 'montage')
hold on
plot([1, M],[Y_0, Y_0])
plot([M_finish M_finish], [1 N-1])
legend('Nozzle axis/midplane', 'end of line extraction')
hold off


%if the offset is not zero (5 pixel threshold), recenter all the images in the dataset:
%NB: re-acquire images when modifying the offset! (Re-run input 1 section)
if abs(offset)>60
    for x=Nimg
        edge_BW(:,:,x) = imtranslate(edge_BW(:,:,x), [0, floor(offset/2)]);
    end
 Y_0 = Y_0 + round(offset/2);
 
    figure(11)
    imshow(edge_BW(:,:,Nimg(1)))
    hold on
    plot([1, M],[Y_0, Y_0])
    plot([M_finish M_finish], [1 N-1])
    legend('Nozzle axis/midplane', 'end of line extraction')
    hold off
end


%% ========= CALIBRATION  PT1: get the scale factor pixel per mm and throat position ============


USE_THROAT = 1; %Flag=1 means that the throat height and location is going to be used to get the 
%scale factor and to initialize the new left to right domain with the zero on the throat.

if USE_THROAT ==0
% another point will be used: the exit is used:  if the exit position
% is known there are no extra pixels before that in the image, then the
% image has been cut and  IS_IMAGE_CUT must be =1. 
%IF the exit (known point) of the nozzle is shifted in the image and there are black pixels before the exit, use
% IS_IMAGE_CUT=0 since the image has not been "cut" to match the exit.
fprintf('define if the image has been cut to have the ext of the nozzle right at the start of the image(column 1) \n')
IS_IMAGE_CUT=1; 
% this means thaton the left there is no black, and the image starts at the exact exit
end

if USE_THROAT==1
    disp('Selected the calibration method that uses the throat. Input required is throat mm ')
    throat_mm = 6.66 ; % nozzle throat dimension  in mm
    U_throatmm = 0.5*0.1; % [mm]
end

if USE_THROAT==0
 disp('Calibration requires a known height of the nozzle (exit preferably) and a distance between the considered point and the throat (usually length of the divergin nozle: exit-throat)')  

known_height_mm = 20.67; %in millimeters
U_known_mm = 0.5*0.1; % [mm] THIS depends on the caliper resolution
known_height_location = 0.25; %relative percentage of the image where the exit or known location is
pixel_adjust = 0; % manually adjust the location to make it as accurate as possible. 

known_X_location = known_height_location*m + pixel_adjust; % in pixels

known_axial_distance_mm = 58.72; %distance in mm  between A known point (exit usually) and throat.

figure(11)
imshow(calibration_img)
hold on
plot([1, M],[Y_0, Y_0])
plot([known_X_location known_X_location], [1 n])
legend('Nozzle axis/midplane', 'known Height')
hold off

end %end of the usethroat=0;


Calibr_start_domain = 0.03 ; %percentage of the calibration image: start acquiring walls of the calibration img. he has to be less than or equal to c_x_location
Calibr_end_domain = 0.95;%percentage of the calibration image: end acquiring walls of the calibration img

adim_general_location = [Calibr_start_domain Calibr_end_domain] ; % the interval of interest to get get the curves of the wall and then the throat: get it wide but do not acquire if the walls are cropped/dark/too blury

threshold_imbinarize = 0.1; % lower threshold to get less details in B&W image. IF the image is all white you have to increase this. if is too dark decrease it.
calibration_img_BW = imbinarize(calibration_img, threshold_imbinarize); % binarization of the image with Imbinarize

m_start = round(adim_general_location(1)*m); %pixel number to start acquiring walls in the calibration
m_end = round(adim_general_location(2)*m); %pixel number to end acquiring walls in the calibration Img

%polinomial best fit to find the walls. Depending on the interval selected
%the degree of the polynomial can be 1 to 5.
poly_order_walls = 5;


% This procedure below is just the extraction of vectors x and y that
% represent the points of the wall where the binarized threshold is met so
% the value of the image pixel shifts from 0 to 1.

% Upper curve
i=1;
column=m_start;
while column < m_end
    
    flag=0;
    row=1;
    while row<Y_0 && flag==0
        if calibration_img_BW(row,column)==1
            x_fit_u(i)=column;
            y_fit_u(i)=row;
            i=i+1;
            flag=1;
        else
            row=row+1;
        end
    end
  column=column+1;
end
    
% Lower curve
i=1;
column=m_start;
while column < m_end
    
    flag=0;
    row=n;
    while row>Y_0 && flag==0
        if calibration_img_BW(row,column)==1
            x_fit_l(i)=column;
            y_fit_l(i)=row;
            i=i+1;
            flag=1;
        else
            row=row-1;
        end
    end
  column = column + 1;
end

[P1, S1] = polyfit(x_fit_u, y_fit_u, poly_order_walls);
[P2, S2] = polyfit(x_fit_l, y_fit_l, poly_order_walls);

x_pixeldomain = m_start:1:m_end; %domain in pixels for the calibration img only in the selected range.

[upper_wall , delta_upper] = polyval(P1, x_pixeldomain, S1); %delta is the standard dev: 68% conf lvl
[lower_wall , delta_lower] = polyval(P2, x_pixeldomain, S2);

%clear one use variables
x_fit_u = 0;
y_fit_u = 0;
x_fit_l = 0;
y_fit_l = 0;

if USE_THROAT == 1 %RECOMMENDED CHOICE
    %best choice: assuming that the nozzle walls are aligned one another: (maximum error <=1pixel)
    [throat_pixels, index_throat] = min(lower_wall - upper_wall);
    throat_pixels = floor(throat_pixels);
    
%scale factor calculation:  pixel per mm
    scale_factor = throat_pixels/throat_mm; % pixels per mm
%knowing the x pixel domain, we extracted also the index at which the throat is located so the troat location in pixels is known.
    M_throatposition = x_pixeldomain(index_throat);
    
%evaluating the uncertainty in the throat location
    [value1,idx1] = min(lower_wall);
    [value2,idx2] = max(upper_wall);
    U_th_loc = 0.5 + round(abs(x_pixeldomain(idx1) - x_pixeldomain(idx2))); %[pixels]
    U_throatheight = sqrt( (2*floor(delta_lower(index_throat)))^2 + (2*delta_upper(index_throat))^2 ); %[pixels, 95%confLvl]
    U_scalefactor= sqrt( (U_throatheight/throat_pixels)^2 + (U_throatmm/throat_mm)^2)*scale_factor; %[pixel per mm]
    
%check the throat height from the above (this is calculated from the order of the polynomial and the calcibration image)
    height_throat_cal = throat_pixels/scale_factor;

end %end of usethroat==1

if USE_THROAT == 0 && Calibr_start_domain<=known_height_location
disp(' NB: Calibr start domain must be less than known height location. If you do no want this constraint, repeat the whole cycle pixel by pixel to detect the local distance, but no error estimation is available: NOT RECOMMENDED')

    [residual, index_known]= min(abs(known_X_location-x_pixeldomain));
    height_pixels = lower_wall(index_known)-upper_wall(index_known);
    
    U_knownheight = sqrt( (2*floor(delta_lower(index_known)))^2 + (2*delta_upper(index_known))^2 ); %[pixels, 95%confLvl]
    
    scale_factor= height_pixels/known_height_mm; %pixels per mm
    U_scalefactor= sqrt( (U_knownheight/height_pixels)^2 + (U_known_mm/known_height_mm)^2)*scale_factor; %[pixel per mm]
        
    
    %now the throat must be found:
    if IS_IMAGE_CUT==0
%identify the left border with a pixel threshold border acquisition on the midplane only if the nozzle
%starts later in the image:
        halfpixels_border_average = 10; %[pixels]
        row= Y_0 - halfpixels_border_average;
        i=1; 
        while row <= Y_0+halfpixels_border_average
            c=1;
            border_found=0;
            while border_found==0
                if calibration_img_BW(row,c)==1
                    x_border(i)=c;
                    border_found=1;
                    i=i+1;
                else
                    c=c+1;
                end
            end
            row=row+1;
        end
        X_border_pixels = mean(x_border); %position in pixels where the nozzle starts in the Schlieren  (exit!)
        
        M_throatposition = X_border_pixels + scale_factor*known_axial_distance_mm;
        
    else %if the image is not cut:
             M_throatposition = scale_factor*known_axial_distance_mm;
    end
end %end of usethroat==0
    


calibrY0_1 = mean((lower_wall(1:15)+upper_wall(1:15))/2) ; %left
calibrY0_2 = mean((lower_wall(end-50:end)+upper_wall(end-50:end))/2) ; %right
rotation_calibration_midplane= atan((calibrY0_2 - calibrY0_1)/(x_pixeldomain(end)-x_pixeldomain(1)))*180/pi;
disp('degrees for the calibration image adjustment rotation [deg] \n')
rotation_calibration_midplane

calibrY_0 = mean([calibrY0_1,calibrY0_2 ]);

figure(2)
imshowpair(calibration_img,calibration_img_BW, 'montage' )
hold on
plot([1, m],[Y_0, Y_0], '--')
plot([1, m],[calibrY_0, calibrY_0])
plot([adim_general_location(1)*m , adim_general_location(1)*m], [1 n-1])
plot([adim_general_location(2)*m , adim_general_location(2)*m], [1 n-1])
hold off

figure(21)
imshow(calibration_img )
hold on
plot([1, m],[calibrY_0, calibrY_0])
plot([adim_general_location(1)*m , adim_general_location(1)*m], [1 n-1])
plot([adim_general_location(2)*m , adim_general_location(2)*m], [1 n-1])
plot( x_pixeldomain, floor(upper_wall), 'linewidth', 2)
plot( x_pixeldomain, floor(lower_wall), 'linewidth', 2)
% plot( [x_pixeldomain(index_upper), x_pixeldomain(index_lower)],[1 n-1])
plot( [M_throatposition, M_throatposition],[1 n-1], '--')
hold off


if USE_THROAT==1
disp('scale factor pixel/mm | 95%Unc scalefactor  |  Uncertainty in the throat location in pixels')
[scale_factor, U_scalefactor , U_th_loc] 
else
 disp('scale factor pixel/mm | 95%Unc scalefactor | throat locaiton ')
[scale_factor, U_scalefactor , M_throatposition]       
end




%% ========= CALIBRATION PT2: image alignment using extrapolated throat position ===========: 

%throat identification: knowing the walls (upper and lower) it is possible to superimpose the 
%curves obtained from the calibration img to the actual image (1st frame as a reference)
%to derive the throat location3

% --------------------schlieren  frame acquisition from the images I(. . x) ---------------------------

threshold_schlieren=0.04; % lower threshold to get less details in B&W image. if the img is too white, increase threshold.

Schlieren_1frame_BW= imbinarize(I(:,:,Nimg(1)), threshold_schlieren); % binarization of the image with Imbinarize
%Schlieren_1frame_BW = imtranslate( imbinarize(I(:,:,Nimg(1)), threshold_schlieren) , [0 floor(offset/2)]); % binarization of the image with Imbinarize


C_schlieren_end_domain=0.88;

%polinomial best fit to find the walls. Depending on the interval selected
%the degree of the polynomial can be 1 to 5.
poly_order_walls_schlieren = 5;

adim_walldomain = [Calibr_start_domain , C_schlieren_end_domain] ; % the interval of interest to get get the curves of the walls in the schlieren: the starting point is the same as the calbration, but the end IS LESS THAN THAT: depends on brightness of the img


figure(3)
imshow(Schlieren_1frame_BW)
hold on
plot([1, m],[Y_0, Y_0])
plot([adim_walldomain(1)*m , adim_walldomain(1)*m], [1 n-1])
plot([adim_walldomain(2)*m , adim_walldomain(2)*m], [1 n-1])
hold off

m_start= round(adim_walldomain(1)*m); %pixel number to start acquiring wall
m_end= round(adim_walldomain(2)*m); %pixel numbet to end acquiring walls 

%The same procedure used for the calibration walls is used below
% Upper curve
i=1;
column=m_start;
while column < m_end
    flag=0;
    row=1;
    while row<Y_0 && flag==0
        if Schlieren_1frame_BW(row,column)==1
            x_fit_u(i)=column;
            y_fit_u(i)=row;
            i=i+1;
            flag=1;
        else
            row=row+1;
        end
    end
  column=column+1;
end  
% Lower curve
i=1;
column=m_start;
while column < m_end
    flag=0;
    row=n;
    while row>Y_0 && flag==0
        if Schlieren_1frame_BW(row,column)==1
            x_fit_l(i)=column;
            y_fit_l(i)=row;
            i=i+1;
            flag=1;
        else
            row=row-1;
        end
    end
  column=column+1;
end

%now also the walls of the schlieren images are known:
[SchlierenW1, Ssw1]= polyfit(x_fit_u, y_fit_u, poly_order_walls_schlieren);
[SchlierenW2, Ssw2]= polyfit(x_fit_l, y_fit_l, poly_order_walls_schlieren);

Walls_x_pixeldomain= round(adim_walldomain(1)*m):1:round(adim_walldomain(2)*m); %pixel domain for wall acquisition for the schlieren img

[Swall_u , Sdelta_u] = polyval(SchlierenW1, Walls_x_pixeldomain, Ssw1); 
[Swall_l , Sdelta_l] = polyval(SchlierenW2, Walls_x_pixeldomain, Ssw2); 
%SWall_l= Schlieren image extracted walls,  delta is the standard dev: 68% conf lvl

figure(4)
imshow(Schlieren_1frame_BW )
hold on
plot([1, m],[Y_0, Y_0], '--')
plot([adim_walldomain(1)*m , adim_walldomain(1)*m], [1 n-1])
plot([adim_walldomain(2)*m , adim_walldomain(2)*m], [1 n-1])
plot( Walls_x_pixeldomain, floor(Swall_u), 'linewidth', 2)
plot( Walls_x_pixeldomain, floor(Swall_l), 'linewidth', 2)
plot( x_pixeldomain, floor(upper_wall),'--', 'linewidth', 2)
plot( x_pixeldomain, floor(lower_wall),'--', 'linewidth', 2)
plot( [M_throatposition M_throatposition],[1 n-1])
hold off

%Now these things are known: SCALE FACTOR, throat position in mm --> it is possible to transform the
%image from pixels to the physical adimensional plane at the end of the program. 

%A new pixels domain called "Walls_x_pixeldomain"
% is useful to move along the walls in that domain and extract lines.


%% INPUT 3: Discretization ======================================================
tic

Initial_dx_offset=90; %[pixels]: use =0 to not skip any pixels in the line acquisition
%use if it is required to start acquiring lines n pixels further from the left side of the image

intervals = 30; % define number of intervals (only for the considered part of the nozzle)

window_start = 1;  %first interrogation window in which you want to SEE lines
window_finish = intervals; % last interrogation window in which you want to SEE lines

upp_max = 38; %maximum semi-height of the windows along the midplane, above the axis
low_max = 38; %maximum semi-height of the windows along the midplane, below the axis: 
% symmetric windows along the chosen axis is recommended:   upp_max == low_max

bw_step = 2; % base_window over the step: so the step is half the base window. Keep value=2.

dx_const = floor( bw_step*(M_finish-Initial_dx_offset)/intervals );

% implementation of the non constant step:
dx_min = 62; %  value based on uncertainty analysis. Useful for linear decrement of dx. NOT recommended to go below 30 pixels
dx_max = 2*dx_const - dx_min; % trapezoidal distribution (linear dx decrease...): calculated to cover the same distance as dx_const 

for int = 1:1:intervals
 dx(int)= floor( -((dx_max-dx_min)/intervals)*int + dx_max );
 
 NL(int) = Y_0 - floor(low_max*dx(int)/dx_max); % lower bound of the interrogation window on the y axis
 NU(int) = Y_0 + floor(upp_max*dx(int)/dx_max); % upper bound of the interrogation window on the y axis

end


Hough_theta_step = 0.2 ; %degrees discretization for the hough transform


estimated_min_mu = 30 ;         %[deg] current simulation minimum estimated angle (do NOT be conservative)

C0 = sqrt(2) ; % initial factor for max length lines to be identified

C_min= 0.5/(sqrt(2)*sin(estimated_min_mu*pi/180)); % final minimum factor for minimum line length detection

fill_gap_lines = 1; % initial number of  pixel skip in the line detection
max_pixel_skip = 3; % max number of  pixel skip in the line detection

n_peaks = 3;

toc
%% ====================== CALCULATION : MIDPLANE MU ANGLE AND MACH EXTRACTION  ============================================================
line_skip_count = zeros(1,intervals);
information_lost= zeros(1,intervals);

% ----------------serial implementation------------------------
tic
for x = Nimg 
% USE HOUGH TRANSFORM ON BINARY IMAGE MADE WITH EDGE
%for each frame, for each interval create the int_window and apply HOUGH transform 
   
accum = Initial_dx_offset; %accum is the variable that represents the step. ACCUM=0 if you want to acquire from first pixel.

  for i = 1:1:intervals
      
    int_window = edge_BW(NL(i):1:NU(i) , 1+accum:1:dx(i)+accum, x );
    local_cartesian_axis_origins(i, :) = [ round(1+accum) , NL(i) ]; %constant if the images have constant pixels/dimensions
    center_pixels(i,:)= [ round(dx(i)/2 + accum) , Y_0]; %constant if the images have constant pixels/dimensions
    
    accum = accum + floor((dx(i))/2) ;

    [H,THETA, RHO1] = hough( int_window , 'theta', -75:Hough_theta_step:75 );

    %FIND PEAKS OF THE TRANSFORM
    PEAKS = houghpeaks(H,n_peaks);
    
    min_length_lines(i) = C0*length(int_window(:,1)); %the min length it identifies in the transform, based on the Height of the window.
    WINDOWS{x,i}= int_window; 
    
%Cycle to maximise line length and quality of the extracted lines
    maxlinefound = 0;
    while maxlinefound == 0 % FIND LINES and SAVE RESULTS : repeat the line identification up until all max length lines are found 
         LINES{x,i}= houghlines( int_window, THETA, RHO1, PEAKS,'FillGap',fill_gap_lines,'MinLength',min_length_lines(i));
            
         % if the line IS found: exit the while and proceed
            if isempty(LINES{x,i})==0 
                maxlinefound=1;
            end
            % if the line is NOT found but i still can reduce length: decrease the min length to be detected.
            if isempty(LINES{x,i})==1 && min_length_lines(i)>floor( C_min*length(int_window(:,1)) ) 
               min_length_lines(i)= min_length_lines(i)-2;  
            end
            % if the line is NOT found but i have already short lines increase the pixel skip
            if isempty(LINES{x,i})==1 && min_length_lines(i)<=floor( C_min*length(int_window(:,1)) ) && fill_gap_lines<=max_pixel_skip 
                fill_gap_lines = fill_gap_lines+1;
                gap_skip(x,i)= fill_gap_lines;
            end
            if isempty(LINES{x,i})==1 && min_length_lines(i)<=floor( C_min*length(int_window(:,1)) ) && fill_gap_lines>max_pixel_skip
                information_lost(i)=information_lost(i)+1;
                maxlinefound=1; % if its impossible to find a line, just skip it... later it will use the previous one found
            end
            
    end
    
  end  % for all the intervals hough tansform for a single frame was applied
 
   
%now for each window of interest, extract data from lines.  
  for k = window_start:1:window_finish
    
      if isempty(LINES{x,k})==0 % if there ARE lines n that windo k:
          
         xy_local = [LINES{x,k}.point1 ;  LINES{x,k}.point2 ]; % (has all the lines)
         xy_m= length(xy_local(1,:)); % format: [x_line1pt1 , y_line1pt1 , x_line2pt1 yline2pt1 ; x_line1pt2 , y_line1pt2 , x_line2pt2 yline2pt2 ]
         % -> represets how many lines there are: if I have length 4 i have 2 lines.
           local_SR_matrix = zeros(2,xy_m);
           local_SR_matrix(1,1:2:xy_m) = local_cartesian_axis_origins(k,1) ;
           local_SR_matrix(1,2:2:xy_m) = local_cartesian_axis_origins(k,2);
           local_SR_matrix(2,1:2:xy_m) = local_cartesian_axis_origins(k,1) ;
           local_SR_matrix(2,2:2:xy_m) = local_cartesian_axis_origins(k,2);  
           xy = xy_local(:,:) + local_SR_matrix(:,:);  %struct where i save the first line detected in global cartesian coordinates
           CARTESIANLINES{x,k}.xy = xy;
           theta_angles= extractfield(LINES{x,k}, 'theta') ; % its a local temporary variable containing all angles of all the lines detected. coose the best one. choose the best one
           
           if x==Nimg(1) %for the first frame take the first angle as the best one
               th_opt=1; %choose the first angle detected.
               best_theta(x,k)= theta_angles(th_opt);
               LinesLength(x,k)= sqrt( (xy(2,1)-xy(1,1))^2 + (xy(2,2)-xy(1,2))^2 ) ;  % xy(point, coordinate)-> x_2-x_1= xy(point2,coordinate1)-xy(point1,coordinate1)
                CenterLines(x,k,1) = (xy(1,1)+xy(2,1))/2; %is the X of the center of the Line detected in the global SR
                CenterLines(x,k,2) = (xy(1,2)+xy(2,2))/2; %is the Y of the center of the Line detected in the global SR
           
            misalignment(x,k) = atan( (CenterLines(x,k,2)-calibrY_0)/(M_throatposition-CenterLines(x,k,1)))*180/pi ;% angle of misalignment of the velocity, in DEGREES: 
            %NB that is linear misalignment assuming a linear distribution of velocity (ULTRA CONSERVATIVE)
           end
           
           if  x>Nimg(1) %even if you only have one angle this works: choose the best angle.
               adim_position= center_pixels(k,1)/M_finish; 
               ideal_theta(k)= abs(90 - ideal_mi(adim_position) );    
                
                for m=1:1:length(theta_angles(:)) % CONSIDER USING THE THEORETICAL THETA if u have it. otherwise usse theta_avg.
                    variation_th(m) = abs( ideal_theta(k)- abs(theta_angles(m)) ); %always positive
                end
                
             [min_var , th_opt] = min(variation_th); %find the minimum variation and its index=th_opt
             best_theta(x,k)= theta_angles(th_opt); %save the best theta and carry on 
             mm= th_opt*2; % index to select the th_opt line from the xy matrix
             LinesLength(x,k)= sqrt( (xy(2,mm-1)-xy(1,mm-1))^2 + (xy(2,mm)-xy(1,mm))^2 ) ;  % xy(point, coordinate)-> x_2-x_1= xy(point2,coordinate1)-xy(point1,coordinate1);             
             CenterLines(x,k,1) = (xy(1,mm-1)+xy(2,mm-1))/2; %is the X of the center of the Line detected in the global SR
             CenterLines(x,k,2) = (xy(1,mm)+xy(2,mm))/2; %is the Y of the center of the Line detected in the global SR
             
             misalignment(x,k) = atan( (CenterLines(x,k,2)-calibrY_0)/(M_throatposition-CenterLines(x,k,1)) )*180/pi ;% angle of misalignment of the velocity, in DEGREES
             %NB that is linear misalignment assuming a linear distribution of velocity (EXTRA CONSERVATIVE)

             %saving the line corresponding to the best theta angle %th_op       
           end
                      
        theta_avg(k) = mean( abs(best_theta(: , k )) ); %the avg theta on the axial coordinate is used to determine the best angle
       
        variation_th=0; % re initializing the vector for next k-th calculation
      end %End of "if there Are lines".
      
      if isempty(LINES{x,k})==1 && x==Nimg(1) && k==1 % if there are NOT lines in that window and its the FIRST FRAME
        fprintf( 'DO NOT CONTINUE: FATAL ERROR: lines not found at 1st window. RESTART and change line acquisition parameters \n', k)
        best_theta(x,k)= NaN  ;
        LinesLength(x,k)= NaN;
      end
      if isempty(LINES{x,k})==1 && x==Nimg(1)  % if there are NOT lines in that window and its the FIRST FRAME
        fprintf( 'SERIOUS ERROR (first frame error): lines not found at window %d. Using k-1th window info. Best to restart and change line acquisition parameters \n', k)
        best_theta(x,k)= best_theta(x,k-1)  ;
        LinesLength(x,k)= LinesLength(x,k-1);
      end
      if isempty(LINES{x,k})==1 && x>Nimg(1) % if there are NOT lines in that window (x>1)
        fprintf( 'minor error: lines not found at window %d, at frame %d . Using line info of  previous frame. CHECK lineskipcount vector \n', k, x)
        best_theta(x,k)= best_theta(x-1,k)  ;
        LinesLength(x,k)= LinesLength(x-1,k);
        line_skip_count(k)=line_skip_count(k)+1;
      end
      
% %             % extraction of Mach lines and mach number
            if best_theta(x,k)<=0 && best_theta(x,k)>-70 % if the polar coordinate angle is negative, it means the line has a positive angle with respect to the x axis
                mi_angle(x,k) = best_theta(x,k) +90 ;
                Mach(x,k) = 1/sin(mi_angle(x,k)*pi/180);
            end
            if best_theta(x,k)>0 && best_theta(x,k)<70  % the +- 70 deg is to avoid detecting impossible mi angles(less than 20deg->M=3)
                mi_angle(x,k) = best_theta(x,k) -90 ;
                Mach(x,k) = 1/sin(- mi_angle(x,k)*pi/180); 
            end
            if best_theta(x,k)<=-70 || best_theta(x,k)>=70 
                if x>1
                best_theta(x,k) = best_theta(x-1,k);
                mi_angle(x,k) = 90- abs(best_theta(x,k));
                Mach(x,k) = 1/sin(mi_angle(x,k)*pi/180);
                else
                best_theta(x,k) = best_theta(x,k-1);
                mi_angle(x,k) = 90 - abs(best_theta(x,k));
                Mach(x,k) = 1/sin(mi_angle(x,k)*pi/180);  
                end
            end
    
  end  % for the single frame save results: Lines and angles.
     
end %finish cycle  on all frames
toc


%% COMMENTED WHEN RUNNING SUCCESSFULL SIMULATION: 

%% 1) checking data at a certain frame
%Display lines on Int_windows at a certain frame (For CHECKING step by step purposes)
% 
% x=300; % INPUT: frame at which to check the windows and lines extracted
% 
% for k = window_start:1:window_finish
%    figure(k+10) 
%    imshow(WINDOWS{x,k})
%    hold on
%    xy = [LINES{x,k}.point1; LINES{x,k}.point2];
%    plot(xy(:,1),xy(:,2),'LineWidth',2,'Color','green');
%    hold off
% end

%% 2) checking data in a location 
% 
% k= 3; %location
% frame_start=280;
% frame_end=340;
% stepp=5;
% 
% for x = frame_start:stepp:frame_end
%    figure(x) 
%    imshow(WINDOWS{x,k})
%    hold on
%    xy = [LINES{x,k}.point1; LINES{x,k}.point2];
%    plot(xy(:,1),xy(:,2),'LineWidth',2,'Color','green');
%    hold off
% end



%% ============ RESULTS ANALYSIS AND STATYSTICS  ===================================

%for each axial coordinate(int_window) calculate the statystical quantities of the various frames analyzed.
Number_of_samples= length(Nimg);


for k=1:1:intervals

%Filter nonsense mi angle results:
    
    [mi_angle_filtered(Nimg(1):Nimg(end),k), TF(Nimg(1):Nimg(end),k), L, U, C]=filloutliers(abs(mi_angle(Nimg(1):Nimg(end),k)), 'linear', 'percentiles', [10, 90] );
    mi_angle_filtered(1:Nimg(1)-1,k) = NaN;
    
    Mach_filtered(Nimg(1):Nimg(end),k) = filloutliers(Mach(Nimg(1):Nimg(end),k), 'linear', 'percentiles', [10, 90]);
    Mach_filtered(1:Nimg(1)-1,k) = NaN;
    
% Substitute with Nans the line length of the outliers:     
    indx_result(Nimg(1):Nimg(end)) = ( abs(mi_angle(Nimg(1):Nimg(end),k))==mi_angle_filtered(Nimg(1):Nimg(end),k)); %find indices where the outliers were replaced, replace that  with NaNs
    for xx=Nimg
    if indx_result(xx)==0
        LinesLength(xx,k)= NaN;
        misalignment(xx,k) = NaN;
    end
    end
    
%statystics on the Length of the lines detected
    Avg_LinesLength(k) = nanmean(LinesLength(Nimg(1):Nimg(end),k)); %calculate new average without outliers
    Unc_LL(k) = 2*nanstd(LinesLength(Nimg(1):Nimg(end),k)); %95% uncertainty on the length of the lines

%[degrees] uncertainty on the angular resolution with uniform distribution at 95% conf Lvl
    U_Angular_res(k) = (1/(1+Avg_LinesLength(k)^2))*Unc_LL(k)*180/pi;  
     
    Avg_angular_res(k) = atan( 1/Avg_LinesLength(k) )*180/pi; %[deg] average angular res based on the length of the identified lines, with uniform distribution   

% ANGULAR RESOLUTION 95 conf lvl at each location k:   
    Angular_res_95(k) = (Avg_angular_res(k) + U_Angular_res(k)); %[deg]  95%conf lvl.  (worst resolution= largest angle of resolution) - LINEAR DISTRIBUTION at 100% conf Lvl --> division by 0.95 goes from 95 to 100 percent if u want
  
%SUPER conservative estimate of misalignment [deg]: it is an indication of how far from the midplane the lines are.  
    Avg_misalignment(k) = nanmean(misalignment(:,k)); 

    
%evaluate type A uncertainty directly on the mach angles 
    mi_angle_avg(k)= nanmean(abs(mi_angle_filtered(:,k)));  %average mach line angle mi
    S_mi(k)= nanstd(abs(mi_angle_filtered(:,k)));           %std deviation of the mi angle for each station - 68% Conf Lvl

% evaluation of type B uncertainty on the lines    
    U_extraction(k) = sqrt( Angular_res_95(k)^2 + (0.95*Hough_theta_step/2)^2 ); %95%confLvl uncertainty in the line extraction

%evaluate type A uncertainty directly on the Mach number    
    Mach_avg(k) = nanmean(Mach_filtered(:,k));   %average Mach number M
    S_Mach(k) = nanstd(Mach_filtered(:,k)); %std deviation of the Mach number for each station (68%ConfLvl)

% Type B unc on the Mach coming from the extraction uncertainty on the mi angle
    U_Mach_ext(k) = (pi/180)*U_extraction(k)*( Mach_avg(k)*sqrt( Mach_avg(k)^2 -1 ) ) ; %Propagation of the angular extraction errors to the Mach results

% all the Uncerainties of the angle mi are in Degrees!
end

%NB: the extended uncertainty (95%confidence level) is 2 Times the total standard uncertainty


% check  for correlations
[R , P] = corrcoef(S_mi, Angular_res_95, 'rows', 'complete');
    if P(1,2)<=0.2
       RHO1 =  R(1,2);
    else
        RHO1=0;
    end
    
[R , P] = corrcoef(S_Mach, U_Mach_ext, 'rows','complete')
    if P(1,2)<=0.2
       RHO2 =  R(1,2);
    else
        RHO2=0;
    end
    
% Final total expanded uncertanties
for k=1:1:intervals

     U_mi(k)= sqrt( (2*S_mi(k))^2 + (U_extraction(k))^2 + 2*S_mi(k)*U_extraction(k)*RHO1 );  %total extended std uncertainty with 95%Conf Lvl

     U_Mach(k) =  sqrt( (2*S_Mach(k))^2 + (U_Mach_ext(k)^2) + 2*S_Mach(k)*U_Mach_ext(k)*RHO2 );  %  95%conf lvl
     
end
    
  
% Check the filtering of results:
figure(100)
k_vec= [ 20]; %k locations where to plot the time-results(frames)
for a=k_vec
    plot(Nimg(1):1:Nimg(end), abs(mi_angle(Nimg,a)), '--b')
    hold on
    plot(Nimg(1):1:Nimg(end), mi_angle_filtered(Nimg,a), 'r')
end
grid on
xlabel(' Sample Number')
ylabel(' Mi angle')
legend('raw Data' ,'Filtered data')
title(' filtered results')
hold off
% saveas(gcf, 'filtered results', 'bmp')

%% Using the local Checking method, if there are nonsense result still there due to the binarization here is possible to avoid considering them

% Window_to_remove = [36]; 
% 
% Mach_avg(Window_to_remove)=NaN;
% U_Mach(Window_to_remove) = NaN;
% 
% mi_angle_avg(Window_to_remove)=NaN;
% S_mi(Window_to_remove) = NaN;
% Angular_res_95(Window_to_remove)=NaN;
% U_extraction(Window_to_remove)=NaN;
% U_mi(Window_to_remove)=NaN;

%% RESULTS PT 1: Translating results from pixels to an adimensional system of reference.

locations = center_pixels( 1:intervals , 1 , 1 )' ;  %positions  in pixels from start to finishing, of the center pixels of the windows
inverted_locations= abs(locations(end:-1:1)-M_throatposition); %inverting the locations

%  ----------------- Physical domain [mm]
x_domain_mm =inverted_locations./scale_factor; %domain in mm
U_x_NRS_mm_frac = sqrt( (U_scalefactor/scale_factor)^2 + (U_th_loc/M_throatposition)^2 );
U_x_NRS_mm = U_x_NRS_mm_frac*x_domain_mm;
U_x_mm_tot = sqrt( (U_x_NRS_mm).^2+ ( floor(dx(end:-1:1)./2)/scale_factor).^2  ); %total uncertainty in the dimensional mm domain NRS.


% ------------------- Adimensionalized domain: (adim NRS with less uncertainty( no need to include the passage to mm !))
adim_locations = (M_throatposition-locations)/throat_pixels; % adim locations with the throat
%and moving the reference zero to the throat: lower values are closer to the throat.
% layout: [ adim_x_closer to exit  , ..... , adim_x_closer to throat ];
U_adim_frac = sqrt( (U_throatheight/throat_pixels)^2 + (U_th_loc/M_throatposition)^2 );
U_adim = U_adim_frac*adim_locations; %adim, from exit to throat orientation, the zero is the throat.
U_Xadim_tot = sqrt( (U_adim).^2+  (dx./(2*throat_pixels)).^2  ); %uncertainty on the adimensionalized locations 

%invert location for plotting purposes:
inv_adim_locations = adim_locations(end:-1:1);


% ------------------------- pixel domain: ((From throat to exit))
U_X_NRS_pixels = U_th_loc  ;       %The uncertainty due to the location of the throat only, since no adimensionalization or transformation to mm is applied.
U_Xtot = sqrt( U_X_NRS_pixels.^2 + floor(dx(end:-1:1)./2).^2 ); % total 100%confLvL uncertainty in pixels on the adimensionalized locations system of reference



%% SAVE VARIABLES
tic

if simulation_save_results == 1 

    fprintf('Saving variables... \n')
    save 'variablesPR26_midplane_lowerUnc.mat'
    fprintf('DONE: Saving variables... \n')

end


%this line saves the calibration data which will be used in the Wall line
%detection program.
%NECESSARY TO THE WALL DETECTION PROGRAM.
save ('calibration_dataPR26.mat', 'M_throatposition', 'scale_factor', 'throat_pixels','throat_mm', 'U_x_NRS_mm_frac' , ...
    'U_adim_frac' , 'x_pixeldomain' , 'upper_wall' , 'lower_wall' , 'delta_upper' , 'delta_lower')

toc


%% ---------------- Load VARIABLES ----------- Just to plot previously achieved results


if LOAD_PREVIOUS_RESULTS==1
tic

fprintf(' Loading variables... \n')

load 'variablesPR26_midplane_lowerUnc.mat'

fprintf('DONE: Loading variables... \n')

toc
end


%%  =======  =============== P L O T T I N G   S E C T I O N ================  ===================


%% Display results / Plots : the graphs are in the adimensionalized physical plane x/H_t where H_t=throat height in mm

%MI ANGLE  axial distribution 
figure(5)
plot(adim_locations(window_finish:-1:window_start) , mi_angle_avg(window_finish:-1:window_start) , '*')
hold on
for k=window_finish:-1:window_start
    plot([adim_locations(k) adim_locations(k)] , [mi_angle_avg(k)-U_mi(k) mi_angle_avg(k)+U_mi(k)], 'r')
    plot([adim_locations(k)-U_Xadim_tot(k) adim_locations(k)+U_Xadim_tot(k)] , [mi_angle_avg(k) mi_angle_avg(k)], 'c')
end
grid on
xlabel(' x/H_t with uncertainty bars')
ylabel(' Mi angle axial distribution with uncertainty bars')
legend('Experimental data results')
title(' experimental average mi angles and uncertainty bars')
hold off
%saveas(gcf, 'mi_axial', 'bmp')
%savefig('mi_adimloc.fig')

% % MACH axial distribution With uncertainties 
% figure(6)
% plot(adim_locations(window_finish:-1:window_start) , Mach_avg(window_finish:-1:window_start) , '*')
% hold on
% for k=window_finish:-1:window_start
%     plot([adim_locations(k) adim_locations(k)] , [Mach_avg(k)-U_Mach(k) Mach_avg(k)+U_Mach(k)], 'b')
%     plot([adim_locations(k)-U_Xadim_tot(k) adim_locations(k)+U_Xadim_tot(k)] , [Mach_avg(k) Mach_avg(k)], 'c')
% end
% grid on
% % xlabel('adimensionalized axis of the nozzle with the throat height x/H_t','Interpreter','latex')
% xlabel('x/H_t','Interpreter','latex')
% ylabel('M distribution','Interpreter','latex')
% % legend('Experimental data','Interpreter','latex')
% % title('Experimental average Mach and uncertainty bars')
% hold off
% % saveas(gcf, 'Mach_axial', 'bmp')
% set(gca,'FontSize',16)
% %print -depsc Mach_axial
% savefig('mach_adimloc.fig')

% Mach axial distribution With uncertainties (nicer version)
figure(600)
plot(adim_locations(window_finish:-1:window_start) , Mach_avg(window_finish:-1:window_start) , '*')
hold on
for k=window_finish:-1:window_start
    errorbar(adim_locations(k)  ,Mach_avg(k),U_Mach(k),'r-','HandleVisibility','off');
    hold on
    errorbar(adim_locations(k),Mach_avg(k),U_Xadim_tot(k),'b-','horizontal','HandleVisibility','off');
end
    errorbar(adim_locations(k)  ,Mach_avg(k),U_Mach(k),'r-');
    hold on
    errorbar(adim_locations(k),Mach_avg(k),U_Xadim_tot(k),'b-','horizontal');
grid on
xlabel('x/H_t','Interpreter','latex')
ylabel('M distribution','Interpreter','latex')
legend(' $M_{avg}$', '$U_{tot}$ M', '$U_{tot}$ Position')
set(gca,'FontSize',16)
%print -depsc Mach_axial
%savefig('Mach_axial_adimloc_nicer.fig')


% Mach axial distribution With uncertainties  DIMENSIONAL PLOT [mm]
figure(7)
x_domain_mm_flip = flip(x_domain_mm);
U_x_mm_tot_flip = flip(U_x_mm_tot);
plot(x_domain_mm_flip(window_finish:-1:window_start) , Mach_avg(window_finish:-1:window_start) , '*')
hold on
for k=window_finish:-1:window_start
    plot([x_domain_mm_flip(k) x_domain_mm_flip(k)] , [Mach_avg(k)-U_Mach(k) Mach_avg(k)+U_Mach(k)], 'b')
    plot([x_domain_mm_flip(k)-U_x_mm_tot_flip(k) x_domain_mm_flip(k)+U_x_mm_tot_flip(k)] , [Mach_avg(k) Mach_avg(k)], 'c')
end
grid on
xlabel('nozzle axis [mm]')
ylabel(' Mach axial distribution')
legend('Experimental data results')
title(' experimental average Mach and uncertainty bars')
hold off
%saveas(gcf, 'Mach_axial', 'bmp')
%savefig('mach_pixels.fig')


% Plot the uncertainty contributions (unc. breakdown) on the Mi angle 
%U_mi (total expanded uncertainty on the mi angle)
figure(8)
plot([inv_adim_locations(window_start) inv_adim_locations(window_finish)] , [Hough_theta_step/2 Hough_theta_step/2] , '--r')
hold on
plot(inv_adim_locations(window_start:1:window_finish) , Angular_res_95(window_finish:-1:window_start) , '--c')
plot(inv_adim_locations(window_start:1:window_finish) , 2*S_mi(window_finish:-1:window_start) , '--b')
plot(inv_adim_locations(window_start:1:window_finish) , U_mi(window_finish:-1:window_start) , 'k')
grid on
set(gca,'FontSize',16)
xlabel(' $X/H_t$','Interpreter','latex')
ylabel('$\mu$ expanded uncertainties  ','Interpreter','latex')
legend('Hough resolution', 'AR', '$U_{data,\mu}$ ', '$U_{\mu,tot}$ ','Interpreter','latex')
hold off
%saveas(gcf, 'Uncertainties_mi_axial', 'bmp')
%savefig('unc_mi_breakdown.fig')


% Plot the uncertainty contributions on the  Mach (Unc. breakdown)
%U_mach (total expanded uncertainty on the mach)
figure(9)
plot(inv_adim_locations(window_start:1:window_finish) , U_Mach_ext(window_finish:-1:window_start) , '--r') %the ext stands for the extraction to the mach number
hold on
plot(inv_adim_locations(window_start:1:window_finish) , 2*S_Mach(window_finish:-1:window_start) , '--b')
plot(inv_adim_locations(window_start:1:window_finish) , U_Mach(window_finish:-1:window_start) , 'k')
grid on
% xlabel('adimensionalized axis of the nozzle with the throat height x/H_t','Interpreter','latex')
xlabel('x/H_t')
ylabel('U_M Breakdown')
% legend('extracted line resolution propagated to the Mach ', 'U_\textrm{tot} Mach','Interpreter','latex')
legend('$U_{\textrm{extraction}}$', '$U_{\textrm{typeA}}$ ','$U_{\textrm{tot,exp}}$ on Mach')
% title('Expanded uncertainties the Mach Number')
set(gca,'FontSize',14)
%print -depsc Uncertainties_mach
hold off
% saveas(gcf, 'Uncertainties_mach', 'bmp')
%savefig('unc_mach_breakdown.fig')


%%
% Plotting the uncertainties contributions on the axial coordinate
figure(10)
plot(inv_adim_locations , U_adim(end:-1:1) , '--r')
 hold on 
plot(inv_adim_locations , dx(end:-1:1)./(2*throat_pixels) , '--c')
plot(inv_adim_locations , U_Xadim_tot(end:-1:1) , 'k')
grid on
xlabel('x/H_t')
ylabel('Position Uncertainties Breakdown ')
% legend('extracted line resolution propagated to the Mach ', 'total expanded uncertainty on the Mach','Interpreter','latex')
legend('$U_{X,NRS}$ ', '$U_{\Delta X}$ ', '$U_\textrm{X,tot} $')
% title('DIscretisation Uncertainty on the Mach Number')
set(gca,'FontSize',14)
%print -depsc Uncertainties_mach_position
hold off
% saveas(gcf, 'Uncertainties_mach_axial', 'bmp')
%savefig('unc_X_breakdown.fig')



%% Display average calculated lines on the first Schlieren image 

figure(11)
imshow( I(:,25:end,Nimg(1)) )
hold on
for k = window_start :1: window_finish
    
delta_x(k) = 0.5*Avg_LinesLength(k)*cos(mi_angle_avg(k)*pi/180);
delta_y(k) = 0.5*Avg_LinesLength(k)*sin(mi_angle_avg(k)*pi/180);
  
   plot( center_pixels(k,1,1), center_pixels(k,2,1), 'ro') %plotting centers of int windows, which for now are constant for various frames
hold on
  plot( [center_pixels(k,1,1)-delta_x(k) center_pixels(k,1,1)+delta_x(k)], [center_pixels(k,2,1)+delta_y(k) center_pixels(k,2,1)-delta_y(k)],'c', 'linewidth', 2 ) % plot ([x1 x2] [y1 y2)]
hold on

end
hold off
%saveas(gcf, 'superimposed_lines', 'bmp')

% SAVING AVERAGE LINES TO SUPERIMPOSE THEM ON A SCHLIEREN IMAGE IN THE WALL
% LINE DETECTION PROGRAM:
save ('midplane_avg_lines.mat', 'delta_x', 'delta_y', 'center_pixels', 'window_start', 'window_finish')




%% displaying some useful quantities, with pixels system of reference.
disp(' |Extended Experim uncertainty in the mach line angle (in radiants)|');
U_extraction

disp('|Experim propagated experimental Unc in the Mach number| ')
U_Mach_ext

disp(' |Extended Unc of the mach line angle|');
U_mi

disp('|Extended Unc of the Mach number| ')
U_Mach


% the misalignment can be neglected as long as the lines are touching the
% midplane.


%% Plotting with latex fonts

addpath(genpath(cd)) %genpath: generate path string p = genpath(folderName)
                     %addpath: adds a path string to the top of the search path
% close all
set(groot, 'defaultAxesTickLabelInterpreter','latex'); 
set(groot, 'defaultLegendInterpreter','latex');

%%%%%%%%%%Import Data from Simulation %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% tic
% % [n,s,r] = xlsread('ORCHID_Case_Plots.xlsx');
% [n,s,r] = xlsread('PR025.xlsx', 1);
% toc

% T = readtable('ORCHID_Case_Plots.xlsx','Sheet',2,'Range','A1:G11'); % First read the file in
% T = readtable('ORCHID_Case_Plots.xlsx','Sheet',2); % First read the file in. I had to remove the quotations in Liams spreadsheet. 

T = readtable('PR025.xlsx','Sheet',1); % First read the file in. I had to remove the quotations in Liams spreadsheet.
% T = readtable('PR025.xlsx','Sheet',2); % First read the file in. I had to remove the quotations in Liams spreadsheet.
%clean table
T.Var8=[];
T.Var9=[];
T.Var10=[];
T.Var11=[];
T.Var12=[];
T.Var13=[];
T.Var14=[];

% new_s = strrep(s,'\\TUD278272\D__ORCHID_control_system_02___Data_DSC\TUD278272\Temp Lib\','');
% new_s = strrep(s,'\\KOKSLAAN-PC\F__Labview_Database_DSC\TUD278272\Temp Lib\','');
% new_s = strrep(s,'Scannivalve PI-401','Scannivalve_P01');
% s = s(1,:);% first row of headings, in case you have some others.
% new_s = strrep(s,'"x_coord"','x_coord');
% new_s = strrep(new_s,'"y_coord"','y_coord');
% new_s = strrep(new_s,'"Pressure"','Pressure');
% new_s = strrep(new_s,'"Pressure_Coefficient"','Pressure_Cooefficient');
% new_s = strrep(new_s,'"Mach_Number"','Mach_Number');

%Modify variable names
new_s = {'Global_Index' 'x_coord' 'y_coord' 'Nondim_Pressure' 'Pressure_Coefficient' 'Mach_Number' 'Dimensional_Pressure'};
T.Properties.VariableNames = new_s;
%clean table
T.Global_Index=[];
T.Nondim_Pressure=[];
T.Pressure_Coefficient=[];
%Remove the variables that are not needed
%T = removevars(T,{'FC001on','FC001sp','FC001sp','FC003on','FC003sp','FN003_005on','FN003_005sp','LIA001','MOV002a','MOV002b','MOV006a','MOV006b','MOVI002a' ...
 %   'MOVI002b','MOVI006a','MOVI006b','PCV003','PCV003on','PCVI003' ... 
 %   'PU002on','Scannivalve_P01','Scannivalve_P02','Scannivalve_P03','Scannivalve_P04','Scannivalve_P05','Scannivalve_P06','Scannivalve_P07','Scannivalve_P08','Scannivalve_P09' ... 
 %   'Scannivalve_P10','Scannivalve_P11','Scannivalve_P12','Scannivalve_P13','Scannivalve_P14'});

%%%%%%%%%%%%%%%%%%%%%%%%%%%% Midplane %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ------------------ Find nozzle midline numbers ---------------------
% Split the data into two separate tables. For the midplane and the top 

%find the data on the midplane only
i = 1;
k = 1;
for i = 1:length(T.y_coord)
    if T.y_coord(i) == 0                   %Condition on y coordinate y = 0
        Mid.x_coord(k) = T.x_coord(i);
        Mid.y_coord(k) = T.y_coord(i);
        Mid.Pressure(k) = T.Dimensional_Pressure(i);
        Mid.Mach_Number(k) = T.Mach_Number(i);
%         Mid.X_adjusted(k) = T.X_adjusted(i);
        k = k +1;
    end 
end 

figure(10)
plot(Mid.x_coord,Mid.Mach_Number,'-');
hold on

%%%%%%%%%%%%%%%%%%%%%%%%%%%% Top %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % ------------------ Find nozzle top numbers ---------------------
% 
% i = 1;
% k = 1;
% 
% for i = 1:length(T.y_coord)
%     if T.y_coord(i) > 0                 %Condition on y coordinate y > 0
%         Top.x_coord(k) = T.x_coord(i);
%         Top.y_coord(k) = T.y_coord(i);
%         Top.Pressure(k) = T.Dimensional_Pressure(i);
%         Top.Mach_Number(k) = T.Mach_Number(i);
% %         Top.X_adjusted(k) = T.X_adjusted(i);
%         k = k +1;
%     end 
% end 
% 
% figure(11)
% plot(Top.x_coord,Top.Mach_Number,'.');
% hold on
% 
% 
% figure(12)
% plot(Top.x_coord(1:10),Top.y_coord(1:10));
% hold on


% ------------------------------------------------------------------------ 
% Sort the data to put in order of x position
% -------------------------------------------------------------------------
%first sort the data on the x coordinate. All data corresponds to
%increasing x but then the data will be a combination of data on minplane
%and the top profile. 
xpossorted = sort(T.x_coord);
Ypossorted = zeros(length(xpossorted),1);
Msorted = zeros(length(xpossorted),1);
Pressuresorted = zeros(length(xpossorted),1);
            
for i = 1:length(xpossorted)
    for j = 1:length(T.x_coord)
        if T.x_coord(j) == xpossorted(i)
            Ypossorted(i) = T.y_coord(j);
            Msorted(i) = T.Mach_Number(j);
            Pressuresorted(i) = T.Dimensional_Pressure(j); % 
        end
    end
end
% figure(13)
% plot(xpossorted,Msorted,'-');
% hold on


% figure(14)
% plot(xpossorted(1:10),Ypossorted(1:10));
% hold on

%use this loop to separate the data and target thetop profile
i = 1;
k = 1;
for i = 1:length(Ypossorted)
    if Ypossorted(i) > 0                 %Condition on y coordinate y > 0
        Top.x_coord(k) = xpossorted(i);
        Top.y_coord(k) = Ypossorted(i);
        Top.Pressure(k) = Pressuresorted(i);
        Top.Mach_Number(k) = Msorted(i);
%         Top.X_adjusted(k) = T.X_adjusted(i);
        k = k +1;
    end 
end 

figure(15)
plot(Top.x_coord,Top.Mach_Number,'-');
hold on


figure(16)
plot(Top.x_coord,Top.y_coord);
hold on
plot(Mid.x_coord,Mid.y_coord, '-r');




% i = 1;
% k = 1;
% for i = 1:length(T.y_coord)
%     if T.y_coord(i) == 0                   %Condition on y coordinate y = 0
%         Mid.x_coord(k) = xpossorted(i);
%         Mid.y_coord(k) = Ypossorted(i);
%         Mid.Pressure(k) = Pressuresorted(i);
%         Mid.Mach_Number(k) = Mcentresorted(i);
% %         Mid.X_adjusted(k) = T.X_adjusted(i);
%         k = k +1;
%     end 
% end 
% 
% figure(17)
% plot(Mid.x_coord,Mid.Mach_Number,'-');
% hold on

% Arrange the data on the top profile in acending order. At the moment the nodes are not
% ordered in any particular way. Order based on increasing mach number or
% decreasing pressure.

% % ------------------ Sort nozzle top numbers ---------------------
% Mach_limits = [min(Top.Mach_Number) max(Top.Mach_Number)];
% Mach = sort(Top.Mach_Number);             % A new vector with Mach sorted from low to high (some problems near start and end --- Mach decreases sometimes --- problems with mesh?)
% 
% 
% i = 1;
% k = 1;
% for i = 1:length(Mach)
%     idx = find(Top.Mach_Number == Mach(i));
%     
%     Top_order.x_coord(k) = Top.x_coord(idx);
%     Top_order.y_coord(k) = Top.y_coord(idx);
%     Top_order.Pressure(k) = Top.Pressure(idx);
%     Top_order.Mach_Number(k) = Top.Mach_Number(idx);
%     Top_order.X_adjusted(k) = Top.X_adjusted(idx);
%         
%     k = k +1;
% end 
% 
% 
% % Find the throat coordinate geometrically. 
% 
% y_throat = min(Top_order.y_coord);            %The throat is located at the minimum of the ordered y_coordinate vector
% inx_throat = find(Top_order.y_coord == y_throat);       %Take the index and use it to find the coordiante of the x_adjusted
% x_throat_adjusted = Top_order.X_adjusted(inx_throat);


y_throat = min(Top.y_coord);            %The throat is located at the minimum of the ordered y_coordinate vector
top_inx_throat = find(Top.y_coord == y_throat);       %Take the index and use it to find the coordiante of the x_adjusted
Top.x_throat = Top.x_coord(top_inx_throat);

% Find the throat coordinate 
% M_throat_logicalIndexes = Mid.Mach_Number(Mid.Mach_Number>=0.99 & Mid.Mach_Number<=1.01);
% M_throat_logicalIndexes = Mid.Mach_Number > 0.95 & Mid.Mach_Number < 1.1;

%Find the closest value in Mid.Mach_Number to 1. However, the geometrical
%thorat will not correspond to the position of M = 1, exactly. This is
%correct. I only made it since it should be very close --- extra check. 
[m,i] = min(abs(Mid.Mach_Number-1));
% inx_throat = find(Mid.Mach_Number == M_throat);
x_throat_check = Top.x_coord(i);

%the resulting x coordinate should be father away than the geometrical
%throat. 

%Since the above was based on the top profile. There will be a
%corresponding x coordinate with the above index which should match the x
%coordinate from the top profile. check

%x_throat_middle_check =  Mid.x_coord(inx_throat); %Not true since there
%may be less entries on the midplane. The x_cordinate, which was obtained
%from the y profile will be the same as the midplane x coordinate



%%%%%%%%%%%%%%%%%% Need only the diverging portions of the nozzle %%%%%%%
%TOP. 
%Take only the diverging part of the nozzle
Top_diverging.x_coord = Top.x_coord(top_inx_throat:end);
Top_diverging.y_coord = Top.y_coord(top_inx_throat:end);
Top_diverging.Pressure = Top.Pressure(top_inx_throat:end);
Top_diverging.Mach_Number = Top.Mach_Number(top_inx_throat:end);

%Given that the throat is the reference in the experiment the x coordinates
%must be changed again. The throat starts at 0. Minus the first entry from
%the x_adjusted vector.

Top_diverging.x_coord_ref = Top_diverging.x_coord - Top.x_coord(top_inx_throat);


figure(18)
plot(Top_diverging.x_coord,Top_diverging.Mach_Number, 'r.');
hold on
plot(Top_diverging.x_coord_ref,Top_diverging.Mach_Number, 'k.');
hold on


figure(19)
plot(Top_diverging.x_coord,Top_diverging.y_coord, 'r.');
hold on
plot(Top_diverging.x_coord_ref,Top_diverging.y_coord, 'k.');
hold on

%%%%%%%%%%%%%%%%%% Need only the diverging portions of the nozzle %%%%%%%
%Middle
%Take only the diverging part of the nozzle
       %Use the x coordinate of the top profile to find the x coordinate of the middle. This
%way the geometrical throat cooresponds. 

% [m,i] = min(abs(Top.x_throat-1));
% % inx_throat = find(Mid.Mach_Number == M_throat);
% x_throat_check = Top.x_coord(i);


[m,Mid_inx_throat] = min(abs(Mid.x_coord - Top.x_throat));
Mid.x_throat = Mid.x_coord(Mid_inx_throat);

Mid_diverging.x_coord = Mid.x_coord(Mid_inx_throat:end);
Mid_diverging.y_coord = Mid.y_coord(Mid_inx_throat:end);
Mid_diverging.Pressure = Mid.Pressure(Mid_inx_throat:end);
Mid_diverging.Mach_Number = Mid.Mach_Number(Mid_inx_throat:end);

%Given that the throat is the reference in the experiment the x coordinates
%must be changed again. 

Mid_diverging.x_coord_ref = Mid_diverging.x_coord - Mid.x_coord(Mid_inx_throat+1);


figure(19)
plot(Mid_diverging.x_coord,Mid_diverging.Mach_Number, 'r.');
hold on
plot(Mid_diverging.x_coord_ref,Mid_diverging.Mach_Number, 'k.');
hold on


figure(20)
plot(Mid_diverging.x_coord,Mid_diverging.y_coord, 'r.');
hold on
plot(Mid_diverging.x_coord_ref,Mid_diverging.y_coord, 'k.');
hold on

% figure(7)
% hold on
% plot(Mid_diverging.X_adjusted_ref*10*100, Mid_diverging.Mach_Number , 'o')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%% Plot the midline Mach %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%SIMULATION and EXPERIMENT%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

plot_lines_setup;

fig_name = 'fig1';

%Axis and legend 
xxlabel = '$x/H_\textrm{th}$ [-]';
yylabel = '$M_{\textrm{mid}}$ [-]';
% legendd = {'PRSV' 'Experiment' 'Nozzle Throat'};
legendd = {'SU2 -- PRSV' 'Experiment'};
location = 'southeast';

% Line type and markers type
Plot_op.Linestyle = {'--' 'none' 'none' 'none' 'none' 'none'};
Plot_op.Linewidth = [0.5 0.5 0.5 0.5 0.5 0.5];
Plot_op.Marker    = {[]  'o'  'o'  's'    'd'    'o' };
Plot_op.MarkerFaceColor = {'black' 'black' 'black' 'black' 'black' 'black' 'black' 'black' };
Plot_op.MarkerEdgeColor = {'black' 'black' 'black' 'black' 'black' 'black' 'black' 'black' };
Plot_op.Color    = {'r' 'b' 'k'};




X(1).X = Mid_diverging.x_coord_ref./y_throat;  %The x axis for the simulation
% X(1).X = Mid_diverging.x_coord_ref*10*100;
% adim_x_domain_half = adim_x_domain/2;           %The nozzle throat half height is half the adim_x_domain
adim_x_domain_half = x_domain_mm./(throat_mm/2);
X(2).X = adim_x_domain_half;                       %The x axis for the experiement  


% X(2).X =x_domain_mm;
% X(1).X_throat

Y(1).Y = Mid_diverging.Mach_Number;            %The y data for the simulation
Mach_avg(Mach_avg==0) = NaN;
Y(2).Y = flip(Mach_avg);                       %The y data for the experiment  



%Statistics
%Mean
U_Mach(U_Mach==0) = NaN;
E(1).E_std_Mach =  flip(U_Mach);
E(1).E_std_Discretization =  flip(U_Xadim_tot);
% E(2).E_std = X; %Put here the uncertainty of the location. Probably this comes from the estimation of the throat location AND? 
%Std. Deviation


x_lim = [round(min(X(1).X),2)   round(max(X(1).X), 2)];
x_lim = [0 10];
y_lim = [round(min(Mid.Mach_Number),2)   round(max(Mid.Mach_Number),2)];
y_lim = [1.1   2.1];

plot_X_vs_Yerrorbar(X         , Y      ,  E  , ...
    x_lim     , y_lim      , ...
    xxlabel   , yylabel    , ...
    legendd   , location   , ...
    x_lab_fac , y_lab_fac  , ...
    fig_name  , print_name , ...
    width     , Plot_op    , ...
    fs        , fs_fac, ...
    font      , latex      , ...
    textt);


% print -depsc Machvsnondim_PR025NT002
% savefig('Machvsnondim_PR025NT002.fig')

%% Automated plots
%[status,sheets] = xlsfinfo('PR025.xlsx');

load 'CFD_PR26_IGPoly.mat'
X(1).X = M_midplane(:,1);
Y(1).Y = M_midplane(:,2);
% X(2).X = M_wall(:,1);
% Y(2).Y = M_wall(:,2);
clear M_midplane
clear M_wall


load 'CFD_PR26_PRPoly.mat'
X(2).X = M_midplane(:,1);
Y(2).Y = M_midplane(:,2);
% X(2).X = M_wall(:,1);
% Y(2).Y = M_wall(:,2);
clear M_midplane
clear M_wall


load 'CFD_PR26_PRnonPoly.mat'
X(3).X = M_midplane(:,1);
Y(3).Y = M_midplane(:,2);
% X(1).X = M_wall(:,1);
% Y(1).Y = M_wall(:,2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%% Plot the midline Mach %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%SIMULATION and EXPERIMENT%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

plot_lines_setup;

fig_name = 'fig1';

%Axis and legend 
xxlabel = '$x/H_\textrm{th}$ [-]';
yylabel = '$M_{\textrm{mid}}$ [-]';
% legendd = {'PRSV' 'Experiment' 'Nozzle Throat'};
% legendd = {'PR Nonpolytropic' 'PR Polytropic' 'IG Polytropic' 'IG Nonpolytropic' 'Experiment'};
legendd = {'IG Polytropic' 'PR Polytropic' 'PR Nonpolytropic'  'Experiment'};
location = 'southeast';

% Line type and markers type
Plot_op.Linestyle = {'-' '--' '-.' 'none' 'none'};
Plot_op.Linewidth = [0.5 0.5 0.5 0.5 0.5 0.5];
Plot_op.Marker    = {[]  [] []    'o'    'o' };
Plot_op.MarkerFaceColor = {'black' 'black' 'black' 'black' 'black' 'black' 'black' 'black' };
Plot_op.MarkerEdgeColor = {'black' 'black' 'black' 'black' 'black' 'black' 'black' 'black' };
Plot_op.Color    = {'c' 'g' 'b' 'r' 'k'};


% X(1).X = Mid_diverging.x_coord_ref./y_throat;  %The x axis for the simulation
% X(1).X = Mid_diverging.x_coord_ref*10*100;
% adim_x_domain_half = adim_x_domain/2;           %The nozzle throat half height is half the adim_x_domain
% adim_x_domain_half = x_domain_mm./(throat_mm/2);
% X(3).X = adim_x_domain_half;                       %The x axis for the experiement  
X(4).X = inv_adim_locations; 

% X(2).X =x_domain_mm;
% X(1).X_throat

% Y(1).Y = Mid_diverging.Mach_Number;            %The y data for the simulation
Mach_avg(Mach_avg==0) = NaN;
Y(4).Y = flip(Mach_avg);                       %The y data for the experiment  



%Statistics
%Mean
U_Mach(U_Mach==0) = NaN;
E(4).E_std_Mach =  flip(U_Mach);
E(4).E_std_Discretization =  flip(U_Xadim_tot);
% E(2).E_std = X; %Put here the uncertainty of the location. Probably this comes from the estimation of the throat location AND? 
%Std. Deviation


x_lim = [round(min(X(1).X),2)   round(max(X(1).X), 2)];
x_lim = [0 6.0];
y_lim = [round(min(Y(1).Y),2)   round(max(Y(1).Y),2)];
y_lim = [1.0   2.4];

plot_X_vs_Yerrorbar2(X         , Y      ,  E  , ...
    x_lim     , y_lim      , ...
    xxlabel   , yylabel    , ...
    legendd   , location   , ...
    x_lab_fac , y_lab_fac  , ...
    fig_name  , print_name , ...
    width     , Plot_op    , ...
    fs        , fs_fac, ...
    font      , latex      , ...
    textt);
grid on
 print -depsc Midplane_assessment_PR26
% savefig('Mach_mid_PR025NT002.fig')

endf=2;
