%% = = = = = = = = Statystical convergence assessment for the line extraction tools= = = = = = = ==== = = = =
clear all
close all
clc
addpath(genpath(cd)) %genpath: generate path string p = genpath(folderName)
                     %addpath: adds a path string to the top of the search path
  
%% =================== INPUT 0 =================================  
%best mach equation coming from cfd simulation + pre processing of images through external software (Davis) for image alignment
%the x must be adimensionalized x=[0,1], and represents the start(zero) and
%end(1) of the acquisition domain.
estimated_machangle_start = 30; %[deg] - towards exit, at the start of the domain
estimated_machangle_middle = 50; %[deg]
estimated_machangle_end = 60; %[deg]  - towards throat, at the end of the domain
xx_f= [0 , 0.5 ,  1]';
yy_f = [estimated_machangle_start , estimated_machangle_middle, estimated_machangle_end ]';

ideal_mi = fit(xx_f, yy_f, 'poly2'); %this is the ideal estimation of the mi angle .

%Gets the total number of images in the directory
a = dir(['PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection' '/*.tif']);
total = size(a,1)-1;

%% _________ TYPE OF RUN ____________

% DECIDE IF A FULL ANALYSIS WANTS TO BE PERFORMED
FULL_ANALYSIS = 1; % DEFAULT: =1. If this =0, no alalysis is performed, previous data will be loaded and plotted.

if FULL_ANALYSIS==1
    fprintf('decide if the results will be saved or not \n')

    % DECIDE IF THE ANALYSIS WILL SAVE RESULTS OR NOT:
simulation_save_results = 0 ; %if =1 , results will be saved. if=0, it is a provvisory Run and only figures will be saved, no data.
end

%% -------- Input 1: acquisition of images and binarization  ----------------

if FULL_ANALYSIS==1 % perform Stat conv calculation 
    
    
    %----- read the images------
    
    FolderIn='PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection';
    %FolderIn ='PR025NT002_Bobcat_Zoom_Run1_SS_Fabio_process_Align';
    
    RootIn = 'B0';
%     RootIn = 'Align0000';
    
    %rotation of calibration and data so that they match
    image_rotate_cal = -1.4; %[degrees]
    image_rotate_data =-1.4 ; %[degrees]
    
    %read the image used to calculate the scale factor and throat position
    %calibration_img = imread('calibration_PR025NT002_Bobcat_Zoom_Run1_SS_Fabio_process_Align.bmp');
    calibration_img = imread('PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection/B00001.tif');
    
    %This is compuls. If no calibration (do the experiment again) but if can't
    %then read the first frame.
    
    %The dimensions of the calibration img are the dimension i want for the
    %Schlieren imgs, in case the processed schlieren have different (larger) dimensions
    m = length(calibration_img(1,:)); %columns
    n = length(calibration_img(:,1)); %rows
    
    %Choose the acquisition method:
    MPI = 1;     %1 means the calculation is done in series
    %2 means the calculation is done in parallel
    
    %binarization parameters:
    brightness_data =50; % [pixels] Adapt the brightness of the images
    %threshold_imgad = [0.1  0.95]; %[low, high]: adjust brightness: increase higher threshold to get white out of light grey area
    threshold_imgad=NaN;
    
    % Edge detection  Parameters:
    threshold = [0.005, 0.01];
    gauss_noise_filter = 2.5; % gaussian sigma filter. Increase to reduce noise.
    

    Nimg=247:1:total; % when is the first run: use only 10 images to calibrate all parameters needed.
    %Nimg=1:1:total; %define the vector representing ALL the images/frames available on which apply the stat conv program
    
    
    if MPI==1
        %------------------ serial implementation ---------------------
        tic
        for x = Nimg
            fprintf('Acquiring image %s %d\n',RootIn,x)
            %StringIn = sprintf('%s/%s%04d.tif',FolderIn,RootIn,x);
            StringIn = sprintf('%s/%s%04d.tif',FolderIn,RootIn,x);
            
            Raw_img = imread(StringIn);
            Raw_img = imrotate(Raw_img, image_rotate_data); %rotate calibration img of x degrees IF it is not parallel to ground
            
%             grayscale(:,:) = rgb2gray(Raw_img(1:n,1:m,:)); %if the image is rgb, convert to greyscale (FOR BMPs)
            
            grayscale = Raw_img(1:n,1:m)+brightness_data; % add brightness
            
            if isnan(threshold_imgad)
                I(:,:,x) = imadjust(grayscale);
            else
                I(:,:,x) = imadjust(grayscale, threshold_imgad); % [low, high]: adjust brightness: increase lower threshold to get white out of light grey area
            end
            %adjust this value to achieve better detected lines on the midplane of
            %the image. Always use the automatic to SEE how it behaves.
            %If the higher threshold is too low then you will saturate some parts of the image
            
            %Binarized images:
            edge_BW(:,:,x) = edge( I(:,:,x), 'Canny', threshold, gauss_noise_filter );
        end
        toc
    end %end of serial implementation
    
    %------------------ parallel implementation ------------------%
    if MPI==2
        tic
        parfor t = Nimg(1): Nimg(end)
            
            %[edge_BW_2, I_2, grayscale_2] = edge_BWparbmp(t, brightness_data, threshold, gauss_noise_filter,FolderIn,RootIn ,threshold_imgad,image_rotate_data,n,m);
            [edge_BW_2, I_2] = edge_BWpar(t, brightness_data, threshold, gauss_noise_filter,FolderIn,RootIn ,threshold_imgad,image_rotate_data);
            % disp(V);
            % figure(3000)
            % imshow(V(:,:,t))
            % edge_BW_2(:,:,t) = V(1);
            % edge_BW(:,:,t) = V(:,:,t);
            edge_BW(:,:,t) = edge_BW_2(:,:,t);
            I(:,:,t) = I_2(:,:,t);
            %         grayscale(:,:,t) = grayscale_2;
        end
        toc
        
    end %end parallel implementation
    
    
    
    %%  INPUT 2: Working space definition
    
    N = length(edge_BW(:,1,1)); % number of rows of the binarized schlieren img
    M = length(edge_BW(1,1:end,1)); % number of columns of the binarized schlieren img
    offset = 5; %increment offset to move up, decrement to move down in the image
    Y_0 = N/2 - offset; % should be the centerline of the nozzle if the image is centered
    
    %indentify the position at which the method finishes extracting lines, in pixels:
    M_finish = round(0.72*M);
    
    % %identify throat position in pixels
    %     M_throatposition = round(0.68*M);
    
    %Plot original first img with throat and axis, and the edge binarization version
    %Allows to see if the midplane is correctly centered
    figure(1)
    imshowpair(I(:,:,Nimg(1)), edge_BW(:,:,Nimg(1)), 'montage')
    hold on
    plot([1, M],[Y_0, Y_0])
    plot([M_finish M_finish], [1 N-1])
    legend('Nozzle axis', 'end of line extraction')
    hold off
    
    
    %% INPUT 3: define discretization and calculation parameters =============================
    % Define number of intervals
    intervals = 30; % only for divergent part of nozzle
    
    window_start = 1;  %first int window where lines are going to be extracted
    window_finish = intervals; % last int window where lines are going to be extracted
    
    Initial_dx_offset=5; %[pixels]: use =0 to not skip any pixels in the line acquisition
    %use if it is required to start acquiring lines n pixels further from the left side of the image
    
    %create the length in pixels of each Int window (NON CONSTANT step)
    dx_const = floor( 2*(M_finish-Initial_dx_offset)/intervals );
    
    dx_min = 38; % TO ESTABLISH based on uncertainty analysis. Useful for linear decrement of dx.
    
    dx_max = 2*dx_const - dx_min; % trapezoidal distribution (linear dx decrease...): calculated to cover the same distance as dx const
    
    upp_max = 25;
    low_max = 25;
    
    for int = 1:1:intervals
        dx(int)= floor( -((dx_max-dx_min)/intervals)*int + dx_max );
        
        NL(int) = Y_0 - round(low_max*dx(int)/dx_max); % lower bound of the interrogation window on the y axis
        NU(int) = Y_0 + round(upp_max*dx(int)/dx_max); % upper bound of the interrogation window on the y axis
        
    end
    
    Hough_theta_step=0.2;
    
    first_frame=247; % first frame of th dataset
    start_from = 247; % first number of images on which to calculate results
    StatConvIterations = 10;
    StatConvVector = floor(linspace(start_from , total, StatConvIterations));
    statystical_conv_step = StatConvVector(end)-StatConvVector(end-1); %This is the stepsize used when computing the 2nd order moments
    
    % lines extraction parameters:
    C0 = 1; % initial factor for max length lines to be identified
    C_min = 0.45; % end min factor for min line length
    fill_gap_lines = 1;
    max_pixel_skip = 3;
    
    LocalPeaks_number=2; %number of local peaks that hthe transform is going to find. Max 3 (recommended 2 or 3)
    
    
    %% ================== Statystical convergence calculation =====================================%%
    
    s_cycle = 1; %This variable counts with s in order to correctly store the statistical quantities at the end of the for loop
    
    tic
    for s = StatConvVector(2:end) %cycle for statystical analysis
        
        
        Nimg = first_frame:1:s; %vector representing the frames
        
        % ----------------serial calculation------------------------
        %(parallel definetly not recommended: errors that pop up while calculating are
        % extremely important.
        
        
        for x = Nimg
            % USE HOUGH TRANSFORM ON BINARY IMAGE MADE WITH EDGE
            %for each frame, for each interval create the int_window and apply HOUGH transform
            
            accum = Initial_dx_offset; %accum is the variable that represents the step. ACCUM=0 if you want to acquire from first pixel.
            
            for i = 1:1:intervals
                
                int_window = edge_BW(NL(i):1:NU(i) , 1+accum:1:dx(i)+accum, x );
                local_cartesian_axis_origins(i, :) = [ round(1+accum) , NL(i) ]; %constant if the images have constant pixels/dimensions
                center_pixels(i,:)= [ round(dx(i)/2 + accum) , Y_0]; %constant if the images have constant pixels/dimensions
                
                accum = accum + floor((dx(i))/2) ;
                
                [H,THETA, RHO1] = hough( int_window , 'theta', -70:Hough_theta_step:70 );
                
                %FIND PEAKS OF THE TRANSFORM
                PEAKS = houghpeaks(H,LocalPeaks_number);
                
                min_length_lines(i) = C0*length(int_window(:,1)); %the min length it identifies in the transform, based on the Height of the window.
                WINDOWS{x,i}= int_window;
                
                %Cycle to maximise line length and quality of the extracted lines
                maxlinefound = 0;
                while maxlinefound == 0 % FIND LINES and SAVE RESULTS : repeat the line identification up until all max length lines are found
                    LINES{x,i}= houghlines( int_window, THETA, RHO1, PEAKS,'FillGap',fill_gap_lines,'MinLength',min_length_lines(i));
                    
                    % if the line IS found: exit the while and proceed
                    if isempty(LINES{x,i})==0
                        maxlinefound=1;
                    end
                    % if the line is NOT found but i still can reduce length: decrease the min length to be detected.
                    if isempty(LINES{x,i})==1 && min_length_lines(i)>floor( C_min*length(int_window(:,1)) )
                        min_length_lines(i)= min_length_lines(i)-2;
                    end
                    % if the line is NOT found but i have already short lines increase the pixel skip
                    if isempty(LINES{x,i})==1 && min_length_lines(i)<=floor( C_min*length(int_window(:,1)) ) && fill_gap_lines<=max_pixel_skip
                        fill_gap_lines = fill_gap_lines+1;
                        gap_skip(x,i)= fill_gap_lines;
                    end
                    if isempty(LINES{x,i})==1 && min_length_lines(i)<=floor( C_min*length(int_window(:,1)) ) && fill_gap_lines>max_pixel_skip
                        maxlinefound=1; % if its impossible to find a line, just skip it... later it will use the previous one found
                    end
                end
                
            end  % for all the intervals hough tansform for a single frame was applied
            
            
            %now for each window of interest, extract data from lines.
            for k = window_start:1:window_finish
                
                if isempty(LINES{x,k})==0 % if there ARE lines n that windo k:
                    
                    xy_local = [LINES{x,k}.point1 ;  LINES{x,k}.point2 ]; % (has all the lines)
                    xy_m= length(xy_local(1,:)); % format: [x_line1pt1 , y_line1pt1 , x_line2pt1 yline2pt1 ; x_line1pt2 , y_line1pt2 , x_line2pt2 yline2pt2 ]
                    % -> represets how many lines there are: if I have length 4 i have 2 lines.
                    local_SR_matrix = zeros(2,xy_m);
                    local_SR_matrix(1,1:2:xy_m) = local_cartesian_axis_origins(k,1) ;
                    local_SR_matrix(1,2:2:xy_m) = local_cartesian_axis_origins(k,2);
                    local_SR_matrix(2,1:2:xy_m) = local_cartesian_axis_origins(k,1) ;
                    local_SR_matrix(2,2:2:xy_m) = local_cartesian_axis_origins(k,2);
                    xy = xy_local(:,:) + local_SR_matrix(:,:);  %struct where i save the first line detected in global cartesian coordinates
                    CARTESIANLINES{x,k}.xy = xy;
                    theta_angles= extractfield(LINES{x,k}, 'theta') ; % its a local temporary variable containing all angles of all the lines detected. coose the best one. choose the best one
                    
                    if x==Nimg(1) %for the first frame take the first angle as the best one
                        th_opt=1; %choose the first angle detected.
                        best_theta(x,k)= theta_angles(th_opt);
                        LinesLength(x,k)= sqrt( (xy(2,1)-xy(1,1))^2 + (xy(2,2)-xy(1,2))^2 ) ;  % xy(point, coordinate)-> x_2-x_1= xy(point2,coordinate1)-xy(point1,coordinate1)
                        CenterLines(x,k,1) = (xy(1,1)+xy(2,1))/2; %is the X of the center of the Line detected in the global SR
                        CenterLines(x,k,2) = (xy(1,2)+xy(2,2))/2; %is the Y of the center of the Line detected in the global SR
                        
                    end
                    
                    if  x>Nimg(1) %even if you only have one angle this works: choose the best angle.
                        adim_position= center_pixels(k,1)/M_finish;
                        ideal_theta(k)= abs(90 - ideal_mi(adim_position) );
                        
                        for m=1:1:length(theta_angles(:)) % CONSIDER USING THE THEORETICAL THETA if u have it. otherwise usse theta_avg.
                            variation_th(m) = abs( ideal_theta(k)- abs(theta_angles(m)) ); %always positive
                        end
                        
                        [min_var , th_opt] = min(variation_th); %find the minimum variation and its index=th_opt
                        best_theta(x,k)= theta_angles(th_opt); %save the best theta and carry on
                        mm= th_opt*2; % index to select the th_opt line from the xy matrix
                        LinesLength(x,k)= sqrt( (xy(2,mm-1)-xy(1,mm-1))^2 + (xy(2,mm)-xy(1,mm))^2 ) ;  % xy(point, coordinate)-> x_2-x_1= xy(point2,coordinate1)-xy(point1,coordinate1);
                        CenterLines(x,k,1) = (xy(1,mm-1)+xy(2,mm-1))/2; %is the X of the center of the Line detected in the global SR
                        CenterLines(x,k,2) = (xy(1,mm)+xy(2,mm))/2; %is the Y of the center of the Line detected in the global SR
                        
                        %saving the line corresponding to the best theta angle %th_op
                    end
                    
                    theta_avg(k) = mean( abs(best_theta(: , k )) ); %the avg theta on the axial coordinate is used to determine the best angle
                    
                    variation_th=0; % re initializing the vector for next k-th calculation
                end %End of "if there Are lines".
                
                if isempty(LINES{x,k})==1 && x==Nimg(1) && k==1 % if there are NOT lines in that window and its the FIRST FRAME
                    fprintf( 'DO NOT CONTINUE: FATAL ERROR: lines not found at 1st window. RESTART and change line acquisition parameters \n', k)
                    best_theta(x,k)= NaN  ;
                    LinesLength(x,k)= NaN;
                end
                if isempty(LINES{x,k})==1 && x==Nimg(1)  % if there are NOT lines in that window and its the FIRST FRAME
                    fprintf( 'SERIOUS ERROR (first frame error): lines not found at window %d. Using k-1th window info. Best to restart and change line acquisition parameters \n', k)
                    best_theta(x,k)= best_theta(x,k-1)  ;
                    LinesLength(x,k)= LinesLength(x,k-1);
                end
                if isempty(LINES{x,k})==1 && x>Nimg(1) % if there are NOT lines in that window (x>1)
                    fprintf( 'minor error: lines not found at window %d, at frame %d . Using line info of  previous frame. CHECK lineskipcount vector. \n', k, x)
                    best_theta(x,k)= best_theta(x-1,k)  ;
                    LinesLength(x,k)= LinesLength(x-1,k);
                end
                
                % extraction of Mach lines and mach number
                if best_theta(x,k)<=0 && best_theta(x,k)>-70 % if the polar coordinate angle is negative, it means the line has a positive angle with respect to the x axis
                    mi_angle(x,k) = best_theta(x,k) +90 ;
                    Mach(x,k) = 1/sin(mi_angle(x,k)*pi/180);
                end
                if best_theta(x,k)>0 && best_theta(x,k)<70  % the +- 70 deg is to avoid detecting impossible mi angles(less than 20deg->M=3)
                    mi_angle(x,k) = best_theta(x,k) -90 ;
                    Mach(x,k) = 1/sin(- mi_angle(x,k)*pi/180);
                end
                if best_theta(x,k)<=-70 || best_theta(x,k)>=70
                    if x>1
                        best_theta(x,k) = best_theta(x-1,k);
                        mi_angle(x,k) = 90- abs(best_theta(x,k));
                        Mach(x,k) = 1/sin(mi_angle(x,k)*pi/180);
                    else
                        best_theta(x,k) = best_theta(x,k-1);
                        mi_angle(x,k) = 90 - abs(best_theta(x,k));
                        Mach(x,k) = 1/sin(mi_angle(x,k)*pi/180);
                    end
                end
                
            end  % for the single frame save results: Lines and angles.
            
        end %finish cycle  on frames
        
        
        %--------------statystical quantities calculation ------------
        %for each axial coordinate(int_window) calculate the statystical quantities  of the various frames analyzed.
        
        Number_of_samples = length(Nimg);
        
        %Preallocation
        %     Avg_LinesLength = zeros(window_finish(end));
        %     Avg_angular_res = zeros(op,window_finish(end));
        %     std_dev_angular_res = zeros(op,window_finish(end));
        %     statysticalU_mi_exper = zeros(op,window_finish(end));
        %     U_mi_exper = zeros(window_finish(end));
        %
        %     mi_angle_avg = zeros(op,window_finish(end));
        %     S_mi = zeros(op,window_finish(end));
        %     statysticalU_mi = zeros(op,window_finish(end));
        %     U_mi = zeros(op,window_finish(end));
        
        for k = window_start:1:window_finish
            
            mi_angle_filtered(Nimg(1):Nimg(end),k)=filloutliers(abs(mi_angle(Nimg(1):Nimg(end),k)), 'linear', 'percentiles', [10, 90] );
            mi_angle_filtered(1:Nimg(1)-1,k) = NaN;
            
            Mach_filtered(Nimg(1):Nimg(end),k) = filloutliers(Mach(Nimg(1):Nimg(end),k), 'linear', 'percentiles', [10, 90]);
            Mach_filtered(1:Nimg(1)-1,k) = NaN;
            
            Avg_LinesLength(s_cycle,k)= nanmean(LinesLength(first_frame:Nimg(end),k));
            std_LL(s_cycle,k) = nanstd(LinesLength(first_frame:Nimg(end),k));
            Avg_angular_res(s_cycle,k) = atan( 1/Avg_LinesLength(s_cycle,k) ); %average angular res based on the length of the identified lines
            
            % to get the uncertainty that the mean is actually the true mean i devide
            % by sqrt of N samples. This shows that for a high enough number of
            % samples the extended uncertanties I get  are the true one
            
            mi_angle_avg(s_cycle,k)= nanmean(abs(mi_angle_filtered(first_frame:Nimg(end),k)));  %average mach line angle mi
            S_mi(s_cycle,k)= nanstd(abs(mi_angle(first_frame:Nimg(end),k)));           %std deviation of the mi angle for each station - 68% Conf Lvl
            statysticalU_mi(s_cycle,k)= 2*(S_mi(s_cycle,k)/sqrt(Number_of_samples)) ;
            
            Mach_avg(s_cycle,k) = nanmean(Mach_filtered(first_frame:Nimg(end),k));              %average Mach number M
            S_Mach(s_cycle,k) = nanstd(Mach(first_frame:Nimg(end),k)); %std deviation of the Mach number for each station (68%ConfLvl)
            
            statysticalU_Mach(s_cycle,k) =  2*(S_Mach(s_cycle,k)/sqrt(Number_of_samples));
            % all the Uncerainties of the angle mi are in Degrees!
            
            
        end
        
        s_cycle = s_cycle + 1;
        
        
        
    end
    toc
    
    %% checking data
    %Display lines on Int_windows at a certain frame (For CHECKING step by step purposes)
    
%     x=10;
%     for k = window_start:2:window_finish
%        figure(k+10)
%        imshow(WINDOWS{x,k})
%        hold on
%        xy = [LINES{x,k}.point1; LINES{x,k}.point2];
%        plot(xy(:,1),xy(:,2),'LineWidth',2,'Color','green');
%        hold off
%     end
    %% SAVING RESULTS:
  if simulation_save_results==1  
    fprintf('clearing some space \n')
    clear I
    clear grayscale
    clear Raw_img
    fprintf('DONE: clearing some space \n')
    
    fprintf('saving results... \n')
    tic
    save('Variables_StatConv_PR22')
    toc
    fprintf('DONE: saving results... \n')
  end  % end of: I wan to save results flag.
    
end % end of the : perform full analysis


%% Loading RESULTS:

if FULL_ANALYSIS ==0 % if no analysis is performed, load results from previous analysis
fprintf('Loading previous results... \n')
tic
load('Variables_StatConv')
toc
fprintf('DONE:Loading previous results... \n')
end

%% =========== RESULTS of the statystical convergence study  ====================

locations = center_pixels( window_start:window_finish , 1 , 1 )' ;  % in pixels
s_cycle=s_cycle-1;
n_frames = (1:1:s_cycle-1)*statystical_conv_step;


% PLOT 1:  mi angle avg value
figure(5)
plot(n_frames, mi_angle_avg(1:1:s_cycle-1,5))
hold on
plot(n_frames, mi_angle_avg(1:1:s_cycle-1,10))
plot(n_frames, mi_angle_avg(1:1:s_cycle-1,15))
plot(n_frames, mi_angle_avg(1:1:s_cycle-1,20))
 plot(n_frames, mi_angle_avg(1:1:s_cycle-1,25))
grid on
grid minor
legend(' k=5', 'k=10', 'k=15', 'k=20', 'k=25')
xlabel('number of images')
ylabel('average \mu')
% title(' average \mu angle as a function of the number of iterations, in various k-locations along the nozzle')
hold off
%print -depsc mi_angle_avg_Adam % rename-label it
%savefig('mi_angle_avg_statconv.fig')

%  PLOT 2: std deviation of mi angle 
figure(6)
plot(n_frames, S_mi(1:1:s_cycle-1,2))
hold on
plot(n_frames, S_mi(1:1:s_cycle-1,10))
plot(n_frames, S_mi(1:1:s_cycle-1,18))
plot(n_frames, S_mi(1:1:s_cycle-1,26))
plot(n_frames, S_mi(1:1:s_cycle-1,30))
grid on
legend(' k=2', 'k=10', 'k=18', 'k=26', 'k=30')
xlabel('number of images')
ylabel(' \mu std. deviation ')
% title(' mi angle std deviations with number of iterations, in various k-locations along the nozzle')
hold off
%print -depsc mi_angle_std_Adam % rename-label it
%savefig('mi_angle_std_stat_conv.fig')


%% Final LaTex Plots:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% This block is for nice plots %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% close all
set(groot, 'defaultAxesTickLabelInterpreter','latex'); 
set(groot, 'defaultLegendInterpreter','latex');

plot_lines_setup;

fig_name = 'fig1';

%Axis and legend 
xxlabel = '$N_\textrm{im}$';
yylabel = '$\mu_{\textrm{avg}}$ [-]';
% legendd = {'PRSV' 'Experiment' 'Nozzle Throat'};
legendd = {' k=2' 'k=10' 'k=18' 'k=26' 'k=30'};
location = 'northeast';

% Line type and markers type
Plot_op.Linestyle = {'-' '--' '-.' '-' '-.' 'none'};
Plot_op.Linewidth = [0.5 0.5 0.5 0.5 0.5 0.5];
Plot_op.Marker    = {[]  []   []   []     []     []};
Plot_op.MarkerFaceColor = {'black' 'black' 'black' 'black' 'black' 'black' 'black' 'black' };
Plot_op.MarkerEdgeColor = {'black' 'black' 'black' 'black' 'black' 'black' 'black' 'black' };
Plot_op.Color    = {'k' 'k' 'k' 'k' 'k' 'k'};




X(1).X = n_frames;  %The x axis for the simulation
% X(1).X = Mid_diverging.x_coord_ref*10*100;
% adim_x_domain_half = adim_x_domain/2;           %The nozzle throat half height is half the adim_x_domain
% adim_x_domain_half = x_domain_mm./(throat_mm/2);
% X(2).X = adim_x_domain_half;                       %The x axis for the experiement  


% X(2).X =x_domain_mm;
% X(1).X_throat

Y(1).Y = mi_angle_avg;            %The y data for the simulation
% Mach_avg(Mach_avg==0) = NaN;
% Y(2).Y = flip(Mach_avg);                       %The y data for the experiment  



%Statistics
%Mean
% U_Mach(U_Mach==0) = NaN;
% E(1).E_std_Mach =  flip(U_Mach);
% E(1).E_std_Discretization =  flip(U_Xadimtot);
% E(2).E_std = X; %Put here the uncertainty of the location. Probably this comes from the estimation of the throat location AND? 
%Std. Deviation


x_lim = [round(min(X(1).X),2)   round(max(X(1).X), 2)];
% x_lim = [0 10];
y_lim = [round(min(min(Y(1).Y)),3)   round(max(max(Y(1).Y)),3)];
y_lim = [29  60];

plot_X_vs_Y(X         , Y      ,s_cycle,  ...
    x_lim     , y_lim      , ...
    xxlabel   , yylabel    , ...
    legendd   , location   , ...
    x_lab_fac , y_lab_fac  , ...
    fig_name  , print_name , ...
    width     , Plot_op    , ...
    fs        , fs_fac, ...
    font      , latex      , ...
    textt);


fig_name = 'fig2';

%Axis and legend 
xxlabel = '$N_\textrm{im}$';
yylabel = '$\mu_{\textrm{avg}}$ [-]';
% legendd = {'PRSV' 'Experiment' 'Nozzle Throat'};
legendd = {' k=2' 'k=10' 'k=18' 'k=26' 'k=30'};
location = 'northeast';

% Line type and markers type
Plot_op.Linestyle = {'-' '--' '-.' '-' '-.' 'none'};
Plot_op.Linewidth = [0.5 0.5 0.5 0.5 0.5 0.5];
Plot_op.Marker    = {[]  []   []   []     []     []};
Plot_op.MarkerFaceColor = {'black' 'black' 'black' 'black' 'black' 'black' 'black' 'black' };
Plot_op.MarkerEdgeColor = {'black' 'black' 'black' 'black' 'black' 'black' 'black' 'black' };
Plot_op.Color    = {'k' 'k' 'k' 'k' 'k' 'k'};




X(1).X = n_frames;  %The x axis for the simulation
% X(1).X = Mid_diverging.x_coord_ref*10*100;
% adim_x_domain_half = adim_x_domain/2;           %The nozzle throat half height is half the adim_x_domain
% adim_x_domain_half = x_domain_mm./(throat_mm/2);
% X(2).X = adim_x_domain_half;                       %The x axis for the experiement  


% X(2).X =x_domain_mm;
% X(1).X_throat

Y(1).Y = S_mi;            %The y data for the simulation
% Mach_avg(Mach_avg==0) = NaN;
% Y(2).Y = flip(Mach_avg);                       %The y data for the experiment  



%Statistics
%Mean
% U_Mach(U_Mach==0) = NaN;
% E(1).E_std_Mach =  flip(U_Mach);
% E(1).E_std_Discretization =  flip(U_Xadimtot);
% E(2).E_std = X; %Put here the uncertainty of the location. Probably this comes from the estimation of the throat location AND? 
%Std. Deviation


x_lim = [round(min(X(1).X),2)   round(max(X(1).X), 2)];
% x_lim = [0 10];
y_lim = [round(min(min(Y(1).Y)),3)   round(max(max(Y(1).Y)),3)];
y_lim = [0.2  1];

plot_X_vs_Y(X         , Y      ,s_cycle,  ...
    x_lim     , y_lim      , ...
    xxlabel   , yylabel    , ...
    legendd   , location   , ...
    x_lab_fac , y_lab_fac  , ...
    fig_name  , print_name , ...
    width     , Plot_op    , ...
    fs        , fs_fac, ...
    font      , latex      , ...
    textt);








%%
% PLOT 3: Mach average values 
figure(7)
plot(n_frames, Mach_avg(1:1:s_cycle-1,2))
hold on
plot(n_frames, Mach_avg(1:1:s_cycle-1,10))
plot(n_frames, Mach_avg(1:1:s_cycle-1,22))
plot(n_frames, Mach_avg(1:1:s_cycle-1,26))
 plot(n_frames, Mach_avg(1:1:s_cycle-1,30))
grid on
grid minor
legend(' k=2', 'k=10', 'k=22', 'k=26', 'k=30')
xlabel('number of images')
ylabel('average Mach')
% title(' Mach avg values with number of iterations, in various k-locations along the nozzle')
hold off
% print -depsc Mach_avg_Adam % rename-label it
%savefig('Mach_avg_statconv.fig')
%%


% PLOT 4 $:  std deviation of mach
figure(8)
plot(n_frames, S_Mach(1:1:s_cycle-1,1))
hold on
plot(n_frames, S_Mach(1:1:s_cycle-1,10))
plot(n_frames, S_Mach(1:1:s_cycle-1,20))
plot(n_frames, S_Mach(1:1:s_cycle-1,30))
grid on
grid minor
legend(' k=1', 'k=10', 'k=20', 'k=30')
xlabel('number of images')
ylabel('Mach std. deviation')
% title(' Mach std deviations with number of iterations, in various k-locations along the nozzle')
hold off
%print -depsc Mach_std_Adam % rename-label it
%savefig('Mach_std_statconv.fig')


% 
% % PLOT 5: Avg angular resolution 
% figure(9)
% plot(n_frames, Avg_angular_res(1:1:s_cycle-1,1))
% hold on
% plot(n_frames, Avg_angular_res(1:1:s_cycle-1,10))
% plot(n_frames, Avg_angular_res(1:1:s_cycle-1,15))
% plot(n_frames, Avg_angular_res(1:1:s_cycle-1,20))
% plot(n_frames, Avg_angular_res(1:1:s_cycle-1,30))
% grid on
% legend(' k=1', 'k=10', 'k=15', 'k=20', 'k=30')
% xlabel('number of images')
% ylabel('average AR')
% %title(' avg angular resolution with number of iterations, in various k-locations along the nozzle')
% hold off
% %print -depsc angular_res_avg_Adam % rename-label it
% savefig('angular_res_avg_statconv.fig')
% 
% 
% 
% % PLOT 6: std deviation of mi angle 
% figure(10)
% plot(n_frames, std_LL(1:1:s_cycle-1,1))
% hold on
% plot(n_frames, std_LL(1:1:s_cycle-1,10))
% plot(n_frames, std_LL(1:1:s_cycle-1,15))
% plot(n_frames, std_LL(1:1:s_cycle-1,20))
% plot(n_frames, std_LL(1:1:s_cycle-1,30))
% grid on
% legend(' k=1', 'k=10', 'k=15', 'k=20', 'k=30')
% xlabel('number of images')
% ylabel('u_{LL}')
% % title(' std deviation on the lines length as a function of the number of iterations, in various k-locations along the nozzle')
% hold off
% %print -depsc angular_res_std_Adam % rename-label it
% savefig('u_LL_statconv.fig')
% 
% 

%% Plots of the statystical convergence of the uncertainties:

% verify that with a sufficient number of samples the statystical uncertainties tend to the real ones:
figure(11)
plot(n_frames, statysticalU_mi(1:1:s_cycle-1,5))
hold on
plot(n_frames, statysticalU_mi(1:1:s_cycle-1,10))
plot(n_frames, statysticalU_mi(1:1:s_cycle-1,15))
plot(n_frames, statysticalU_mi(1:1:s_cycle-1,20))
% plot(n_frames, statysticalU_mi(1:1:s_cycle-1,25))
plot(n_frames, statysticalU_mi(1:1:s_cycle-1,30))
grid on
legend(' k=5', 'k=10', 'k=15', 'k=20', 'k=30')
xlabel('number of images')
ylabel('u_{\mu}')
% title(' statystical  uncertainty on mi angles with number of iterations, in various k-locations along the nozzle')
hold off
%print -depsc statist_uncertainty_mu_Adam % rename-label it
%savefig('statist_uncertainty_mu_statconv.fig')



figure(13)
plot(n_frames, statysticalU_Mach(1:1:s_cycle-1,2))
hold on
plot(n_frames, statysticalU_Mach(1:1:s_cycle-1,10))
plot(n_frames, statysticalU_Mach(1:1:s_cycle-1,22))
plot(n_frames, statysticalU_Mach(1:1:s_cycle-1,26))
plot(n_frames, statysticalU_Mach(1:1:s_cycle-1,30))
% plot(n_frames, statysticalU_Mach(1:1:s_cycle-1,30))
grid on
legend(' k=2', 'k=10', 'k=22', 'k=26', 'k=30')
xlabel('number of images')
ylabel('u_{M}/N^{0.5}')
% title(' statystical Mach uncertainty U_M with number of iterations, in various k-locations along the nozzle')
hold off
%print -depsc statist_uncertainty_Mach_Adam % rename-label it
%savefig('statist_uncertainty_Mach_statconv.fig')




