%% =======================  UPPER AND LOWER WALLS MI ANGLE AND MACH EXTRACTION  ============================================================
% this program Works well on bmp and 16 bit tiffs
%.Tif is recommended.

%% INPUT 0 %%
tic
  
clear all; clc; close all

set(groot, 'defaultAxesTickLabelInterpreter','latex'); 
set(groot, 'defaultLegendInterpreter','latex');

addpath(genpath(cd)) %genpath: generate path string p = genpath(folderName)
                     %addpath: adds a path string to the top of the search path
  
fprintf('decide if it is necessary to LOAD previous results: \n')
% Loading previous results will not prevent the full calculation: go down
% to the plotting section if it is necessary to ONLY plot previous results.
LOAD_PREVIOUS_RESULTS = 0 ; 
%if =1 , results will be LOADED


fprintf('decide if the results will be saved or not \n')
% DECIDE IF THE ANALYSIS WILL SAVE RESULTS OR NOT:
simulation_save_results = 0 ; 
%if =1 , results will be saved.

toc
%% =========== INPUT 1 : read and binarize images ========================== 
%FolderIn = 'PR025NT002_Bobcat_Zoom_Run1_SS_Fabio_process_Align';
FolderIn='PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection';

% RootIn = 'B0';
RootIn = 'B0';

calibration_img = imread('PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection/B00001.tif'); %read the image used to calculate the scale factor and throat position
%calibration_img = imread('calibration_PR025NT002_Bobcat_Zoom_Run1_SS_Fabio_process_Align.bmp'); %read the image used to calculate the scale factor and throat position

%dimensions of the calibr img are the dimension i want for the Schlieren imgs
m = length(calibration_img(1,:)); %columns
n = length(calibration_img(:,1)); %rows

%%%%%%%%%%%%%%%%% Adjustment Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%rotate the calibration to align with the schlieren. 
image_rotate_cal = -1.4; %[degrees] %rotates the calibration image 
% image_rotate_data = image_rotate_cal;
image_rotate_data = -1.4;  % Rotates the experimental data
calibration_img = imrotate(calibration_img, image_rotate_cal); %rotate calibration img of x degrees IF it is not parallel to ground

% calibration_img=calibration_img(1:end-30, 1:end); %when you rotate an image, extra(black) pixels are added: you can remove them if you want. 
calibration_img = calibration_img(1:end, 1:end); %when you rotate an image, extra(black) pixels are added: you can remove them if you want. 
%It could be that pixels are not added below a certain rotation. if not
%added then comment it out. 

%Choose the acquisition method:
MPI =1;      %1 means the calculation is done in series
             %2 means the calculation is done in parallel
             %3 means the calculation is done both in series and parallel (for comparison)
a = dir(['PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection' '/*.tif']);
total = size(a,1);

Nimg = 227:1:total-10; %vector that represents all the images to acquire

threshold = [0.01, 0.1]; % [low,high]: increase low to avoid detecting small gradients; decrease high to detect smaller gradients.

gauss_noise_filter = 4; % gaussian sigma filter: Increase to reduce noise

%******************* Apply the filters above to the raw data
%Modify the acquisition cycle depending on the type of image you have

%%%%%%%%%%%%%%%%% Adjustment Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

brightness_data = 60;
% threshold_imgad = [0.05  0.99];
threshold_imgad = NaN;
%MODIFY the acquisition cycle depending on the type of image you have

%-----------------series implementation--------------------
if MPI == 1 || MPI == 3
    tic
    for x = Nimg
        %This for loop takes the raw data: rotates it, increases the brightness, adjusts the intensity of the image to
        %make the lines more visible, easily detectable. Then it takes the image
        %and binarises (1,0)s such that edges can be detected.
        
        fprintf('Acquiring image %s %d\n',RootIn,x)
        %StringIn = sprintf('%s/%s%04d.bmp',FolderIn,RootIn,x); Uncomment this line if you have bmps
        StringIn = sprintf('%s/%s%04d.tif',FolderIn,RootIn,x);  %Uncomment this line if you have tifs
        
    
        Raw_img = imread(StringIn);
        Raw_img = imrotate(Raw_img, image_rotate_data); %rotate calibration img of x degrees IF it is not parallel to ground
%for bmps: uncomment these 2 lines.
        %grayscale = rgb2gray(Raw_img(1:n,1:m,:)); %if the image is rgb(bmpfiles), convert to greyscale. This depends on the image.     
        %grayscale= grayscale+ brightness_data; %for bmps

        grayscale = Raw_img(1:n,1:m,:) + brightness_data; % add brightness if the image is too dark

        if isnan(threshold_imgad)
            I(:,:,x) = imadjust(grayscale);
        else
            I(:,:,x) = imadjust(grayscale, threshold_imgad); % [low, high]: adjust brightness: increase lower threshold to get white out of light grey area
        end
        %adjust this value to achieve better detected lines on the midplane of
        %the image. Always use the automatic to SEE how it behaves. 
        %If the higher threshold is too low then you will saturate some parts of the image

      %Binarized images:        
        edge_BW(:,:,x) = edge( I(:,:,x), 'Canny', threshold, gauss_noise_filter );
        %edge_BW(:,:,x) = imcomplement(edge_BW(:,:,x)); %black line over white background: just for reference or better picture       
    end
    toc
end


%-----------------parallel implementation---------------------- %
tic
%preallocation 
% edge_BW = zeros(n,m,Nimg);
% I = zeros(n,m,Nimg);
if MPI == 2 || MPI == 3
    tic
    parfor t = Nimg(1): Nimg(end)
        
        %[edge_BW_2, I_2, grayscale_2] = edge_BWparbmp(t, brightness_data, threshold, gauss_noise_filter,FolderIn,RootIn ,threshold_imgad,image_rotate_data,n,m);
         [edge_BW_2, I_2] = edge_BWpar(t, brightness_data, threshold, gauss_noise_filter,FolderIn,RootIn ,threshold_imgad,image_rotate_data);
        % disp(V);
        % figure(3000)
        % imshow(V(:,:,t))
        % edge_BW_2(:,:,t) = V(1);
        % edge_BW(:,:,t) = V(:,:,t);
        edge_BW(:,:,t) = edge_BW_2(:,:,t);
        I(:,:,t) = I_2(:,:,t);
%         grayscale(:,:,t) = grayscale_2;
    end
    toc
end


%% INPUT 2:  wall  parameters 

walldomain_start= 0.1; %in percentage of the image: where the walla cquisition should start
walldomain_end = 0.72;  %in percentage of the image: where the walla cquisition should start

adim_walldomain = [walldomain_start walldomain_end] ; % the interval of interest to get get the curves of the walls
m_start= round(adim_walldomain(1)*m);
m_end= round(adim_walldomain(2)*m);

threshold_schlieren= 0.07; % lower threshold to get less details in B&W image


offset =0; %increment offset to move up, decrement to move down in the image

Y_0 = n/2 - offset;

N = length(edge_BW(:,1,Nimg(1))); % number of rows of the binarized schlieren img
M = length(edge_BW(1,1:end,Nimg(1))); % number of columns of the binarized schlieren img
%if the offset is not zero (or certain threshold), recenter all the images in the dataset:
%NB: re-acquire images when modifying the offset! (Re-run input 1 section)
if abs(offset)>50
    for x=Nimg
        edge_BW(:,:,x) = imtranslate(edge_BW(:,:,x), [0, floor(offset/2)]);
    end
 Y_0 = Y_0 + round(offset/2);
 
    figure(11)
    imshow(edge_BW(:,:,Nimg(1)))
    hold on
    plot([1, M],[Y_0, Y_0])
    plot([m_end m_end], [1 N-1])
    legend('Nozzle axis/midplane', 'end of line extraction')
    hold off
end

if abs(offset)>50
  Schlieren_1frame_BW = imtranslate( imbinarize(I(:,:,Nimg(1)), threshold_schlieren), [0 floor(offset/2)]); % binarization of the image with Imbinarize
else
  Schlieren_1frame_BW = imbinarize(I(:,:,Nimg(1)), threshold_schlieren); % binarization of the image with Imbinarize 
end


figure(1)
imshow(Schlieren_1frame_BW)

figure(2)
imshowpair(I(:,:,Nimg(1)),edge_BW(:,:,Nimg(1)), 'montage' )
hold on
plot([1, m],[Y_0, Y_0])
plot([adim_walldomain(1)*m , adim_walldomain(1)*m], [1 n-1])
plot([adim_walldomain(2)*m , adim_walldomain(2)*m], [1 n-1])
hold off


%order of the polynomial curve that has to fit the walls in the specified domain
poly_order_walls = 5;


% --------------- CHOICE OF THE WALL -------------
%by default the program will compute the analysis on BOTH walls. it is
%possible here to avoid the computation on the worst wall saving time. NB:
%to do this, flow symmetry must be proven first.
%The default is BEST_WALL=''. This means both walls will be analyzed.

BEST_WALL = 'lower';

USE_BEST_WALL_DATA=0;% if this is one, the wall with the lowest uncertainty will be used to evaluate 
%the wall angle and uncertainty reguardless if the wall is the one where
%lines are extracted or not. This reduces uncertainty assuming that the
%walls should be equal.


%% ============= CALIBRATION 1: Schlieren wall detection

% Upper curve
i=1;
column=m_start;
while column < m_end
    flag=0;
    row=1;
    while row<Y_0 && flag==0
        if Schlieren_1frame_BW(row,column)==1
            x_fit_u(i)=column;
            y_fit_u(i)=row;
            i=i+1;
            flag=1;
        else
            row=row+1;
        end
    end
    column=column+1;
end
% Lower curve
i=1;
column=m_start;
while column < m_end
    flag=0;
    row=n;
    while row>Y_0 && flag==0
        if Schlieren_1frame_BW(row,column)==1
            x_fit_l(i)=column;
            y_fit_l(i)=row;
            i=i+1;
            flag=1;
        else
            row=row-1;
        end
    end
    column=column+1;
end
%now i have also the walls of the schlieren image.

[SchlierenW1, Ssw1]= polyfit(x_fit_u, y_fit_u, poly_order_walls);
[SchlierenW2, Ssw2]= polyfit(x_fit_l, y_fit_l, poly_order_walls);
Walls_x_pixeldomain= m_start:1:m_end;
[Swall_u , Sdelta_u] = polyval(SchlierenW1, Walls_x_pixeldomain, Ssw1); %delta is the standard dev: 68% conf lvl
[Swall_l , Sdelta_l] = polyval(SchlierenW2, Walls_x_pixeldomain, Ssw2);

figure(111)
imshow(Schlieren_1frame_BW )
hold on
plot([1, m],[Y_0, Y_0], '--')
plot([adim_walldomain(1)*m , adim_walldomain(1)*m], [1 n-1])
plot([adim_walldomain(2)*m , adim_walldomain(2)*m], [1 n-1])
plot( Walls_x_pixeldomain, floor(Swall_u), 'linewidth', 2)
plot( Walls_x_pixeldomain, floor(Swall_l), 'linewidth', 2)


%% =========== INPUT 3 final inputs from line_detection_program and discretization parameters -----------------

load('calibration_dataPR26.mat') %loading throat location and scale factor from lone detection program

% now you have the calibration walls( x_pixeldomain, upper_wall, lowerwall, std deviations, M_throatposition, scale factor, fractional uncertainty of X_NRS ecc) and all details:

figure(222)
imshow(Schlieren_1frame_BW )
hold on
plot([adim_walldomain(1)*m , adim_walldomain(1)*m], [1 n-1])
plot([adim_walldomain(2)*m , adim_walldomain(2)*m], [1 n-1])
plot([1, m],[Y_0, Y_0], '--')
plot( Walls_x_pixeldomain, floor(Swall_u), 'linewidth', 2)
plot( Walls_x_pixeldomain, floor(Swall_l), 'linewidth', 2)
plot( x_pixeldomain, floor(upper_wall), 'linewidth', 1)
plot( x_pixeldomain, floor(lower_wall), 'linewidth', 1)



%Remember:
% 1= upper wall above y_0; 2 lower wall below y_0.
%remeber that the starting  point of counting is top left.-> SR is top left for pixels.


USE_SCHLIEREN_WALLS = 1; % Flag=1: RECOMMENDED  only if walls are decent.
%if it is 1 then the detected walls on the schlieren are used,
%if it is 0 then the user must specify the wall angle: by default the calibration image walls are used.

if USE_SCHLIEREN_WALLS==0
    fprintf(' Using data from the calibration img in calibration_data.mat \n')
    
    % localize the offset starting point:
    ii=1;
    flag=0;
    while flag==0
       if x_pixeldomain(ii) == Walls_x_pixeldomain(1) 
           flag=1;
       else
           ii=ii+1;
       end
    end
    indx_start= ii;
end

hough_theta_step = 0.25; % resolution of the hough transform

theta_vector_l = [ -75:hough_theta_step:-5 , 5:hough_theta_step:75]; % polar coordinate System of reference angles considered for the transform for the lower curve
theta_vector_u = [ -75:hough_theta_step:-5 , 5:hough_theta_step:75];% polar coordinate System of reference angles considered for the transform for the upper curve

PixelSkipFactor = 3;
max_PixelSkipFactor = 5;
%the pixel skip factors represent the number of pixels that a line can skip to still be considered a line.
%It depends on the quality of the image and the quality of lines you have in that img. It also stongly depends on
% the length of the line you want to acquire. Increasing LinesLengthFactor
% must be accompanied by an increase in Pixel skip. It is NOT recommended to go beyond 8 pixels skip.
  
px_sk = PixelSkipFactor;  %initial reset value of pixel skip factor;   


% TRY ALWAYS TO HAVE VERY SIMILAR WINDOW SIZES!
h_window1 = 76; %height in pixels
base_window1 = 70; %base in pixels
y_top1= round(Swall_u(1))+2; %position of starting point for the windows: its top left corner is swall

h_window2 = 76; %height in pixels
base_window2 =70;  %base in pixels
y_bot2 = floor(Swall_l(1)); % lower pixel of the int window of the lower wall.
y_top2 = y_bot2 - h_window2-2; %the starting point of the matriz is top left corner always


alpha_w_max = 15*pi/180; %estimate the maximum wall angle of the analyzed part of the nozzle
mu_w_min = 35*pi/180 ; %estimate the low-average mach angle detected on the walls
initial_LineLengthFactor = 0.8 ;%initial minimum length of the detected line = percentage of the smaller side of the window
min_LinesLengthFactor = (0.5)*0.5*( (1+(base_window1/h_window1)*tan(alpha_w_max))/sin(alpha_w_max+mu_w_min) ); %final minimum length of the detected line = percentage of the smaller side of the window.
% going from a 99.7% of detecting lins to a 85% chance considering a
% gaussian distribution=deviding by 2.

if USE_BEST_WALL_DATA==0
   if  USE_SCHLIEREN_WALLS==1
       if mean(Sdelta_l)<mean(Sdelta_u)
           Sdelta_u=Sdelta_l;
       else
           Sdelta_l=Sdelta_u;
       end
       
   else
       if mean(delta_lower)<mean(delta_upper)
           delta_upper=delta_lower;
       else
           delta_lower=delta_upper;
       end
   end
       
end



%% ======================== CALCULATION: Wall MI ANGLE AND MACH EXTRACTION  ============================================================calculation on walls

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%serial implementation%%%%%%%%%%%%%%%%%%%%%%%%%
tic
total_loss1=0;
total_loss2=0;

for x = Nimg
    
    counter = Walls_x_pixeldomain(1); % start at the beginning of the wall x domain, number of pixels on x, from global SR
    accum1 = 1; %accumulation of pixels : n*basewindow for upper wall
    accum2 = 1; %accumulation of pixels : n*basewindow for lower wall
    
    dy1=y_top1; %initialization of vertical shift in pixels
    dy2=y_top2;
    
    i=1;
    linefound1=0;
    linefound2=0;
    
    while counter <= Walls_x_pixeldomain(end)- max([base_window1 , base_window2])
        
        PixelSkipFactor = px_sk;   %reset the pixel skip to first value;
        
        if isequal(BEST_WALL,'upper') || isempty(BEST_WALL)==1
            % USE HOUGH TRANSFORM ON BINARY IMAGE MADE WITH EDGE FOR THE UPPER WALL
            int_window1 = edge_BW((dy1):1:(h_window1+dy1) , counter:1:counter+base_window1 , x) ;
            
            center_pixels1(i,:)=[floor( counter+(base_window1/2) ), dy1+(length(int_window1(:,1))/2) ];
            local_axis_origin1(i, :) = [ counter , dy1 ]; % (x,y) in pixels, remember that origin is always TOP LEFT of the windows in pixel references.
            
            if USE_SCHLIEREN_WALLS==1
                wall_angle_u(i)= atan( (Swall_u(accum1+base_window1)-Swall_u(accum1))/base_window1 )*180/pi;
                % invert the sign to have the convention that diverging wall angles are positive, so upper wall has the positive atan sign, the lower is inverted
                
                %already 95%conf level
                unc_wall_angle_u(i)=180/pi*min( [(2*Sdelta_u(accum1+base_window1)*cos(wall_angle_u(i)*pi/180)^2)/(base_window1-4*Sdelta_u(accum1+base_window1)*cos(wall_angle_u(i)*pi/180)*sin(wall_angle_u(i)*pi/180)) ,...
                    (2*Sdelta_u(accum1)*cos(wall_angle_u(i)*pi/180)^2)/(base_window1-4*Sdelta_u(accum1)*cos(wall_angle_u(i)*pi/180)*sin(wall_angle_u(i)*pi/180))]);
                
            end
            
            if USE_SCHLIEREN_WALLS==0
                wall_angle_u(i)= atan( (upper_wall(indx_start+accum1+base_window1)-upper_wall(indx_start+accum1))/base_window1 )*180/pi;
                % invert the sign to have the convention that diverging wall angles are positive, so upper wall has the positive atan sign, the lower is inverted
                
                %already 95%conf level
                unc_wall_angle_u(i)=180/pi*min( [(2*delta_upper(indx_start+accum1+base_window1)*cos(wall_angle_u(i)*pi/180)^2)/(base_window1-4*delta_upper(indx_start+accum1+base_window1)*cos(wall_angle_u(i)*pi/180)*sin(wall_angle_u(i)*pi/180)) ,...
                    (2*delta_upper(indx_start+accum1)*cos(wall_angle_u(i)*pi/180)^2)/(base_window1-4*delta_upper(indx_start+accum1)*cos(wall_angle_u(i)*pi/180)*sin(wall_angle_u(i)*pi/180))]);
            end
            
            
            
            accum1=accum1+floor(base_window1/2);
            
            [H,THETA, RHO] = hough( int_window1, 'theta', theta_vector_u );
            
            min_length_lines1= initial_LineLengthFactor*length(int_window1(:,1)) ;
            
            PEAKS1 = houghpeaks(H,3);
            
            maxlinefound1=0;
            while maxlinefound1==0 % FIND LINES and SAVE RESULTS : repeat the line identific up until all max length lines are found
                LINES1{x,i}= houghlines( int_window1, THETA, RHO, PEAKS1,'FillGap',PixelSkipFactor,'MinLength',min_length_lines1);
                
                % if the line IS found: exit the while and proceed
                if isempty(LINES1{x,i})==0
                    maxlinefound1=1;
                end
                % if the line is NOT found but i still can reduce length: decrease the min length to be detected.
                if isempty(LINES1{x,i})==1 && min_length_lines1>floor( min_LinesLengthFactor*length(int_window1(:,1)))
                    min_length_lines1= min_length_lines1-1;
                end
                % if the line is NOT found but i have already short lines increase the pixel skip
                if isempty(LINES1{x,i})==1 && min_length_lines1<=floor(min_LinesLengthFactor*length(int_window1(:,1)) ) && PixelSkipFactor<=max_PixelSkipFactor
                    PixelSkipFactor = PixelSkipFactor+1;
                    gap_skip1(x,i)= PixelSkipFactor;
                end
                if isempty(LINES1{x,i})==1 && min_length_lines1<=floor( min_LinesLengthFactor*length(int_window1(:,1)) ) && PixelSkipFactor>max_PixelSkipFactor
                    maxlinefound1=1; % if its impossible to find a line, just skip it... later it will use the previous one found
                    total_loss1=total_loss1+1;
                end
            end
            
            WINDOWS1{x,i}=int_window1;
            
            if isempty(LINES1{x,i})==0 %line found: save that and and move along the upper wall
                thetas = extractfield(LINES1{x,i}, 'theta'); %extracted thetas
                linefound1=1;
                best_theta= thetas(1); %for now get the first one.
                if best_theta>0 %for the upper wall, theta>0 is the "flow Ddirection" mach line
                    mi_angle_u(x,i) = 90-best_theta + wall_angle_u(i); %wall angles are positive if the nozzle is diverging
                    Mach_u(x,i) = 1/sin(mi_angle_u(x,i)*pi/180);
                end
                if best_theta<0
                    mi_angle_u(x,i) = 90+best_theta -wall_angle_u(i);
                    Mach_u(x,i) = 1/sin(mi_angle_u(x,i)*pi/180);
                end
                
                %extracting true xy lines from the lines1 structure and saving them
                all_coord_pt1=  extractfield(LINES1{x,i}, 'point1'); %in the row: xy, xy... in the columns: various locations i
                all_coord_pt2=  extractfield(LINES1{x,i}, 'point2');
                mm=1*2; %variable to choose the right lines in xy matrix. For now get the first one.
                %saving the line corresponding to the best theta angle %th_opt
                xy1_pt1(i,:)= [all_coord_pt1(mm-1) , all_coord_pt1(mm)]; %x , y pt 1 del giusto th_best
                xy1_pt2(i,:)= [all_coord_pt2(mm-1) , all_coord_pt2(mm)]; %x , y pt 2 del giusto th_best
                
                LinesLength_u(x,i)= sqrt( (xy1_pt2(i,1)-xy1_pt1(i,1))^2 + (xy1_pt2(i,2)-xy1_pt1(i,2))^2 ) ;
                
                dy1 =  floor( Swall_u(accum1))+2;
            end %end of Islinefound=yes
            if isempty(LINES1{x,i})==1  %line NOT found, use nan
                fprintf( 'error: upper wall line not found at window %d, at frame %d. using nan \n Best to restart and change line acquisition parameters \n', i, x)
                mi_angle_u(x,i)=NaN;
                LinesLength_u(x,i)=NaN;
                Mach_u(x,i) = NaN;
                %             xy1_pt1(i,:)= xy1_pt1(i-1,:);
                %             xy1_pt2(i,:)= xy1_pt2(i-1,:);
                %             LinesLength_u(x,i)= sqrt( (xy1_pt2(i,1)-xy1_pt1(i,1))^2 + (xy1_pt2(i,2)-xy1_pt1(i,2))^2 ) ;
                dy1 =  floor( Swall_u(accum1));
            end %end line NOT found
            
            %clear one use only  variables:
            thetas=0;
            best_theta=0;
            
        end %end of the use upper wall
        
        
        if isequal(BEST_WALL,'lower') || isempty(BEST_WALL)==1
            % USE HOUGH TRANSFORM ON BINARY IMAGE MADE WITH EDGE FOR THE LOWER WALL
            
            int_window2 =  edge_BW((dy2):1:(h_window2+dy2) , counter:1:counter+base_window2 , x ) ;
            
            center_pixels2(i,:)=[floor( counter+(base_window2/2) ), dy2+(length(int_window2(:,1))/2) ];
            local_axis_origin2(i, :) = [ counter , dy2 ]; % (x,y) in pixels, remember that origin is always TOP LEFT of the windows in pixel references.
            
            if USE_SCHLIEREN_WALLS==1
                wall_angle_l(i)= -atan( (Swall_l(accum2+base_window2)-Swall_l(accum2))/base_window2 )*180/pi;
                % invert the sign to have the convention that diverging wall angles are positive, so upper wall has the positive atan sign, the lower is inverted
                
                %already 95%conf level
                unc_wall_angle_l(i)=180/pi*min( [(2*2*Sdelta_l(accum2+base_window2)*cos(wall_angle_l(i)*pi/180)^2)/(base_window2-2*2*Sdelta_l(accum2+base_window2)*cos(wall_angle_l(i)*pi/180)*sin(wall_angle_l(i)*pi/180)) ,...
                    (2*2*Sdelta_l(accum2)*cos(wall_angle_l(i)*pi/180)^2)/(base_window2-2*2*Sdelta_l(accum2)*cos(wall_angle_l(i)*pi/180)*sin(wall_angle_l(i)*pi/180))]);
            end
            
            if USE_SCHLIEREN_WALLS==0
                wall_angle_l(i)= -atan( (lower_wall(indx_start+accum2+base_window2)-lower_wall(indx_start+accum2))/base_window2 )*180/pi;
                % invert the sign to have the convention that diverging wall angles are positive, so upper wall has the positive atan sign, the lower is inverted
                
                %already 95%conf level
                unc_wall_angle_l(i)=180/pi*min( [(2*2*delta_lower(indx_start+accum2+base_window2)*cos(wall_angle_l(i)*pi/180)^2)/(base_window2-2*2*delta_lower(indx_start+accum2+base_window2)*cos(wall_angle_l(i)*pi/180)*sin(wall_angle_l(i)*pi/180)) ,...
                    (2*2*delta_lower(indx_start+accum2)*cos(wall_angle_l(i)*pi/180)^2)/(base_window2-2*2*delta_lower(indx_start+accum2)*cos(wall_angle_l(i)*pi/180)*sin(wall_angle_l(i)*pi/180))]);
            end
            
            accum2=accum2+floor(base_window2/2);
            
            [H,THETA, RHO] = hough( int_window2, 'theta', theta_vector_l );
            
            min_length_lines2= initial_LineLengthFactor*length(int_window2(:,1)) ;
            
            PEAKS2 = houghpeaks(H,3);
            
            maxlinefound2=0;
            while maxlinefound2==0 % FIND LINES and SAVE RESULTS : repeat the line identific up until all max length lines are found
                LINES2{x,i}= houghlines( int_window2, THETA, RHO, PEAKS2,'FillGap',PixelSkipFactor,'MinLength',min_length_lines2);
                
                % if the line IS found: exit the while and proceed
                if isempty(LINES2{x,i})==0
                    maxlinefound2=1;
                end
                % if the line is NOT found but i still can reduce length: decrease the min length to be detected.
                if isempty(LINES2{x,i})==1 && min_length_lines2>floor( min_LinesLengthFactor*length(int_window2(:,1)) )
                    min_length_lines2 = min_length_lines2-2;
                end
                % if the line is NOT found but i have already short lines increase the pixel skip
                if isempty(LINES2{x,i})==1 && min_length_lines2<=floor( min_LinesLengthFactor*length(int_window2(:,1)) ) && PixelSkipFactor<=max_PixelSkipFactor
                    PixelSkipFactor = PixelSkipFactor+1;
                    gap_skip2(x,i)= PixelSkipFactor;
                end
                if isempty(LINES2{x,i})==1 && (min_length_lines2<=floor( min_LinesLengthFactor*min([length(int_window2(1,:)),length(int_window2(:,1))]) ) && PixelSkipFactor>max_PixelSkipFactor)
                    total_loss2=total_loss2+1;
                    maxlinefound2=1; % if its impossible to find a line, just skip it... later it will use the previous one found
                end
            end
            
            WINDOWS2{x,i}=int_window2;
            
            if isempty(LINES2{x,i})==0 %line found: check if is the correct line, save that and and move along the upper wall
                thetas = extractfield(LINES2{x,i}, 'theta'); %extracted thetas
                linefound2=1;
                best_theta= thetas(1); %for now get the first one.
                if best_theta<0 %for the lower wall, theta<0 is the "flow Direction" mach line
                    mi_angle_l(x,i) = 90+best_theta + wall_angle_l(i); %wall angles are positive if the nozzle is diverging
                    Mach_l(x,i) = 1/sin(mi_angle_l(x,i)*pi/180);
                end
                if best_theta>0
                    mi_angle_l(x,i) = 90-best_theta -wall_angle_l(i);
                    Mach_l(x,i) = 1/sin(mi_angle_l(x,i)*pi/180);
                end
                
                
                %extracting true xy lines from the lines1 structure and saving them
                all_coord_pt1 =  extractfield(LINES2{x,i}, 'point1'); %in the row: xy, xy... in the columns: various locations i
                all_coord_pt2 =  extractfield(LINES2{x,i}, 'point2');
                mm=1*2; %variable to choose the right lines in xy matrix. For now get the first one.
                %saving the line corresponding to the best theta angle %th_opt
                xy2_pt1(i,:) = [all_coord_pt1(mm-1) , all_coord_pt1(mm)]; %x , y pt 1 del giusto th_best
                xy2_pt2(i,:) = [all_coord_pt2(mm-1) , all_coord_pt2(mm)]; %x , y pt 2 del giusto th_best
                
                LinesLength_l(x,i)= sqrt( (xy2_pt2(i,1)-xy2_pt1(i,1))^2 + (xy2_pt2(i,2)-xy2_pt1(i,2))^2 ) ;
                
                dy2 =  floor( Swall_l(accum2))-h_window2-2;
            end %end of Islinefound=yes for the first window
            
            if isempty(LINES2{x,i})==1  %line NOT found
                fprintf( 'error: lower wall line not found at window %d, at frame %d. Best to restart and change line acquisition parameters \n', i, x)
                mi_angle_l(x,i)=NaN;
                LinesLength_l(x,i)=NaN;
                Mach_l(x,i)= NaN;
                %             xy2_pt1(i,:)= xy2_pt1(i-1,:);
                %             xy2_pt2(i,:)= xy2_pt2(i-1,:);
                %             LinesLength_l(x,i)= sqrt( (xy2_pt2(i,1)-xy2_pt1(i,1))^2 + (xy2_pt2(i,2)-xy2_pt1(i,2))^2 ) ;
                dy2 =  floor( Swall_l(accum2))-h_window2-2;
            end %end line NOT found
            %clear one use only  variables:
            thetas = 0;
            best_theta = 0;
            
        end %end of "use the lower wall" if is the best one
        
        
        %increment count and check if limit is reached:
        counter = counter + floor(max(base_window1, base_window2)/2); %best if the windows are the same or almost the same.
        i = i+1;
    end %while cycle on walls : end of image==end of windows.
    
    
end%end the cycle on all the frames
toc
walls_iterations = i-1;


%% --------------------- RESULTS 1:  filtering --------------------------------

%filtering outliers from data


for k=1:1:walls_iterations
    
    if isequal(BEST_WALL,'upper') || isempty(BEST_WALL)==1
    [mi_u(Nimg(1):Nimg(end),k), TF1(Nimg(1):Nimg(end),k), L1, U1, C1] = filloutliers(mi_angle_u(Nimg(1):Nimg(end),k), 'linear', 'percentiles', [15, 85] );
    mi_u(1:Nimg(1)-1,k) = NaN;
    Mach_filtered_u(Nimg(1):Nimg(end),k) = filloutliers(Mach_u(Nimg(1):Nimg(end),k), 'linear', 'percentiles', [15, 85] );
    Mach_filtered_u(1:Nimg(1)-1,k) = NaN;
    end 
    
    if isequal(BEST_WALL,'lower') || isempty(BEST_WALL)==1
    [mi_l(Nimg(1):Nimg(end),k), TF2(Nimg(1):Nimg(end),k), L2, U2, C2] = filloutliers(mi_angle_l(Nimg(1):Nimg(end),k), 'linear', 'percentiles', [15, 85] );
     mi_l(1:Nimg(1)-1,k) = NaN;
    [Mach_filtered_l(Nimg(1):Nimg(end),k)] = filloutliers(Mach_l(Nimg(1):Nimg(end),k), 'linear', 'percentiles', [15, 85] );
     Mach_filtered_l(1:Nimg(1)-1,k) = NaN;
    end
    
end

%find indices where the outliers were replaced, replace that  with a NaN so that it doesn't affect statystics
    if isequal(BEST_WALL,'upper') || isempty(BEST_WALL)==1
       indx_result_u(Nimg(1):Nimg(end)) = ( mi_u(Nimg(1):Nimg(end),k) == mi_angle_u(Nimg(1):Nimg(end),k)); %find indices where the outliers were replaced, replace that  with NaNs
         for xx=Nimg
            if indx_result_u(xx)==0
                LinesLength_u(xx,k)= NaN;   
            end
         end
    end

    if isequal(BEST_WALL,'lower') || isempty(BEST_WALL)==1
        indx_result_l(Nimg(1):Nimg(end)) = ( mi_l(Nimg(1):Nimg(end),k) == mi_angle_l(Nimg(1):Nimg(end),k));
        for xx=Nimg
            if indx_result_l(xx)==0
                LinesLength_l(xx,k)= NaN;
            end
        end
    end


if isempty(BEST_WALL)==0 % if there IS a best wall, substitute mi angle of the other wall with NaNs.
    if isequal(BEST_WALL,'upper')
       mi_angle_l(Nimg(1):Nimg(end),1:walls_iterations)=NaN;
       mi_l(Nimg(1):Nimg(end),1:walls_iterations)=NaN;
    end
    if isequal(BEST_WALL,'lower')
       mi_angle_u(Nimg(1):Nimg(end),1:walls_iterations)=NaN;
       mi_u(Nimg(1):Nimg(end),1:walls_iterations)=NaN;
    end
end
    
  
% Check filtering
figure(100)
k_vec= [12]; %k locations where to plot the time-results(frames)
for a = k_vec
    plot(Nimg(1):1:Nimg(end), mi_angle_u(Nimg(1):1:Nimg(end),a), '--b')
    hold on
    plot(Nimg(1):1:Nimg(end), mi_u(Nimg(1):1:Nimg(end),a), 'b')
    plot(Nimg(1):1:Nimg(end), mi_angle_l(Nimg(1):1:Nimg(end),a), '--r')
    plot(Nimg(1):1:Nimg(end), mi_l(Nimg(1):1:Nimg(end),a), 'r')
end
grid on
xlabel(' Sample Number')
ylabel(' Mi angle')
legend('Data' ,'Filtered data')
title(' filtered results')
hold off
%saveas(gcf, 'filtered results', 'bmp')


%% =================== RESULTS 2:  statystical quantities calculation ============
 
for k = 1:1:walls_iterations

 if isequal(BEST_WALL,'upper') || isempty(BEST_WALL)==1
%Upper Wall    
    avg_mi_u(k)= nanmean(mi_u(Nimg(1):Nimg(end),k));
    S_mi_u(k) = nanstd(mi_u(Nimg(1):Nimg(end),k)); % std dev 68%Conf Lvl

    Avg_LinesLength_u(k) = nanmean(LinesLength_u(Nimg(1):Nimg(end),k)); %calculate new average without outliers
    Unc_LL_u(k) = 2*nanstd(LinesLength_u(Nimg(1):Nimg(end),k)); %95% uncertainty on the length of the lines
    %[degrees] uncertainty on the angular resolution with uniform distribution at 95% conf Lvl
    U_Angular_res_u(k) = (1/(1+Avg_LinesLength_u(k)^2))*Unc_LL_u(k)*180/pi;  
     
    Avg_angular_res_u(k) = atan( 1/Avg_LinesLength_u(k) )*180/pi; %[deg] average angular res based on the length of the identified lines, with uniform distribution   
    Angular_res_95_u(k) = (Avg_angular_res_u(k) + U_Angular_res_u(k)); %[deg]  95%conf lvl.  (worst resolution= largest angle of resolution) - LINEAR DISTRIBUTION at 100% conf Lvl --> division by 0.95 goes from 95 to 100 percent if u want
    U_extraction_u(k) = sqrt( (Angular_res_95_u(k))^2 + (unc_wall_angle_u(k))^2 + (0.95*hough_theta_step/2)^2 ); % 95%Conf LVL total uncertainty. NB hough theta step is 100%conf Lvl becaus is the containment limits. 
      
    Mach_avg_u(k) = nanmean(Mach_filtered_u(Nimg(1):Nimg(end),k));              
    S_Mach_u(k) = nanstd(Mach_filtered_u(Nimg(1):Nimg(end),k)); %std deviation of the Mach number for each station (68%ConfLvl)

    U_Mach_ext_u(k) = (pi/180)*U_extraction_u(k)*( Mach_avg_u(k)*sqrt( Mach_avg_u(k)^2 -1 ) ) ; %Propagation of the angular extraction errors to the Mach result 
 end   

 
 
 if isequal(BEST_WALL,'lower') || isempty(BEST_WALL)==1
%Lower Wall      
    avg_mi_l(k) = nanmean(mi_l(Nimg(1):Nimg(end) ,k));
    S_mi_l(k) = nanstd(mi_l(:,k)); % std dev 68%Conf Lvl

    Avg_LinesLength_l(k) = nanmean(LinesLength_l(Nimg(1):Nimg(end),k)); 
    Unc_LL_l(k) = 2*nanstd(LinesLength_l(Nimg(1):Nimg(end),k)); %95% uncertainty on the length of the lines
    %[degrees] uncertainty on the angular resolution with uniform distribution at 95% conf Lvl
    U_Angular_res_l(k) = (1/(1+Avg_LinesLength_l(k)^2))*Unc_LL_l(k)*180/pi;  
     
    Avg_angular_res_l(k) = atan( 1/Avg_LinesLength_l(k) )*180/pi; %[deg] average angular res based on the length of the identified lines, with uniform distribution   
    Angular_res_95_l(k) = (Avg_angular_res_l(k) + U_Angular_res_l(k)); %[deg]  95%conf lvl.  (worst resolution= largest angle of resolution) - LINEAR DISTRIBUTION at 100% conf Lvl --> division by 0.95 goes from 95 to 100 percent if u want
    U_extraction_l(k) = sqrt( (Angular_res_95_l(k))^2 + (2*unc_wall_angle_l(k))^2 + (0.95*hough_theta_step/2)^2 ); % 95%Conf LVL total uncertainty. NB hough theta step is 100%conf Lvl becaus is the containment limits. 
 
    Mach_avg_l(k) = nanmean(Mach_filtered_l(Nimg(1):Nimg(end),k));              
    S_Mach_l(k) = nanstd(Mach_filtered_l(Nimg(1):Nimg(end),k)); %std deviation of the Mach number for each station (68%ConfLvl)

    U_Mach_ext_l(k) = (pi/180)*U_extraction_l(k)*( Mach_avg_l(k)*sqrt( Mach_avg_l(k)^2 -1 ) ) ; %Propagation of the angular extraction errors to the Mach results

 end
    
 
end

% check correlations ------------
if isequal(BEST_WALL,'upper') || isempty(BEST_WALL)==1
%upper wall
[R , P] = corrcoef(S_mi_u, Angular_res_95_u, 'rows', 'complete');
    if P(1,2)<=0.28
       RHOu1 =  R(1,2);
    else
        RHOu1=0;
    end   
[R , P] = corrcoef(S_Mach_u, U_Mach_ext_u, 'rows','complete')
    if P(1,2)<=0.28
       RHOu2 =  R(1,2);
    else
        RHOu2=0;
    end
end

if isequal(BEST_WALL,'lower') || isempty(BEST_WALL)==1
%lower wall    
[R , P] = corrcoef(S_mi_l, Angular_res_95_l, 'rows', 'complete');
    if P(1,2)<=0.28
       RHOl1 =  R(1,2);
    else
        RHOl1=0;
    end   
[R , P] = corrcoef(S_Mach_l, U_Mach_ext_l, 'rows','complete')
    if P(1,2)<=0.28
       RHOl2 =  R(1,2);
    else
        RHOl2=0;
    end   
end    
    
% Final total expanded uncertanties
for k = 1:1:walls_iterations
   
    if isequal(BEST_WALL,'upper') || isempty(BEST_WALL)==1 
     U_mi_u(k) = sqrt( (2*S_mi_u(k))^2 + (U_extraction_u(k))^2 + 2*S_mi_u(k)*U_extraction_u(k)*RHOu1 ); % 95%Conf LVL 
     U_Mach_u(k) =  sqrt( (2*S_Mach_u(k))^2 + (U_Mach_ext_u(k)^2) + 2*S_Mach_u(k)*U_Mach_ext_u(k)*RHOu2 );  %  95%conf lvl
    end
    
    if isequal(BEST_WALL,'lower') || isempty(BEST_WALL)==1
     U_mi_l(k) = sqrt( (2*S_mi_l(k))^2 + (U_extraction_l(k))^2 + 2*S_mi_l(k)*U_extraction_l(k)*RHOl1 ); % 95%Conf LVL 
     U_Mach_l(k) =  sqrt( (2*S_Mach_l(k))^2 + (U_Mach_ext_l(k)^2) + 2*S_Mach_l(k)*U_Mach_ext_l(k)*RHOl2 );  %  95%conf lvl
    end 
     
     %TOTAL extended std uncertainty with 95%Conf Lvl, considering also exp Uncertainty coming from mi angle uncertainty
   
    
end


%% Save results 
tic

if simulation_save_results == 1 

    fprintf('Saving variables... \n')
    save 'WALLS_variablesPR26.mat'
    fprintf('DONE: Saving variables... \n')

end


%% ---------------- Load VARIABLES ----------- Just to plot previously achieved results

if LOAD_PREVIOUS_RESULTS==1
tic

fprintf(' Loading variables... \n')

load 'WALLS_variablesPR26.mat'

fprintf('DONE: Loading variables... \n')

toc
end

%% ---------------- RESULTS 3:  Domain calibration and uncertainty calculation

WALLS_EQUAL = 1;%RECOMMENDED (Do not change unless strictly necessary!): 
%this means that the step and window size used for the discretization is the same for both walls.


if WALLS_EQUAL ==1
    
    if isequal(BEST_WALL,'upper') || isempty(BEST_WALL)==1 %UPPER WALL
    W_locations = center_pixels1(1:1:walls_iterations,1); %positions  in pixels from start to finishing, of the center pixels of the windows
    end
    if  isequal(BEST_WALL,'lower') || isempty(BEST_WALL)==1 %LOWER WALL
    W_locations = center_pixels2(1:1:walls_iterations,1); %positions  in pixels from start to finishing, of the center pixels of the windows
    end
    
    adim_locations = (M_throatposition-W_locations)/throat_pixels; %adimensionalizing the locations with the throat and moving the reference zero to the throat
    
    U_adim = U_adim_frac*adim_locations;
    U_Xadim_tot = sqrt( (U_adim).^2+ (base_window1/(2*throat_pixels)).^2  ); 
    %invert location for plotting purposes:
     inv_adim_locations = adim_locations(end:-1:1); % from throat to exit direction
    
    
    inverted_locations= abs(W_locations(end:-1:1)-M_throatposition); %inverting the adim locations for plotting purposes
       
    % physical plane ---------------- [mm]   
    x_domain_mm =inverted_locations./scale_factor; %domain in mm
    U_x_NRS_mm = U_x_NRS_mm_frac*x_domain_mm;
    U_x_mm_tot = sqrt( (U_x_NRS_mm).^2+ ( floor(base_window1/2)/scale_factor).^2  ); %total uncertainty in the dimensional mm domain NRS.

    
end
if WALLS_EQUAL==0 % if the wall discretization is not equal, modify the program, choose the best wall and input the new step and center pixels.
    fprintf('input two vectors: W_locations1 and W_locations2\n')
end

%% REMOVING NONSENSE RESULTS IN A WINDOW
% window_remove = [15, 17 , 18, 19];
% Mach_avg_u(window_remove)=NaN;
% U_Mach_u(window_remove)=NaN;
% avg_mi_u(window_remove)=NaN;
% U_mi_u(window_remove)=NaN;



%% ==== PLOTTING ==== :  plotting results with total uncertainty bars

%NB Before plotting check that the plots only include CALCULATED VARIABLES!
% if a certain wall was selected, the other wall must be commented out.

if isempty(BEST_WALL)==1 %plot both wall results
figure(3)
plot(adim_locations(end:-1:1) , avg_mi_u(end:-1:1) , '*')
hold on
plot(adim_locations(end:-1:1) , avg_mi_l(end:-1:1) , 'o')
hold on
for k=walls_iterations:-1:1
    plot([adim_locations(k) adim_locations(k)] , [avg_mi_u(k)+U_mi_u(k) avg_mi_u(k)-U_mi_u(k)], 'b')
    hold on
    plot([adim_locations(k) adim_locations(k)] , [avg_mi_l(k)+U_mi_l(k) avg_mi_l(k)-U_mi_l(k)], 'r')
end
legend('upper', 'lower')
ylabel('\mu angle on walls')
xlabel('adimensionalized axis of the nozzle with the throat height x/H_t')
grid on
%savefig('mi_adimloc_walls.fig')


figure(4)
plot(adim_locations(end:-1:1) , Mach_avg_u(end:-1:1) , '*')
hold on
plot(adim_locations(end:-1:1) , Mach_avg_l(end:-1:1) , 'o')
hold on
for k=walls_iterations:-1:1
    plot([adim_locations(k) adim_locations(k)] , [Mach_avg_u(k)+U_Mach_u(k) Mach_avg_u(k)-U_Mach_u(k)], 'b')
    hold on
    plot([adim_locations(k) adim_locations(k)] , [Mach_avg_l(k)+U_Mach_l(k) Mach_avg_l(k)-U_Mach_l(k)], 'r')
    plot([adim_locations(k)-U_Xadim_tot(k) adim_locations(k)+U_Xadim_tot(k)] , [Mach_avg_u(k) Mach_avg_u(k)], 'c')
    plot([adim_locations(k)-U_Xadim_tot(k) adim_locations(k)+U_Xadim_tot(k)] , [Mach_avg_l(k) Mach_avg_l(k)], 'c')
end
legend('upper', 'lower')
ylabel('Mach number on walls')
xlabel('adimensionalized axis of the nozzle with the throat height x/H_t')
grid on
%savefig('mach_adimloc_walls.fig')
end

if isequal(BEST_WALL,'upper') || isempty(BEST_WALL)==1 %UPPER WALL
    
figure(5) 
hold on
plot(adim_locations(end:-1:1) , Mach_avg_u(end:-1:1) , 'o')
hold on
for k=walls_iterations:-1:1
    errorbar(adim_locations(k)  ,Mach_avg_u(k),U_Mach_u(k),'r-','HandleVisibility','off');
    hold on
    errorbar(adim_locations(k),Mach_avg_u(k),U_Xadim_tot(k),'b-','horizontal','HandleVisibility','off');
end
errorbar(adim_locations(k)  ,Mach_avg_u(k),U_Mach_u(k),'r-');
hold on
errorbar(adim_locations(k),Mach_avg_u(k),U_Xadim_tot(k),'b-','horizontal');
grid on
xlabel('x/H_t','Interpreter','latex')
ylabel('M distribution','Interpreter','latex')
legend(' average Mach', 'total U_M ', '$ total position uncertaity U_x')
set(gca,'FontSize',16)
% print -depsc Mach_lower
%savefig('Mach_u_adimloc_walls_nicer.fig')

%U_mach (total expanded uncertainty on the mach) Upper wall
figure(55)
plot(adim_locations(end:-1:1) , U_Mach_ext_u(end:-1:1) , '--r')
hold on
plot(adim_locations(end:-1:1) , 2*S_Mach_u(end:-1:1) , '--b')
plot(adim_locations(end:-1:1) , U_Mach_u(end:-1:1) , 'k')
grid on
xlabel('x/H_t')
ylabel('U_M Breakdown')
legend('angle extraction uncertainty ', 'random uncertainty','total uncertainty')
% title('Expanded uncertainties the Mach Number')
set(gca,'FontSize',14)
%print -depsc Uncertainties_mach_lowerwall
hold off
hold off
%saveas(gcf, 'Uncertainties_mach_l_breakdown', 'bmp')
%savefig('U_Mach_u_breakdown_walls.fig')

end

if isequal(BEST_WALL,'lower') || isempty(BEST_WALL)==1 %LOWER WALL
    
figure(6) 
hold on
plot(adim_locations(end:-1:1) , Mach_avg_l(end:-1:1) , 'o')
hold on
for k=walls_iterations:-1:1
    errorbar(adim_locations(k)  ,Mach_avg_l(k),U_Mach_l(k),'r-','HandleVisibility','off');
    hold on
    errorbar(adim_locations(k),Mach_avg_l(k),U_Xadim_tot(k),'b-','horizontal','HandleVisibility','off');
end
errorbar(adim_locations(k)  ,Mach_avg_l(k),U_Mach_l(k),'r-');
hold on
errorbar(adim_locations(k),Mach_avg_l(k),U_Xadim_tot(k),'b-','horizontal');
grid on
xlabel('x/H_t','Interpreter','latex')
ylabel('M distribution','Interpreter','latex')
legend(' average Mach', 'total U_M ', '$ total position uncertaity U_x')
set(gca,'FontSize',16)
% print -depsc Mach_lower
%savefig('Mach_l_adimloc_walls_nicer.fig')

%U_mach (total expanded uncertainty on the mach) Upper wall
figure(66)
plot(adim_locations(end:-1:1) , U_Mach_ext_l(end:-1:1) , '--r')
hold on
plot(adim_locations(end:-1:1) , 2*S_Mach_l(end:-1:1) , '--b')
plot(adim_locations(end:-1:1) , U_Mach_l(end:-1:1) , 'k')
grid on
xlabel('x/H_t')
ylabel('U_M Breakdown ')
legend('angle extraction uncertainty ', 'random uncertainty','total uncertainty')
title('Lower Wall')
set(gca,'FontSize',14)
%print -depsc Uncertainties_mach_lowerwall
hold off
hold off
%saveas(gcf, 'Uncertainties_mach_l_breakdown', 'bmp')
%savefig('U_Mach_l_breakdown_walls.fig')

end



%% checking results on graphs: global check on a frame
% choose the frame

frame_check=Nimg(1); 


if isequal(BEST_WALL,'upper') || isempty(BEST_WALL)==1 
   center_pixels_w(:,1) = center_pixels1(:,1);
else
   center_pixels_w(:,1) = center_pixels2(:,1); 
end

figure(7)
imshow( edge_BW(:,:,frame_check) )
hold on
for k=1:1:length(center_pixels_w(:,1))
   
    plot( Walls_x_pixeldomain, floor(Swall_l), 'linewidth', 2)
    plot( Walls_x_pixeldomain, floor(Swall_u), 'linewidth', 2)
    hold on
    
  if isequal(BEST_WALL,'upper') || isempty(BEST_WALL)==1 
    plot( center_pixels1(k,1), center_pixels1(k,2), 'o') %plotting centers of int windows, which for now are constant for various frames
    if isempty(LINES1{frame_check,k})==0
    xy_u = [LINES1{frame_check,k}.point1; LINES1{frame_check,k}.point2];
    xy_u(:,1) = xy_u(:,1) + center_pixels1(k,1) - floor(base_window1/2) ; %the x must be in the global SR in pixels for both points
    xy_u(:,2) = xy_u(:,2) + center_pixels1(k,2) - floor(h_window1/2) ; %the y must be in the global SR in pixels for both points
    %NB: xy is a temporary variable just for the plot of the lines.
    plot(xy_u(:,1),xy_u(:,2),'LineWidth',2,'Color','green');
    end
  end
  
  if isequal(BEST_WALL,'lower') || isempty(BEST_WALL)==1 
    plot( center_pixels2(k,1), center_pixels2(k,2), 'o')
    if isempty(LINES2{frame_check,k})==0
    xy_l = [LINES2{frame_check,k}.point1; LINES2{frame_check,k}.point2];
    xy_l(:,1) = xy_l(:,1) + center_pixels2(k,1) - floor(base_window2/2) ; %the x must be in the global SR in pixels for both points
    xy_l(:,2) = xy_l(:,2) + center_pixels2(k,2) - floor(h_window2/2) ; %the y must be in the global SR in pixels for both points
    %NB: xy is a temporary variable just for the plot of the lines.
    plot(xy_l(:,1),xy_l(:,2),'LineWidth',2,'Color','green');
    end
  end
  
end
%savefig('superimposed_lines_walls.fig')



%% ===---===---===--- TROUBLESHOOT AND LOCAL CHECKS: comment all when the simulation is succesfull. ===---===---===
%% window by window Plot: local check on a frame
% UNCOMMENT FOR CHECKING PURPOSES
%
% x= 2;
% for k = 1:1:i-1
%    figure(k+10)
%    imshow(WINDOWS1{x,k})
%    hold on
%    xy = [LINES1{x,k}.point1; LINES1{x,k}.point2];
%    plot(xy(:,1),xy(:,2),'LineWidth',2,'Color','green');
%    hold off
% end

%% checking data in a location 
% 
%  k= 20; %location
% frame_start=10;
% frame_end=100;
% stepp=5;
% % 
% for x = frame_start:stepp:frame_end
%    figure(x) 
%    imshow(WINDOWS1{x,k})
%    hold on
%    xy = [LINES1{x,k}.point1; LINES1{x,k}.point2];
%    plot(xy(:,1),xy(:,2),'LineWidth',2,'Color','green');
%    hold off
% end


%% 0000000000=================  SUPERIMPOSED LINES ON A SCHLIEREN IMAGE ==============000000000000000
% PLOTTING MIDPLANE LINES AND WALL LINES ON A SCHLIEREN FRAME.
load 'midplane_avg_lines.mat'

figure(1000)
imshow( I(:,:,Nimg(1)) )
hold on
% plot( x_pixeldomain, floor(upper_wall), 'linewidth', 2, 'Color','white')
% plot( x_pixeldomain, floor(lower_wall), 'linewidth', 2, 'Color','white')
plot( Walls_x_pixeldomain, floor(Swall_u)-5, 'linewidth', 1, 'Color','yellow')
plot( Walls_x_pixeldomain, floor(Swall_l)-5, 'linewidth', 1, 'Color','yellow')
plot( [1 m],  [Y_0 Y_0], 'r --' )
for k=window_start:1:window_finish %midplane plot
    
plot( center_pixels(k,1,1), center_pixels(k,2,1), 'r o') %plotting centers of int windows, which for now are constant for various frames
plot( [center_pixels(k,1,1)-delta_x(k) center_pixels(k,1,1)+delta_x(k)], [center_pixels(k,2,1)+delta_y(k) center_pixels(k,2,1)-delta_y(k)],'c', 'linewidth', 2 ) % plot ([x1 x2] [y1 y2)]
hold on

end
hold on
for k=1:1:length(center_pixels1(:,1))

  if isequal(BEST_WALL,'upper') || isempty(BEST_WALL)==1 
    delta_x_u(k) = 0.5*Avg_LinesLength_u(k)*cos( (avg_mi_u(k) - abs(wall_angle_u(k)))*pi/180 );
    delta_y_u(k) = 0.5*Avg_LinesLength_u(k)*sin( (avg_mi_u(k)- abs(wall_angle_u(k)))*pi/180 );
    
plot( center_pixels1(k,1), center_pixels1(k,2), 'r o') %plotting centers of int windows, which for now are constant for various frames
plot( [center_pixels1(k,1)-delta_x_u(k) center_pixels1(k,1)+delta_x_u(k)], [center_pixels1(k,2)+delta_y_u(k) center_pixels1(k,2)-delta_y_u(k)],'c', 'linewidth', 2 ) % plot ([x1 x2] [y1 y2)]
  end
  
  if isequal(BEST_WALL,'lower') || isempty(BEST_WALL)==1 
    delta_x_l(k) = 0.5*Avg_LinesLength_l(k)*cos( (avg_mi_l(k)- abs(wall_angle_l(k)))*pi/180 );
    delta_y_l(k) = 0.5*Avg_LinesLength_l(k)*sin( (avg_mi_l(k)- abs(wall_angle_l(k)))*pi/180 );
    
plot( center_pixels2(k,1), center_pixels2(k,2), 'r o') %plotting centers of int windows, which for now are constant for various frames
plot( [center_pixels2(k,1)-delta_x_l(k) center_pixels2(k,1)+delta_x_l(k)], [center_pixels2(k,2)+delta_y_l(k) center_pixels2(k,2)-delta_y_l(k)],'c', 'linewidth', 2 ) % plot ([x1 x2] [y1 y2)]
  end    
    
end


%% Automated plots

load 'CFD_PR26_IGPoly.mat'
X(1).X = M_wall(:,1);
Y(1).Y = M_wall(:,2);
clear M_wall


load 'CFD_PR26_PRPoly.mat'
X(2).X = M_wall(:,1);
Y(2).Y = M_wall(:,2);
clear M_wall


load 'CFD_PR26_PRnonPoly.mat'
X(3).X = M_wall(:,1);
Y(3).Y = M_wall(:,2);


if isequal(BEST_WALL,'upper') 
   Mach_avg =  Mach_avg_u;
   U_Mach = U_Mach_u;
end

if isequal(BEST_WALL,'lower')
   Mach_avg =  Mach_avg_l;
   U_Mach = U_Mach_l;  
end


%%%%%%%%%%%%%%%%%%%%%% Plot the midline Mach %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% SIMULATION and EXPERIMENT %%%%%%%%%%%%%%%%%%%%%%%

plot_lines_setup;

fig_name = 'fig1';

%Axis and legend 
xxlabel = '$x/H_\textrm{th}$ [-]';
yylabel = '$M_{\textrm{mid}}$ [-]';
% legendd = {'PRSV' 'Experiment' 'Nozzle Throat'};
% legendd = {'PR Nonpolytropic' 'PR Polytropic' 'IG Polytropic' 'IG Nonpolytropic' 'Experiment'};
legendd = {'IG Polytropic' 'PR Polytropic' 'PR Nonpolytropic'  'Experiment'};
location = 'southeast';

% Line type and markers type
Plot_op.Linestyle = {'-' '--' '-.' 'none' 'none'};
Plot_op.Linewidth = [0.5 0.5 0.5 0.5 0.5 0.5];
Plot_op.Marker    = {[]  [] []    'o'    'o' };
Plot_op.MarkerFaceColor = {'black' 'black' 'black' 'black' 'black' 'black' 'black' 'black' };
Plot_op.MarkerEdgeColor = {'black' 'black' 'black' 'black' 'black' 'black' 'black' 'black' };
Plot_op.Color    = {'c' 'g' 'b' 'r' 'k'};


% X(1).X = Mid_diverging.x_coord_ref./y_throat;  %The x axis for the simulation
% X(1).X = Mid_diverging.x_coord_ref*10*100;
% adim_x_domain_half = adim_x_domain/2;           %The nozzle throat half height is half the adim_x_domain
% adim_x_domain_half = x_domain_mm./(throat_mm/2);
% X(3).X = adim_x_domain_half;                       %The x axis for the experiement  
X(4).X = inv_adim_locations; 

% X(2).X =x_domain_mm;
% X(1).X_throat

% Y(1).Y = Mid_diverging.Mach_Number;            %The y data for the simulation
Mach_avg(Mach_avg==0) = NaN;
Y(4).Y = flip(Mach_avg);                       %The y data for the experiment  



%Statistics
%Mean
U_Mach(U_Mach==0) = NaN;
E(4).E_std_Mach =  flip(U_Mach);
E(4).E_std_Discretization =  flip(U_Xadim_tot);
% E(2).E_std = X; %Put here the uncertainty of the location. Probably this comes from the estimation of the throat location AND? 
%Std. Deviation


x_lim = [round(min(X(1).X),2)   round(max(X(1).X), 2)];
x_lim = [0 7];
y_lim = [round(min(Y(1).Y),2)   round(max(Y(1).Y),2)];
y_lim = [1.0   2.4];

plot_X_vs_Yerrorbar2(X         , Y      ,  E  , ...
    x_lim     , y_lim      , ...
    xxlabel   , yylabel    , ...
    legendd   , location   , ...
    x_lab_fac , y_lab_fac  , ...
    fig_name  , print_name , ...
    width     , Plot_op    , ...
    fs        , fs_fac, ...
    font      , latex      , ...
    textt);
grid on
 print -depsc Wall_assessment_PR26
% savefig('Mach_mid_PR025NT002.fig')

endf=2;

