%% FLOW SYMMETRY ANALYSIS: DETECTION OF ANGLES ABOVE AND BELOW MIDPLANE  ============================================================

clear all; clc; close all

set(groot, 'defaultAxesTickLabelInterpreter','latex'); 
set(groot, 'defaultLegendInterpreter','latex');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ============================= INPUT 1: Reading images and binarization ================================ 
%read all the images and binarize them
%Gets the total number of images in the directory
% a = dir(['PR025NT002_Bobcat_Zoom_Run1_SS_Fabio_process_Align' '/*.bmp']);
a = dir(['PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection' '/*.tif']);
total = size(a,1);

% FolderIn = 'PR025NT002_Bobcat_Zoom_Run1_SS_Fabio_process_Align'; %folder where the images are located
FolderIn='PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection';   %folder where the images are located

 RootIn = 'B0';          %root name of the images 
% RootIn = 'Align0000';                         

% calibration_img = imread('calibration_PR025NT002_Bobcat_Zoom_Run1_SS_Fabio_process_Align.bmp'); %read the image used to calculate the scale factor and throat position
calibration_img = imread('PR026NT001_Zoom_bobcat_ShiftCorrection_NoCut_ShiftCorrection/B00001.tif'); %read the image used to calculate the scale factor and throat position

MPI = 1;          %1 means the calculation is done in series
                  %2 means the calculation is done in parallel
                  %3 means the calculation is done both in series and
                  %parallel (for comparison)

%dimensions of the calibr img are the dimension i want for the Schlieren imgs
m = length(calibration_img(1,:)); %columns
n = length(calibration_img(:,1)); %rows

%Positive rotation is counterclockwise.
%rotate the calibration to align with the schlieren. 
image_rotate_cal = -1.5;                    %[degrees]
%Rotate the schlieren data to align with the calibration. Set 0 if
%calibration and data have a different rotation.
image_rotate_data = - 1.5;
%Note. The operation of rotating the dataset to align with the calibration
%image can be done in davis also. At this point in time the datasets fed
%into this tool are only shifted to align with the calibration image.


%%%%%%%%%%%%%%%%%%%%% DATA SEQUENCE Allocation %%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nimg = 2:1:100; %vector that represents all the images to acquire
% Nimg = 1:1:total; %vector that represents all the images to acquire


%%%%%%%%%%%%%%%%% Hough Transform Binarisation Parameters %%%%%%%%%%%%%%%%%

threshold = [0.01, 0.1]; % edge binarization threshold of the functionn [low,high]: increase low to avoid detecting small gradients; decrease high to detect smaller gradients. Almost never 
   %touch the low. Cannot be 0. Change the high only. If increase gonna have less lines. SHocks for example will be 0.1 The filter is then
   %used to filter out he wobly lines. 
gauss_noise_filter = 2.1;   %reduce filter gonna be more noice.  
%The above two parameters control the binarization. 

% gaussian sigma filter: Increase to reduce noise. 
%The sharper the image, the more lines and chaos you have the more the
%filter needs to be increased. Filter 1.5 is suggested when only one
%type of line is visible in the window and the rest is bluried out so
%not binarized by the Edge algorithm. Filter 3 is recommended when the
%image is sharp and chaotic

%******************* Apply the filters above to the raw data
%Modify the acquisition cycle depending on the type of image you have

%%%%%%%%%%%%%%%%% Adjustment Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

brightness_data = 40;           %Increase or decrease brightness
% threshold_imgad = [0.05  0.88]; %[low, high]: adjust brightness: increase higher threshold to get white out of light grey area
%threshold_imgad = [0.2 0.99];
threshold_imgad = '';

%Change the acquisition cycle depending on what type of images you have

%----------------------- Serial implementation --------------------------%
if MPI == 1 || MPI == 3
    tic
    for x = Nimg
        fprintf('Acquiring image %s %d\n',RootIn,x)
        StringIn = sprintf('%s/%s%04d.tif',FolderIn,RootIn,x);
        
        Raw_img = imread(StringIn);
        Raw_img = imrotate(Raw_img, image_rotate_data); %rotate data img of x degrees IF it is not parallel to ground
        
        
%         grayscale = rgb2gray(RGB(1:n,1:m,:)); %if the image is rgb, convert to greyscale (for bmps)
         
          grayscale = Raw_img + brightness_data;
          
        if isnan(threshold_imgad)
            I(:,:,x) = imadjust(grayscale);
        else
            I(:,:,x) = imadjust(grayscale, threshold_imgad); % [low, high]: adjust brightness: increase lower threshold to get white out of light grey area
        end

        edge_BW(:,:,x) = edge( I(:,:,x), 'Canny', threshold, gauss_noise_filter );
    
    end
    toc
end



%----------------parallel implementation---------------------%
if MPI == 2 || MPI == 3
    tic
    parfor t = 1:size(Nimg,2)
        
%        [edge_BW_2, I_2, grayscale_2] = edge_BWparbmp(t, brightness_data, threshold, gauss_noise_filter,FolderIn,RootIn ,threshold_imgad,image_rotate_data,n,m);
         [edge_BW_2, I_2, grayscale_2] = edge_BWpar(t, brightness_data, threshold, gauss_noise_filter,FolderIn,RootIn ,threshold_imgad,image_rotate_data);
        % disp(V);
        % figure(3)
        % imshow(V(:,:,t))
        % edge_BW_2(:,:,t) = V(1);
%         edge_BW_3(:,:,t) = edge_BW_2(:,:,t);
%         I_3(:,:,t) = I_2(:,:,t);
        edge_BW(:,:,t) = edge_BW_2(:,:,t);
        I(:,:,t) = I_2(:,:,t);
        grayscale(:,:,t) = grayscale_2;
    end
%     isequal(edge_BW(:,:,2),edge_BW_3(:,:,2));
%     result_comp_edge_BW = edge_BW == edge_BW_3;
%     index = find(result_comp_edge_BW(:,:,1) == 0);
%     result_comp_I = I == I_3;
    toc
end

%To check the results:
figure(1)
imshowpair(edge_BW(:,:,Nimg(1)),I(:,:,Nimg(1)), 'montage')

%% =========== INPUT 2: walls data ===========================

walldomain_start = 0.02; %in percentage of the image: where the walla cquisition should start
walldomain_end = 0.8;  %in percentage of the image: where the walla cquisition should end
%the user needs to be careful selcting these two parameters. It defines how
%the polynomial is construted. There is less uncertainty in the polynomial if there is a good
%detection of the wall

adim_walldomain = [walldomain_start walldomain_end] ; % the interval of interest to get get the curves of the walls 
threshold_schlieren = 0.19; % lower threshold to get less details in B&W image. You can use the schlieren BW image (Schlieren_1frame_BW) from figure 1
%to see if there is a good contrast between white and black. nozzle inner
%should be white and the outside should be black. 

Schlieren_1frame_BW = imbinarize(I(:,:,Nimg(1)), threshold_schlieren); % binarization of the image with Imbinarize. 
%This function allows to tune the schlieren data. This allows to identify
%the wall in the actual experiment. Then the calibration image is compared
%to the Schlieren 


offset = 0; %increment offset to move up, decrement to move down in the image. 
%Important if the nozzle is not the center of the FOV. 
Y_0 = n/2 - offset; 

%Schlieren_1frame_BW
figure(2)
imshowpair(I(:,:,Nimg(1)),Schlieren_1frame_BW, 'montage' )
hold on
plot([1, m],[Y_0, Y_0])
plot([adim_walldomain(1)*m , adim_walldomain(1)*m], [1 n-1])
plot([adim_walldomain(2)*m , adim_walldomain(2)*m], [1 n-1])
hold off

m_start= round(adim_walldomain(1)*m);
m_end= round(adim_walldomain(2)*m);

Walls_x_pixeldomain= m_start:1:m_end;

% the polynomial order can be selected freely based on the 
% shape of the wall to approximate. 
poly_order_walls = 5;

%% ====================== WALL ACQUISITION: Acquiring the walls of the schlieren image: ========================= 
%This code block depends on the Schlieren threshold that you set. The
%curves found depend on this threshold. 
% Upper curve
i = 1;
column = m_start;
while column < m_end
    flag=0;
    row=1;
    while row<Y_0 && flag == 0
        if Schlieren_1frame_BW(row,column) == 1
            x_fit_u(i) = column;
            y_fit_u(i) = row;
            i=i+1;
            flag=1;
        else
            row=row+1;
        end
    end
  column=column+1;
end  
% Lower curve
i=1;
column = m_start;
while column < m_end
    flag=0;
    row=n;
    while row>Y_0 && flag==0
        if Schlieren_1frame_BW(row,column)==1
            x_fit_l(i)=column;
            y_fit_l(i)=row;
            i=i+1;
            flag=1;
        else
            row=row-1;
        end
    end
  column=column+1;
end
%now  the walls of the schlieren image in the selected range
%Walls_x_pixeldomain are known

[SchlierenW1, Ssw1]= polyfit(x_fit_u, y_fit_u, poly_order_walls);
[SchlierenW2, Ssw2]= polyfit(x_fit_l, y_fit_l, poly_order_walls);
[Swall_u , Sdelta_u] = polyval(SchlierenW1, Walls_x_pixeldomain, Ssw1); %delta is the standard dev: 68% conf lvl
[Swall_l , Sdelta_l] = polyval(SchlierenW2, Walls_x_pixeldomain, Ssw2);

%recalculating Y_0 now for the flow symmetry check close to the exit: It should be extremely similar to
%the original Y_0 and it is
check_Y_0 =(Swall_l(1:end)+ Swall_u(1:end))/2;
check_Y_0_rotation =180/pi*atan(((Swall_l(end)+ Swall_u(end))/2 - (Swall_l(1)+ Swall_u(1))/2 )/(Walls_x_pixeldomain(end)-Walls_x_pixeldomain(1)) ); %this is used to understand if the image is rotated.
%This will show you, based on the estimation of the wall, the midplane. You
%can use it to understand how much to rotate your image. This is under the
%assumption that you have a good estimation of the walls.

fprintf('overall rotation of the image[in deg]:Positive=counter clockwise rotation. %f \n',check_Y_0_rotation)


figure(21)
imshowpair(  edge_BW(:,:,Nimg(1)),I(:,:,Nimg(1)), 'montage');
hold on
plot([1, m],[Y_0, Y_0])
plot([1, m],[check_Y_0(1), check_Y_0(end)])
plot( Walls_x_pixeldomain, floor(Swall_u), 'y', 'linewidth', 2)
plot( Walls_x_pixeldomain, floor(Swall_l), 'y', 'linewidth', 2)
legend('estimated Y_0' , 'Calculated Y_0')


%Remember:
% 1== upper wall above y_0; 2 lower wall below y_0. remeber that the
% starting  point of counting is top left.-> System of reference is top
% left for pixels. X is right and y is down. This is different to the
% original system of reference.



%% INPUT 3:  CALCULATION PARAMETERS: --------------------------------------
%This might have to be 0 but it is up to the user to check where he is
%evaluating the flow symmetry. 
%This can be changed if you want to change where to evaluate symmetry.
%Obviosly look at the image, if you move too far left there might not be
%walls.

USE_WALLS = 1; % if this flag is==1 acquired walls are used as a stopping method
             % if this flag is ==0 then the method uses the end of the image as a stoppiing method


Switch_angles=0; 
% flag=0 -> Keep the angles like they are: downcount detects "/" angles, upcount "\" angles.
% flag=1 -> Switch the angles for the mach line detection  
%This will switch the orientation of the angles top and bottom. It could be that downwards facing slops on the top are easier to see. 
  

C_symmcheck = 0.04; % its the percentage of the image, divided by 100. This is the constant
%which dictates where on the image the flow symmetry is checked. This has
%to be larger than walldomain_start constant if you want to use walls

Y_shift = 15 ; %pixels: the program will start acquiring lines from the midplane+ or - Y_shift (to further yourself from midplane)

figure(21)
plot([C_symmcheck*m, C_symmcheck*m],[1, n], '--', 'linewidth', 2)
hold off

H_windowcheck =60 ; % pixel height of the flow check windows. This is the size of the windows 
%up and below from the midplane. If it is large there will be less lines
%and if it is lower there will be more uncertainty. y dimension

hough_theta_step = 0.2; %degrees: resolution of hough transform

halfBase_windowcheck = 25 ; %pixel Half-Base of the flow check windows. The x dimension of the window.

X_symmetrycheck = floor(C_symmcheck*m) ; %in Pixels: first axial coordinate for flow symm check
if X_symmetrycheck<halfBase_windowcheck
    disp('ERROR: increase C_symmcheck ')
end


if USE_WALLS==1
 pixelskipUP = 3;   %allowed pixels to skip to still detect a line.
    %Change this value higher UP, this is the bottom line. increase to allow a
    %higher threshold to detect a line.
    C_UP = 0.48;  %this is the percentage line length (minimum). 
    
     %DOWN means downcount: it counts down reducing, so it moves towards the top of the img
    pixelskipDOWN = 2; %allowed pixels to skip to still detect a line, , this is the upper line. the amount of pixels to skip in order to recognise a line.
    C_DOWN = 0.48;  %This is the percentage line length (minimum).
    %lines will be longer or equal to this. in percentage of the height of the window.
end    


if USE_WALLS == 0
    disp('USE_WALLS=0: you are not using an automated stopping method \n')
    lowerlimit =300; %in pixels
    upperlimit = n-300; %in pixels
    %if you don't use the walls you still need to specify the the above two parameters in order to stop the calculation.
 pixelskipUP = 3;
    C_UP = 0.38;
    
    pixelskipDOWN=3;
    C_DOWN=0.38;

end



%% ========================= CALCULATION:  Flow symmetry assessment ====================
[minCheck, Index]= min(abs(Walls_x_pixeldomain-X_symmetrycheck)); %index is the position of the x_check(in pixels) in the wall acquisition system of reference. Mincheck must be zero.

height_ratio= H_windowcheck/(Swall_l(Index)-Swall_u(Index)); 
    if height_ratio>=0.2
       fprintf('the height ratio is too high: decrease window height.\n') 
    end

for x = Nimg

%Checking the flow symmetry with a cycle on Ycoordinate
%NB: check flow symm where the expansion is (almost) completed : try to avoid 2D effects



if USE_WALLS==1 %===========================================
    if C_symmcheck<walldomain_start
        fprintf('C_symmcheck must be greater or equal to walldomain_start!')
    end
    
    if Switch_angles==0    
        theta_vectorUP = -75:hough_theta_step:-15 ;
        theta_vectorDOWN = 15:hough_theta_step:75 ;
    else
        theta_vectorUP = 15:hough_theta_step:75 ;
        theta_vectorDOWN = -75:hough_theta_step:-15 ;
    end    
    %UP means upcount: so it goes up counting in the matrix which is down in the image. at moment it is 25 to 75 degrees
    %This is the polar angle. This range because you want to detect positive
    %angles. This detect the lines above and the block bllow is the other
    %kind. mu = (-75 to -25) + 90, means +25 to 65 degrees mu. If modified you must keep the symmetry.
   
    Ymidplane = check_Y_0(Index);
   
    
    upcount = floor(Ymidplane)+Y_shift; %pixels
    downcount = floor(Ymidplane)-Y_shift; %pixels
    i = 1; %counts above and below midplane windows to check if they are equal or not
    
    while upcount <= (Swall_l(Index)-1.5*ceil(H_windowcheck/2)) || downcount >= (Swall_u(Index)+1.5*ceil(H_windowcheck/2))
        
        int_windowUP = edge_BW( upcount:upcount+H_windowcheck , X_symmetrycheck-halfBase_windowcheck:1:X_symmetrycheck+halfBase_windowcheck , x );
        center_pixelsUP(i,:)= [X_symmetrycheck, upcount+H_windowcheck/2];
        
        int_windowDOWN = edge_BW( downcount-H_windowcheck:downcount , X_symmetrycheck-halfBase_windowcheck:1:X_symmetrycheck+halfBase_windowcheck , x);
        center_pixelsDOWN(i,:)= [X_symmetrycheck, downcount-H_windowcheck/2];
        
        
        %extract lines on the UPcount  H is the accumlation matrix.
        [H,THETA, RHO] = hough( int_windowUP, 'theta', theta_vectorUP );
        min_length_linesUP= C_UP*length(int_windowUP(:,1)) ;
        %minimum length of the line he can find. Don't touch unless it doesn't
        %find a line. If you don't find lines then decrease this value.
        
        PEAKSUP = houghpeaks(H,2);
        %H 2 depends on the image. If you have a very dense lines not paralle
        %which are not parallel to each other then increase to 3.
        LINESUP{x,i}= houghlines( int_windowUP, THETA, RHO, PEAKSUP,'FillGap',pixelskipUP,'MinLength',min_length_linesUP);
        WINDOWSUP{x,i}=int_windowUP;
        
        %extract lines on the DOWNcount
        [H,THETA, RHO] = hough( int_windowDOWN, 'theta', theta_vectorDOWN );
        min_length_linesDOWN= C_DOWN*length(int_windowDOWN(:,1)) ;
        PEAKSDOWN = houghpeaks(H,2);
        LINESDOWN{x,i}= houghlines( int_windowDOWN, THETA, RHO, PEAKSDOWN,'FillGap',pixelskipDOWN,'MinLength',min_length_linesDOWN);
        WINDOWSDOWN{x,i}=int_windowDOWN;
        
        % extract angles and save them only if lines are found
        if isempty(LINESUP{x,i})==0 && isempty(LINESDOWN{x,i})==0 %both lines found:   save useful variables only if 2 lines are found and can compare them.
            thetaUP = extractfield(LINESUP{x,i}, 'theta');
            thetaDOWN = extractfield(LINESDOWN{x,i}, 'theta');
            xy_UP = [LINESUP{x,i}.point1 ;  LINESUP{x,i}.point2 ];
            xy_DOWN = [LINESDOWN{x,i}.point1 ;  LINESDOWN{x,i}.point2 ];
            mm=1*2;
            LinesLengthUP(x,i)= sqrt( (xy_UP(2,mm-1)-xy_UP(1,mm-1))^2 + (xy_UP(2,mm)-xy_UP(1,mm))^2 ) ;  % xy(point, coordinate)-> x_2-x_1= xy(point2,coordinate1)-xy(point1,coordinate1);
            LinesLengthDOWN(x,i)= sqrt( (xy_DOWN(2,mm-1)-xy_DOWN(1,mm-1))^2 + (xy_DOWN(2,mm)-xy_DOWN(1,mm))^2 ) ;  % xy(point, coordinate)-> x_2-x_1= xy(point2,coordinate1)-xy(point1,coordinate1);
            
            if thetaUP(1)>0
                mi_angle_UP(x,i)= 90-thetaUP(1);
                Mach_UP(x,i) = 1/sin(mi_angle_UP(x,i)*pi/180);
            else
                mi_angle_UP(x,i)= thetaUP(1)+90;
                Mach_UP(x,i) = 1/sin(mi_angle_UP(x,i)*pi/180);
            end
            if thetaDOWN(1)>0
                mi_angle_DOWN(x,i)= 90-thetaDOWN(1);
                Mach_DOWN(x,i) = 1/sin(mi_angle_DOWN(x,i)*pi/180);
            else
                mi_angle_DOWN(x,i)= thetaDOWN(1)+90;
                Mach_DOWN(x,i) = 1/sin(mi_angle_DOWN(x,i)*pi/180);
            end
        end %end if line found
        
        if isempty(LINESUP{x,i})==1 || isempty(LINESDOWN{x,i})==1 && x>1
            fprintf('warning: lines not found at iteration %d of the frame %d \n', i,x)
            %replacing data of the pair with NaN
            mi_angle_UP(x,i) = NaN;
            mi_angle_DOWN(x,i) = NaN;
            Mach_UP(x,i) = NaN;
            Mach_DOWN(x,i) = NaN;
            LinesLengthUP(x,i)= NaN;
            LinesLengthDOWN(x,i)=NaN;
            NaNs(x,i)=1;
        end %end lines not found
        
        %If lines aren't found it is a problem of the binarization. go above to
        %the first sections of the code and change the parameters. Move the line
        %where you evaluate or change the binarisation. If he doesn't find any
        %lines in the window height specified then he gives errors. Change
        %window height and /or position and binarisation.
        %Note: The frame is the image and iteration is the space/window number
        %from the centerline.
        
        upcount=upcount+floor(H_windowcheck/2);
        downcount=downcount-floor(H_windowcheck/2);
        i = i+1;
    end %end the cycle on y for flow symmetry check
end


%--------------------------------------------------------------------------------------------
%This is the same as above but there are some differences if you select a different flag, e.g., no walls  
%if you decide not to use walls as a stopping method (not recommended) in case it is necessary

if USE_WALLS == 0
    
    if Switch_angles == 0    
        theta_vectorUP = -75:hough_theta_step:-15 ;
        theta_vectorDOWN = 15:hough_theta_step:75;
    else
        theta_vectorUP = 15:hough_theta_step:75;
        theta_vectorDOWN = -75:hough_theta_step:-15 ;
    end  
    
    Ymidplane = mean(check_Y_0);
    upcount = floor(Ymidplane)+Y_shift;
    downcount = floor(Ymidplane)-Y_shift;
    
    i = 1; %counts above and below midplane windows to check if they are equal or not
    
    while upcount <= (upperlimit-H_windowcheck) || downcount >= (lowerlimit+H_windowcheck)
        
        int_windowUP = edge_BW( upcount:upcount+H_windowcheck , X_symmetrycheck-halfBase_windowcheck:1:X_symmetrycheck+halfBase_windowcheck , x );
        center_pixelsUP(i,:)= [X_symmetrycheck, upcount+H_windowcheck/2];
        
        int_windowDOWN = edge_BW( downcount-H_windowcheck:downcount , X_symmetrycheck-halfBase_windowcheck:1:X_symmetrycheck+halfBase_windowcheck , x);
        center_pixelsDOWN(i,:)= [X_symmetrycheck, downcount-H_windowcheck/2];
        
        %extract lines on the UPcount
        [H,THETA, RHO] = hough( int_windowUP, 'theta', theta_vectorUP );
        min_length_linesUP= C_UP*length(int_windowUP(:,1)) ;
        PEAKSUP = houghpeaks(H,2);
        LINESUP{x,i}= houghlines( int_windowUP, THETA, RHO, PEAKSUP,'FillGap',pixelskipUP,'MinLength',min_length_linesUP);
        WINDOWSUP{x,i}=int_windowUP;
        
        %extract lines on the DOWNcount
        [H,THETA, RHO] = hough( int_windowDOWN, 'theta', theta_vectorDOWN );
        min_length_linesDOWN= C_DOWN*length(int_windowDOWN(:,1)) ;
        PEAKSDOWN = houghpeaks(H,2);
        LINESDOWN{x,i}= houghlines( int_windowDOWN, THETA, RHO, PEAKSDOWN,'FillGap',pixelskipDOWN,'MinLength',min_length_linesDOWN);
        WINDOWSDOWN{x,i}=int_windowDOWN;
        
        % extract angles and save them
        if isempty(LINESUP{x,i})==0 && isempty(LINESDOWN{x,i})==0 %both lines found
            thetaUP = extractfield(LINESUP{x,i}, 'theta');
            thetaDOWN = extractfield(LINESDOWN{x,i}, 'theta');
            xy_UP = [LINESUP{x,i}.point1 ;  LINESUP{x,i}.point2 ];
            xy_DOWN = [LINESDOWN{x,i}.point1 ;  LINESDOWN{x,i}.point2 ];
            mm=1*2;
            LinesLengthUP(x,i)= sqrt( (xy_UP(2,mm-1)-xy_UP(1,mm-1))^2 + (xy_UP(2,mm)-xy_UP(1,mm))^2 ) ;  % xy(point, coordinate)-> x_2-x_1= xy(point2,coordinate1)-xy(point1,coordinate1);
            LinesLengthDOWN(x,i)= sqrt( (xy_DOWN(2,mm-1)-xy_DOWN(1,mm-1))^2 + (xy_DOWN(2,mm)-xy_DOWN(1,mm))^2 ) ;  % xy(point, coordinate)-> x_2-x_1= xy(point2,coordinate1)-xy(point1,coordinate1);
            
            if thetaUP(1)>0
                mi_angle_UP(x,i)= 90-thetaUP(1);
                Mach_UP(x,i) = 1/sin(mi_angle_UP(x,i)*pi/180);
            else
                mi_angle_UP(x,i)= thetaUP(1)+90;
                Mach_UP(x,i) = 1/sin(mi_angle_UP(x,i)*pi/180);
            end
            if thetaDOWN(1)>0
                mi_angle_DOWN(x,i)= 90-thetaDOWN(1);
                Mach_DOWN(x,i) = 1/sin(mi_angle_DOWN(x,i)*pi/180);
            else
                mi_angle_DOWN(x,i)= thetaDOWN(1)+90;
                Mach_DOWN(x,i) = 1/sin(mi_angle_DOWN(x,i)*pi/180);
            end
        end %end if line found
        
        if isempty(LINESUP{x,i})==1 || isempty(LINESDOWN{x,i})==1
            fprintf('ERROR: lines not found at iteration %d of the frame %d \n', i,x)
            %replacing pair lines data with NaN to not consider them
            mi_angle_UP(x,i) = NaN;
            mi_angle_DOWN(x,i) = NaN;
            LinesLengthUP(x,i)= NaN;
            LinesLengthDOWN(x,i)=NaN;
            Mach_UP(x,i) = NaN;
            Mach_DOWN(x,i) = NaN;
            NaNs(x,i)=1;
        end %end lines not found
        
        if isempty(LINESUP{x,i})==1 || isempty(LINESDOWN{x,i})==1 && x==1
            fprintf('serious ERROR: first frame lines not found. Restart procedure \n')
            mi_angle_UP(x,i) = NaN;
            mi_angle_DOWN(x,i) = NaN;
            Mach_UP(x,i) = NaN;
            Mach_DOWN(x,i) = NaN;
            LinesLengthUP(x,i)= NaN;
            LinesLengthDOWN(x,i)=NaN;
        end %end lines not found fist frame
        
        upcount=upcount+floor(H_windowcheck/2);
        downcount=downcount-floor(H_windowcheck/2);
        i=i+1;
    end %end the cycle on y for flow symmetry check
end


end%end the cycle on all the frames
iterations = i-1;


%% Saving only the comparable results and calculation of statystical quantities

%filtering results of angle extractions
%This is used to make the dataset clean for sensible results 

for k=1:1:iterations
    
   [mi_UP(Nimg(1):Nimg(end),k), TF_UP(:,k), L_UP, UUP, C_UP] = filloutliers(mi_angle_UP(Nimg(1):Nimg(end),k), 'linear', 'percentiles', [20, 80] );
   [mi_DOWN(Nimg(1):Nimg(end),k), TF_DOWN(:,k), L_DOWN, U_DOWN, C_DOWN] = filloutliers(mi_angle_DOWN(Nimg(1):Nimg(end),k), 'linear', 'percentiles', [20, 80] ); 
    mi_UP(1:Nimg(1)-1,k) = NaN;
    mi_DOWN(1:Nimg(1)-1,k) = NaN;
    
   M_UP(Nimg(1):Nimg(end),k)=filloutliers(Mach_UP(Nimg(1):Nimg(end),k), 'linear', 'percentiles', [20, 80] );
   M_DOWN(Nimg(1):Nimg(end),k)=filloutliers(Mach_DOWN(Nimg(1):Nimg(end),k), 'linear', 'percentiles', [20, 80] ); 
   M_UP(1:Nimg(1)-1,k) = NaN;
   M_DOWN(1:Nimg(1)-1,k) = NaN;
end


%From the local results it finds the averages through time after you
%filtered. It is a statistical assessment block.


for k=1:1:iterations %for each location, calculate the average mi angles in time on only the good frames
    
    % evaluate Uncertainty due to angular resolution
    indx_result_UP(Nimg(1):Nimg(end)) = ( mi_angle_UP(Nimg(1):Nimg(end),k)==mi_UP(Nimg(1):Nimg(end),k)); %find indices where the outliers were replaced, replace that  with Nans
    indx_result_DOWN(Nimg(1):Nimg(end)) = ( mi_angle_DOWN(Nimg(1):Nimg(end),k)==mi_DOWN(Nimg(1):Nimg(end),k)); %find indices where the outliers were replaced, replace that  with an NaNs
    
    for xx=Nimg
        if indx_result_DOWN(xx)==0
            LinesLengthDOWN(xx,k)= NaN;
        end
        if indx_result_UP(xx)==0
            LinesLengthUP(xx,k)= NaN;
        end
    end
    
    %statystics on the Length of the lines detected
    Avg_LinesLength_DOWN(k) = nanmean(LinesLengthDOWN(Nimg(1):Nimg(end),k)); %calculate new average without outliers
    Unc_LL_DOWN(k) = 2*nanstd(LinesLengthDOWN(Nimg(1):Nimg(end),k)); %95% uncertainty on the length of the lines
    
    Avg_LinesLength_UP(k) = nanmean(LinesLengthUP(Nimg(1):Nimg(end),k)); %calculate new average without outliers
    Unc_LL_UP(k) = 2*nanstd(LinesLengthUP(Nimg(1):Nimg(end),k)); %95% uncertainty on the length of the lines
    
    %[degrees] uncertainty on the angular resolution with uniform distribution at 95% conf Lvl
    U_Angular_res_DOWN(k) = (1/(1+Avg_LinesLength_DOWN(k)^2))*Unc_LL_DOWN(k)*180/pi;
    Avg_angular_res_DOWN(k) = atan( 1/Avg_LinesLength_DOWN(k) )*180/pi; %[deg] average angular res based on the length of the identified lines, with uniform distribution
    Angular_res_95_DOWN(k) = (Avg_angular_res_DOWN(k) + U_Angular_res_DOWN(k)); %[deg]  95%conf lvl.  (worst resolution= largest angle of resolution) - LINEAR DISTRIBUTION at 100% conf Lvl --> division by 0.95 goes from 95 to 100 percent if u want
        AR95_DOWN(k) = 0.95*atan( 1/min(LinesLengthDOWN(Nimg(1):Nimg(end),k)))*180/pi; 

    
    U_Angular_res_UP(k) = (1/(1+Avg_LinesLength_UP(k)^2))*Unc_LL_UP(k)*180/pi;
    Avg_angular_res_UP(k) = atan( 1/Avg_LinesLength_UP(k) )*180/pi; %[deg] average angular res based on the length of the identified lines, with uniform distribution
    Angular_res_95_UP(k) = (Avg_angular_res_UP(k) + U_Angular_res_UP(k)); %[deg]  95%conf lvl.  (worst resolution= largest angle of resolution) - LINEAR DISTRIBUTION at 100% conf Lvl --> division by 0.95 goes from 95 to 100 percent if u want
        AR95_UP(k) = 0.95*atan( 1/min(LinesLengthUP(Nimg(1):Nimg(end),k)))*180/pi; 

    
    avg_mi_DOWN(k)=nanmean(mi_DOWN(:,k)); %average Mach angle on the downcount
    u_std_DOWN(k) = nanstd(mi_DOWN(:,k));
    
    avg_mi_UP(k) = nanmean(mi_UP(:,k)); %average Mach angle on the upcount
    u_std_UP(k) = nanstd(mi_UP(:,k));
    
    
    U_extraction_DOWN(k) = sqrt( (AR95_DOWN(k))^2 + (0.95*hough_theta_step/2)^2 );
    U_extraction_UP(k) = sqrt( (AR95_UP(k))^2 + (0.95*hough_theta_step/2)^2 );
    
    %Mach uncertainties
    %upcount
    Mach_avg_UP(k) = nanmean(M_UP(:,k));
    S_Mach_UP(k) = nanstd(M_UP(:,k)); %std deviation of the Mach number for each station (68%ConfLvl)
    
    U_Mach_ext_UP(k) = (pi/180)*U_extraction_UP(k)*( Mach_avg_UP(k)*sqrt( Mach_avg_UP(k)^2 -1 ) ) ; %Propagation of the angular extraction errors to the Mach results
    
    %downcount
    Mach_avg_DOWN(k) = nanmean(M_DOWN(:,k));
    S_Mach_DOWN(k) = nanstd(M_DOWN(:,k)); %std deviation of the Mach number for each station (68%ConfLvl)
    
    U_Mach_ext_DOWN(k) = (pi/180)*U_extraction_DOWN(k)*( Mach_avg_DOWN(k)*sqrt( Mach_avg_DOWN(k)^2 -1 ) ) ; %Propagation of the angular extraction errors to the Mach results
    
    
end



% check correlations ------------
%upcount
[R , P] = corrcoef(u_std_UP, Angular_res_95_UP, 'rows', 'complete');
    if P(1,2)<=0.28
       RHOu1 =  R(1,2);
    else
        RHOu1=0;
    end   
[R , P] = corrcoef(S_Mach_UP, U_Mach_ext_UP, 'rows','complete')
    if P(1,2)<=0.28
       RHOu2 =  R(1,2);
    else
        RHOu2=0;
    end
 
%downcount    
[R , P] = corrcoef(u_std_DOWN, Angular_res_95_DOWN, 'rows', 'complete');
    if P(1,2)<=0.28
       RHOd1 =  R(1,2);
    else
        RHOd1=0;
    end   
[R , P] = corrcoef(S_Mach_DOWN, U_Mach_ext_DOWN, 'rows','complete')
    if P(1,2)<=0.28
       RHOd2 =  R(1,2);
    else
        RHOd2=0;
    end   
    

    for k=1:1:iterations
        
        %total expanded uncertainties
        
        U_mi_DOWN(k) = sqrt( U_extraction_DOWN(k)^2  + (2*u_std_DOWN(k)^2) + U_extraction_DOWN(k)*(2*u_std_DOWN(k))*RHOd1);  %total expanded unc 95%conf lvl
        U_Mach_DOWN(k) =  sqrt( (2*S_Mach_DOWN(k))^2 + (U_Mach_ext_DOWN(k)^2) + 2*S_Mach_DOWN(k)*U_Mach_ext_DOWN(k)*RHOd2 );  %  95%conf lvl
        
        U_mi_UP(k) =  sqrt( U_extraction_UP(k)^2  + (2*u_std_UP(k)^2) +U_extraction_UP(k)*(2*u_std_UP(k))*RHOu1 ); %total expanded unc 95%conf lvl
        U_Mach_UP(k) =  sqrt( (2*S_Mach_UP(k))^2 + (U_Mach_ext_UP(k)^2) + 2*S_Mach_UP(k)*U_Mach_ext_UP(k)*RHOu2 );  %  95%conf lvl
        
        %TOTAL extended std uncertainty with 95%Conf Lvl, considering also exp Uncertainty coming from mi angle uncertainty
        
    end

U_deltaX = floor(H_windowcheck/2) ;

locations = 1:1:iterations;



% % Remove a location where results do not make sense:
% location_to_remove = 14;
% locations(location_to_remove)=NaN;
% avg_mi_UP(location_to_remove)=NaN;
% avg_mi_DOWN(location_to_remove)=NaN;
% Mach_avg_UP(location_to_remove)=NaN;
% Mach_avg_DOWN(location_to_remove)=NaN;


%% SAVING RESULTS:

SAVE_RESULTS=0;
tic
if SAVE_RESULTS==1
    fprintf('Saving variables... \n')
    save 'variablesFLOWSYMM.mat'
    fprintf('DONE: Saving variables... \n')
end
toc

%% LOADING PREVIOUS RESULTS

LOAD_PREVIOUS_RESULTS=1;

if LOAD_PREVIOUS_RESULTS==1
tic
fprintf(' Loading variables... \n')

load 'variablesFLOWSYMM.mat'

fprintf('DONE: Loading variables... \n')
toc
end


%% PLOTTING==========================================

%% Display average calculated lines on the first Schlieren image 

figure(1111)
imshow( I(:,:,Nimg(1)) )
hold on
for k = 1 :1: iterations
 
    %Upcount
delta_xUP(k) = 0.5*Avg_LinesLength_UP(k)*cos(avg_mi_UP(k)*pi/180);
delta_yUP(k) = 0.5*Avg_LinesLength_UP(k)*sin(avg_mi_UP(k)*pi/180);
  
   plot( center_pixelsUP(k,1), center_pixelsUP(k,2), 'o') %plotting centers of int windows, which for now are constant for various frames
hold on
  plot( [center_pixelsUP(k,1)-delta_xUP(k) center_pixelsUP(k,1)+delta_xUP(k)], [center_pixelsUP(k,2)-delta_yUP(k) center_pixelsUP(k,2)+delta_yUP(k)],'g', 'linewidth', 2 ) % plot ([x1 x2] [y1 y2)]
hold on

    %Downcount
delta_xDOWN(k) = 0.5*Avg_LinesLength_DOWN(k)*cos(avg_mi_DOWN(k)*pi/180);
delta_yDOWN(k) = 0.5*Avg_LinesLength_DOWN(k)*sin(avg_mi_DOWN(k)*pi/180);
  
   plot( center_pixelsDOWN(k,1), center_pixelsDOWN(k,2,1), 'o') %plotting centers of int windows, which for now are constant for various frames
hold on
  plot( [center_pixelsDOWN(k,1)-delta_xDOWN(k) center_pixelsDOWN(k,1)+delta_xDOWN(k)], [center_pixelsDOWN(k,2)+delta_yDOWN(k) center_pixelsDOWN(k,2)-delta_yDOWN(k)],'g', 'linewidth', 2 ) % plot ([x1 x2] [y1 y2)]
hold on

end
hold off
%saveas(gcf, 'superimposed_lines_FLOWSYMMETRY', 'bmp')


figure(2)
plot(locations, avg_mi_UP,'r-')
hold on
plot(locations, avg_mi_DOWN,'b-')
hold on
for a=1:1:iterations
    errorbar(a  ,avg_mi_UP(a),U_mi_UP(a),'r-');
    hold on
    errorbar(a,avg_mi_DOWN(a),U_mi_DOWN(a),'b-');
end
grid on
xlabel('Window Number','Interpreter','latex')
ylabel('\mu')
legend('$\mu_{avg}$ below midplane', '$\mu_{avg}$ above midplane')
set(gca,'FontSize',20)
%print -depsc symmetry_windows   %print saves in eps format.



figure(3)
plot(center_pixelsUP(:,2), avg_mi_UP,'r-')
hold on
plot(center_pixelsDOWN(:,2), avg_mi_DOWN,'b-')
hold on
for a=1:1:iterations
    errorbar(center_pixelsUP(a,2) ,avg_mi_UP(a),U_mi_UP(a),'r-');
    hold on
    errorbar(center_pixelsDOWN(a,2) ,avg_mi_DOWN(a),U_mi_DOWN(a),'b-');
end
grid on
xlabel('Pixels','Interpreter','latex')
ylabel('\mu')
legend('$\mu_{avg}$ below midplane', '$\mu_{avg}$ above midplane')
set(gca,'FontSize',20)
% saveas(gcf,'flowsymm_errorbars.png')



%% Plotting the detected lines on a schlieren at x=.. frame.
%Preallocation
% xy_DOWN = zeros(Nimg(end),intervals);
    
if USE_WALLS==1 
    x=Nimg(1); %frame
    
    figure(111)
    imshow( edge_BW(:,:,Nimg(1)) )
    hold on
    for k=1:1:iterations
        plot( Walls_x_pixeldomain, floor(Swall_l), 'linewidth', 2)
        plot( Walls_x_pixeldomain, floor(Swall_u), 'linewidth', 2)
        plot( [Walls_x_pixeldomain(Index) Walls_x_pixeldomain(Index)], [1 n], '--')
        plot( center_pixelsUP(k,1), center_pixelsUP(k,2), 'o') %plotting centers of int windows, which for now constant for various frames
        plot( center_pixelsDOWN(k,1), center_pixelsDOWN(k,2), 'o') %plotting centers of int windows, which for now constant for various frames
        
        hold on
        
        if isempty(LINESUP{x,k})==0 && isempty(LINESDOWN{x,k})==0 %plot lines only if they are found!
        xy_UP = [LINESUP{x,k}.point1; LINESUP{x,k}.point2];
        xy_UP(:,1) = xy_UP(:,1) + center_pixelsUP(k,1) - halfBase_windowcheck ; %the x must be in the global SR in pixels for both points
        xy_UP(:,2) = xy_UP(:,2) + center_pixelsUP(k,2) - floor(H_windowcheck/2) ; %the y must be in the global SR in pixels for both points
        %NB: xy is a temporary variable just for the plot of the lines.
        plot(xy_UP(:,1),xy_UP(:,2),'LineWidth',2,'Color','green');
        
        xy_DOWN = [LINESDOWN{x,k}.point1; LINESDOWN{x,k}.point2];
        xy_DOWN(:,1) = xy_DOWN(:,1) + center_pixelsDOWN(k,1) - halfBase_windowcheck ; %the x must be in the global SR in pixels for both points
        xy_DOWN(:,2) = xy_DOWN(:,2) + center_pixelsDOWN(k,2) - floor(H_windowcheck/2) ; %the y must be in the global SR in pixels for both points
        %NB: xy is a temporary variable just for the plot of the lines.
        plot(xy_DOWN(:,1),xy_DOWN(:,2),'LineWidth',2,'Color','green');
        end
        if isempty(LINESUP{x,k})==1 || isempty(LINESDOWN{x,k})==1 %lines not found at that frame!
            fprintf('not all lines are found in this frame %d in this position %d \n', x, k)
        end
    end
    
end

if USE_WALLS==0
x=1; %choose the frame    
    
    figure(111)
    imshow( edge_BW(:,:,Nimg(1)) )
    hold on
    for k=1:1:iterations
        
        plot( [X_symmetrycheck X_symmetrycheck], [1 n], '--')
        hold on
        plot( center_pixelsUP(k,1), center_pixelsUP(k,2), 'o') %plotting centers of int windows, which for now constant for various frames
        plot( center_pixelsDOWN(k,1), center_pixelsDOWN(k,2), 'o') %plotting centers of int windows, which for now constant for various frames
        hold on
        
        if isempty(LINESUP{x,k})==0 && isempty(LINESDOWN{x,k})==0 %plot lines only if they are found!
        xy_UP = [LINESUP{x,k}.point1; LINESUP{x,k}.point2];
        xy_UP(:,1) = xy_UP(:,1) + center_pixelsUP(k,1) - halfBase_windowcheck ; %the x must be in the global SR in pixels for both points
        xy_UP(:,2) = xy_UP(:,2) + center_pixelsUP(k,2) - floor(H_windowcheck/2) ; %the y must be in the global SR in pixels for both points
        %NB: xy is a temporary variable just for the plot of the lines.
        plot(xy_UP(:,1),xy_UP(:,2),'LineWidth',2,'Color','green');
        
        xy_DOWN = [LINESDOWN{x,k}.point1; LINESDOWN{x,k}.point2];
        %CHECKFABIO Sometimes xy_DOWN doesn't contain a value. Check with a
        %large number of images
        xy_DOWN(:,1) = xy_DOWN(:,1) + center_pixelsDOWN(k,1) - halfBase_windowcheck ; %the x must be in the global SR in pixels for both points
        xy_DOWN(:,2) = xy_DOWN(:,2) + center_pixelsDOWN(k,2) - floor(H_windowcheck/2) ; %the y must be in the global SR in pixels for both points
        %NB: xy is a temporary variable just for the plot of the lines.
        plot(xy_DOWN(:,1),xy_DOWN(:,2),'LineWidth',2,'Color','green');
        end
        if isempty(LINESUP{x,k})==1 || isempty(LINESDOWN{x,k})==1 %lines not found at that frame!
            fprintf('not all lines are found in this frame %d , at this position %d. \n', x, k)
        end
        
    end
    
end

fprintf(' Avg Mach (above midplane): %f \n',nanmean(Mach_avg_DOWN))
fprintf(' Avg Mach (below midplane): %f \n',nanmean(Mach_avg_UP))

fprintf(' Average Mach lines angles above and below midplane: %f  , %f \n ',nanmean(avg_mi_DOWN),nanmean(avg_mi_UP) )

%% FOR LOCAL CHECKING PURPOSES : window by window Plot
%TROUBLESHOTing
%UP
% x= 2;
% for k = 1:1:i-1
%    figure(k+10) 
%    imshow(WINDOWSUP{x,k})
%    hold on
%    xy = [LINESUP{x,k}.point1; LINESUP{x,k}.point2];
%    plot(xy(:,1),xy(:,2),'LineWidth',2,'Color','green');
%    hold off
% end


%Down. This plots the actual window with the superimposed line without
%purging/filtering. 

% x= 8; %Time/frame coordinate of the image sequence 
% for k = 1:1:i-1   %K is the location /space coordinate.
%    figure(k+30) 
%    imshow(WINDOWSDOWN{x,k})
%    hold on
%    xy = [LINESDOWN{x,k}.point1; LINESDOWN{x,k}.point2];
%    plot(xy(:,1),xy(:,2),'LineWidth',2,'Color','green');
%    hold off
% end
%If there is an error here then you know that the code could not find lines
%in the windows you specified. Look at the binarisation. it will struggle
%to find lines in the imposed direction.

%% checking data in a location 
% 
% k= 10; %location
% frame_start=10;
% frame_end=100;
% stepp=10;
% 
% for x = frame_start:stepp:frame_end
%    figure(x) 
%    imshow(WINDOWSUP{x,k})
%    hold on
%    xy = [LINESUP{x,k}.point1; LINESUP{x,k}.point2];
%    plot(xy(:,1),xy(:,2),'LineWidth',2,'Color','green');
%    hold off
% end
